@@ -56,6 +56,9 @@
 
 /*
  * $Log: SAX2PrintHandlers.hpp,v $
+ * Revision 1.5  2002/11/05 21:46:20  tng
+ * Explicit code using namespace in application.
+ *
  * Revision 1.4  2002/02/01 22:40:44  peiyongz
  * sane_include
  *
@@ -75,6 +78,8 @@
 #include    <xercesc/sax2/DefaultHandler.hpp>
 #include    <xercesc/framework/XMLFormatter.hpp>
 
+XERCES_CPP_NAMESPACE_USE
+
 class SAX2PrintHandlers : public DefaultHandler, private XMLFormatTarget
 {
 public:
@@ -111,8 +116,8 @@ public:
     // -----------------------------------------------------------------------
     void endDocument();
 
-    void endElement( const XMLCh* const uri, 
-					 const XMLCh* const localname, 
+    void endElement( const XMLCh* const uri,
+					 const XMLCh* const localname,
 					 const XMLCh* const qname);
 
     void characters(const XMLCh* const chars, const unsigned int length);
