@@ -56,7 +56,7 @@
 #
 
 #
-# $Id: sanityTest.pl,v 1.15 2002/06/11 19:44:35 peiyongz Exp $
+# $Id: sanityTest.pl,v 1.17 2003/01/30 00:36:12 tng Exp $
 #
 
 
@@ -191,3 +191,9 @@ system ("ThreadTest -parser=sax -n -s -f -v -quiet -threads 10 -time 20 personal
 system ("ThreadTest -parser=dom -n -s -f -v -quiet -threads 10 -time 20 personal-schema.xml");
 
 chdir "..".$pathsep."..";
+
+chdir "tests".$pathsep."DOM".$pathsep."TypeInfo";
+system ("DOMTypeInfoTest");
+
+chdir "..".$pathsep."..";
+
