@@ -58,11 +58,13 @@
  */
 
 /*
- * $Id: DOMRange.hpp,v 1.6 2002/08/22 15:04:57 tng Exp $
+ * $Id: DOMRange.hpp,v 1.8 2003/03/07 19:59:08 tng Exp $
  */
 
 #include <xercesc/util/XercesDefs.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 class DOMNode;
 class DOMDocumentFragment;
 
@@ -76,10 +78,18 @@ protected:
     //  Hidden constructors
     // -----------------------------------------------------------------------
     /** @name Hidden constructors */
-    //@{
+    //@{    
     DOMRange() {};
-    DOMRange(const DOMRange &) {};
-    DOMRange & operator = (const DOMRange &) {return *this;};
+    //@}
+
+private:
+    // -----------------------------------------------------------------------
+    // Unimplemented constructors and operators
+    // -----------------------------------------------------------------------
+    /** @name Unimplemented constructors and operators */
+    //@{
+    DOMRange(const DOMRange &);
+    DOMRange & operator = (const DOMRange &);
     //@}
 
 public:
@@ -554,4 +564,6 @@ public:
 };
 
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
