@@ -59,7 +59,7 @@
 *  handler with the scanner. In these handler methods, appropriate DOM nodes
 *  are created and added to the DOM tree.
 *
-* $Id: DOMParser.cpp,v 1.8 2002/08/14 15:20:37 knoaman Exp $
+* $Id: DOMParser.cpp,v 1.18 2003/05/22 02:26:50 knoaman Exp $
 *
 */
 
@@ -68,14 +68,15 @@
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
+#include <xercesc/internal/XMLScannerResolver.hpp>
 #include <xercesc/sax/EntityResolver.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/sax/ErrorHandler.hpp>
 #include <xercesc/sax/SAXParseException.hpp>
 #include <xercesc/framework/XMLNotationDecl.hpp>
 #include <xercesc/util/IOException.hpp>
-#include <xercesc/internal/XMLScanner.hpp>
-#include <xercesc/validators/DTD/DTDValidator.hpp>
+#include <xercesc/framework/XMLValidator.hpp>
+#include <xercesc/validators/common/GrammarResolver.hpp>
 #include "DOMParser.hpp"
 #include "ElementImpl.hpp"
 #include "AttrImpl.hpp"
@@ -92,40 +93,78 @@
 #include <xercesc/validators/common/ContentSpecNode.hpp>
 #include <xercesc/validators/DTD/DTDAttDefList.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 // ---------------------------------------------------------------------------
 //  DOMParser: Constructors and Destructor
 // ---------------------------------------------------------------------------
-DOMParser::DOMParser(XMLValidator* const valToAdopt) :
+DOMParser::DOMParser( XMLValidator* const  valToAdopt
+                    , MemoryManager* const manager) :
 
-fErrorHandler(0)
-, fEntityResolver(0)
-, fCreateEntityReferenceNodes(true)
-, fToCreateXMLDeclTypeNode(false)
-, fIncludeIgnorableWhitespace(true)
-, fNodeStack(0)
-, fScanner(0)
+    fToCreateXMLDeclTypeNode(false)
+    , fCreateEntityReferenceNodes(true)
+    , fIncludeIgnorableWhitespace(true)
+    , fParseInProgress(false)
+    , fWithinElement(false)
+    , fEntityResolver(0)
+    , fErrorHandler(0)
+    , fNodeStack(0)
+    , fScanner(0)
+    , fDocumentType(0)
+    , fGrammarResolver(0)
+    , fURIStringPool(0)
+    , fValidator(valToAdopt)
+    , fMemoryManager(manager)
 {
-    //
-    //  Create a scanner and tell it what validator to use. Then set us
-    //  as the document event handler so we can fill the DOM document.
-    //
-    fScanner = new XMLScanner(valToAdopt);
-    fScanner->setDocHandler(this);
-    fScanner->setDocTypeHandler(this);
-
-    fNodeStack = new ValueStackOf<DOM_Node>(64);
-    this->reset();
-
-
+    try
+    {
+        initialize();
+    }
+    catch(...)
+    {
+        cleanUp();
+        throw;
+    }
 }
 
 
 DOMParser::~DOMParser()
 {
-    delete fNodeStack;
-    delete fScanner;
+    cleanUp();
 }
 
+// ---------------------------------------------------------------------------
+//  DOMParser: Initialize/CleanUp methods
+// ---------------------------------------------------------------------------
+void DOMParser::initialize()
+{
+    // Create grammar resolver and URI string pool to pass to the scanner
+    fGrammarResolver = new (fMemoryManager) GrammarResolver(fMemoryManager);
+    fURIStringPool = new (fMemoryManager) XMLStringPool(109, fMemoryManager);
+
+    //  Create a scanner and tell it what validator to use. Then set us
+    //  as the document event handler so we can fill the DOM document.
+    fScanner = XMLScannerResolver::getDefaultScanner(fValidator, fMemoryManager);
+    fScanner->setDocHandler(this);
+    fScanner->setDocTypeHandler(this);
+    fScanner->setGrammarResolver(fGrammarResolver);
+    fScanner->setURIStringPool(fURIStringPool);
+
+    fNodeStack = new (fMemoryManager) ValueStackOf<DOM_Node>(64, fMemoryManager);
+    this->reset();
+}
+
+void DOMParser::cleanUp()
+{
+    delete fNodeStack;
+    delete fScanner;
+    delete fGrammarResolver;
+    delete fURIStringPool;
+
+    if (fValidator)
+        delete fValidator;
+}
 
 void DOMParser::reset()
 {
@@ -134,7 +173,7 @@ void DOMParser::reset()
     //  will cause the old one to go away unless application code is also
     //  holding a reference to it.
     //
-    fDocument = DOM_Document::createDocument();
+    fDocument = DOM_Document::createDocument(fMemoryManager);
     resetDocType();
 
     fCurrentParent   = 0;
@@ -218,7 +257,7 @@ bool DOMParser::isUsingCachedGrammarInParse() const
 
 Grammar* DOMParser::getGrammar(const XMLCh* const nameSpaceKey)
 {
-    return fScanner->getGrammar(nameSpaceKey);
+    return fGrammarResolver->getGrammar(nameSpaceKey);
 }
 
 Grammar* DOMParser::getRootGrammar()
@@ -226,11 +265,25 @@ Grammar* DOMParser::getRootGrammar()
     return fScanner->getRootGrammar();
 }
 
-const XMLCh* DOMParser::getURIText(unsigned int uriId)
+const XMLCh* DOMParser::getURIText(unsigned int uriId) const
 {
     return fScanner->getURIText(uriId);
 }
 
+bool DOMParser::getCalculateSrcOfs() const
+{
+    return fScanner->getCalculateSrcOfs();
+}
+
+bool DOMParser::getStandardUriConformant() const
+{
+    return fScanner->getStandardUriConformant();
+}
+
+unsigned int DOMParser::getSrcOffset() const
+{
+    return fScanner->getSrcOffset();
+}
 
 // ---------------------------------------------------------------------------
 //  DOMParser: Setter methods
@@ -326,6 +379,34 @@ void DOMParser::useCachedGrammarInParse(const bool newState)
         fScanner->useCachedGrammarInParse(newState);
 }
 
+void DOMParser::setCalculateSrcOfs(const bool newState)
+{
+    fScanner->setCalculateSrcOfs(newState);
+}
+
+void DOMParser::setStandardUriConformant(const bool newState)
+{
+    fScanner->setStandardUriConformant(newState);
+}
+
+void DOMParser::useScanner(const XMLCh* const scannerName)
+{
+    XMLScanner* tempScanner = XMLScannerResolver::resolveScanner
+    (
+        scannerName
+        , fValidator
+        , fMemoryManager
+    );
+
+    if (tempScanner) {
+
+        tempScanner->setParseSettings(fScanner);
+        tempScanner->setGrammarResolver(fGrammarResolver);
+        tempScanner->setURIStringPool(fURIStringPool);
+        delete fScanner;
+        fScanner = tempScanner;
+    }
+}
 
 // ---------------------------------------------------------------------------
 //  DOMParser: Parsing methods
@@ -663,7 +744,7 @@ void DOMParser::startElement(const  XMLElementDecl&         elemDecl
     DocumentImpl    *docImpl = (DocumentImpl *)fDocument.fImpl;
 
     if (fScanner -> getDoNamespaces()) {    //DOM Level 2, doNamespaces on
-        XMLBuffer buf;
+        XMLBuffer buf(1023, fMemoryManager);
         DOMString namespaceURI = 0;
         DOMString elemQName = 0;
         if (urlId != fScanner->getEmptyNamespaceId()) {  //TagName has a prefix
@@ -701,7 +782,7 @@ void DOMParser::startElement(const  XMLElementDecl&         elemDecl
             if (oneAttrib->getType()==XMLAttDef::ID)
             {
                 if (docImpl->fNodeIDMap == 0)
-                    docImpl->fNodeIDMap = new NodeIDMap(500);
+                    docImpl->fNodeIDMap = new (fMemoryManager) NodeIDMap(500, fMemoryManager);
                 docImpl->fNodeIDMap->add(attr);
                 attr->isIdAttr(true);
             }
@@ -723,7 +804,7 @@ void DOMParser::startElement(const  XMLElementDecl&         elemDecl
             if (oneAttrib->getType()==XMLAttDef::ID)
             {
                 if (docImpl->fNodeIDMap == 0)
-                    docImpl->fNodeIDMap = new NodeIDMap(500);
+                    docImpl->fNodeIDMap = new (fMemoryManager) NodeIDMap(500, fMemoryManager);
                 docImpl->fNodeIDMap->add(attr);
                 attr->isIdAttr(true);
             }
@@ -1065,7 +1146,7 @@ void DOMParser::endAttList
                     DOMString qualifiedName = attr->getFullName();
                     int index = DocumentImpl::indexofQualifiedName(qualifiedName);
 
-                    XMLBuffer buf;
+                    XMLBuffer buf(1023, fMemoryManager);
                     static const XMLCh XMLNS[] = {
                         chLatin_x, chLatin_m, chLatin_l, chLatin_n, chLatin_s, chNull};
 
@@ -1085,7 +1166,7 @@ void DOMParser::endAttList
                             buf.append(XMLUni::fgXMLNSURIName);
                     }
 
-                    insertAttr = new AttrNSImpl((DocumentImpl*)fDocument.fImpl,
+                    insertAttr = new (fMemoryManager) AttrNSImpl((DocumentImpl*)fDocument.fImpl,
                        DOMString(buf.getRawBuffer()),     // NameSpaceURI
                        qualifiedName);   // qualified name
 
@@ -1093,7 +1174,7 @@ void DOMParser::endAttList
                 else
                 {
                     // Namespaces is turned off...
-                    insertAttr = new AttrImpl((DocumentImpl*)fDocument.fImpl, attr->getFullName());
+                    insertAttr = new (fMemoryManager) AttrImpl((DocumentImpl*)fDocument.fImpl, attr->getFullName());
                 }
                 insertAttr->setValue(attr->getValue());
                 // memory leak here
@@ -1337,6 +1418,8 @@ Grammar* DOMParser::loadGrammar(const InputSource& source,
 
 void DOMParser::resetCachedGrammarPool()
 {
-    fScanner->resetCachedGrammarPool();
+    fGrammarResolver->resetCachedGrammar();
 }
 
+XERCES_CPP_NAMESPACE_END
+
