@@ -4,7 +4,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 2001-2002 The Apache Software Foundation.  All rights
+ * Copyright (c) 2001-2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -58,7 +58,7 @@
  */
 
 /*
- * $Id: DOMDocumentImpl.hpp,v 1.9 2002/08/16 16:02:53 tng Exp $
+ * $Id: DOMDocumentImpl.hpp,v 1.21 2003/05/23 03:37:14 knoaman Exp $
  */
 
 //
@@ -70,51 +70,51 @@
 //  name is substituded for the *.
 //
 
-#include <string.h>
-#include <xercesc/util/XercesDefs.hpp>
 #include <xercesc/util/RefArrayOf.hpp>
-#include <xercesc/util/RefVectorOf.hpp>
 #include <xercesc/util/RefStackOf.hpp>
 #include <xercesc/util/RefHashTableOf.hpp>
 #include <xercesc/util/KeyRefPair.hpp>
-
 #include <xercesc/dom/DOMDocument.hpp>
 #include <xercesc/dom/DOMUserDataHandler.hpp>
 #include "DOMNodeImpl.hpp"
 #include "DOMParentNode.hpp"
 #include "DOMDeepNodeListPool.hpp"
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 class DOMAttrImpl;
 class DOMCDATASectionImpl;
 class DOMCommentImpl;
+class DOMConfiguration;
 class DOMDeepNodeListImpl;
 class DOMDocumentFragmentImpl;
 class DOMDocumentTypeImpl;
 class DOMElementImpl;
 class DOMEntityImpl;
 class DOMEntityReferenceImpl;
-class DOMNodeImpl;
 class DOMNotationImpl;
 class DOMProcessingInstructionImpl;
 class DOMTextImpl;
 class DOMNodeIteratorImpl;
+class DOMNormalizer;
 class DOMTreeWalkerImpl;
 class DOMNodeFilter;
 class DOMNodeFilterImpl;
 class DOMImplementation;
 class DOMNodeIDMap;
 class DOMRangeImpl;
-class DOMParentNode;
 class DOMStringPool;
 class DOMBuffer;
+class MemoryManager;
 
 typedef RefVectorOf<DOMRangeImpl>        Ranges;
+typedef RefVectorOf<DOMNodeIteratorImpl>     NodeIterators;
 typedef KeyRefPair<void, DOMUserDataHandler> DOMUserDataRecord;
 typedef RefHashTableOf<DOMUserDataRecord> DOMNodeUserDataTable;
 typedef RefStackOf<DOMNode>               DOMNodePtr;
 
-class CDOM_EXPORT DOMDocumentImpl: public DOMDocument {
+class CDOM_EXPORT DOMDocumentImpl: public XMemory, public DOMDocument {
 public:
     // -----------------------------------------------------------------------
     //  data types
@@ -144,10 +144,11 @@ public:
     DOMNodeIDMap*         fNodeIDMap;     // for use by GetElementsById().
 
 public:
-    DOMDocumentImpl();
+    DOMDocumentImpl(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     DOMDocumentImpl(const XMLCh*     namespaceURI,     //DOM Level 2
                     const XMLCh*     qualifiedName,
-                    DOMDocumentType* doctype);
+                    DOMDocumentType* doctype,
+                    MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     virtual ~DOMDocumentImpl();
 
     void                         setDocumentType(DOMDocumentType *doctype);
@@ -175,7 +176,7 @@ public:
     virtual DOMElement*          getDocumentElement() const;
     virtual DOMNodeList*         getElementsByTagName(const XMLCh * tagname) const;
     virtual DOMImplementation*   getImplementation() const;
-    static  bool                 isXMLName(const XMLCh * s);
+    bool                         isXMLName(const XMLCh * s);
     virtual DOMNodeIterator*     createNodeIterator(DOMNode *root,
                                                     unsigned long whatToShow,
                                                     DOMNodeFilter* filter,
@@ -188,7 +189,13 @@ public:
 
     virtual DOMRange*            createRange();
     virtual Ranges*              getRanges() const;  //non-standard api
+    virtual NodeIterators*       getNodeIterators() const;  //non-standard api
     virtual void                 removeRange(DOMRangeImpl* range); //non-standard api
+    virtual void                 removeNodeIterator(DOMNodeIteratorImpl* nodeIterator); //non-standard api
+
+
+    // Extension to be called by the Parser
+    DOMEntityReference*  createEntityReferenceByParser(const XMLCh * name);
 
 
     //
@@ -254,14 +261,10 @@ public:
     virtual void                 setDocumentURI(const XMLCh* documentURI);
     virtual bool                 getStrictErrorChecking() const;
     virtual void                 setStrictErrorChecking(bool strictErrorChecking);
-    virtual DOMErrorHandler*     getErrorHandler() const;
-    virtual void                 setErrorHandler(DOMErrorHandler* const handler);
     virtual DOMNode*             adoptNode(DOMNode* source);
     virtual void                 normalizeDocument();
-    virtual bool                 canSetNormalizationFeature(const XMLCh* const name, bool state) const;
-    virtual void                 setNormalizationFeature(const XMLCh* const name, bool state);
-    virtual bool                 getNormalizationFeature(const XMLCh* const name) const;
-
+    virtual DOMConfiguration*    getDOMConfiguration() const;
+    virtual void                 setDOMConfiguration(DOMConfiguration *config);
 
     // helper functions to prevent storing userdata pointers on every node.
     void*                        setUserData(DOMNodeImpl* n,
@@ -302,6 +305,7 @@ public:
     void                         releaseDocNotifyUserData(DOMNode* object);
     void                         releaseBuffer(DOMBuffer* buffer);
     DOMBuffer*                   popBuffer();
+    MemoryManager*               getMemoryManager() const;
 
     // Factory methods for getting/creating node lists.
     // Because nothing is ever deleted, the implementation caches and recycles
@@ -321,12 +325,13 @@ private:
     //  data
     // -----------------------------------------------------------------------
     // New data introduced in DOM Level 3
-    XMLCh*                fActualEncoding;
-    XMLCh*                fEncoding;
+    const XMLCh*          fActualEncoding;
+    const XMLCh*          fEncoding;
     bool                  fStandalone;
-    XMLCh*                fVersion;
-    XMLCh*                fDocumentURI;
-
+    const XMLCh*          fVersion;
+    const XMLCh*          fDocumentURI;
+    DOMConfiguration*     fDOMConfiguration;
+    
     RefHashTableOf<DOMNodeUserDataTable>* fUserDataTable;
 
 
@@ -363,31 +368,40 @@ private:
     DOMDocumentType*      fDocType;
     DOMElement*           fDocElement;
     DOMStringPool*        fNamePool;
-
+    DOMNormalizer*        fNormalizer;
     Ranges*               fRanges;
+    NodeIterators*        fNodeIterators;
+    MemoryManager*        fMemoryManager;   // configurable memory manager
 
     int                   fChanges;
     bool                  errorChecking;    // Bypass error checking.
 
 };
 
+inline MemoryManager* DOMDocumentImpl::getMemoryManager() const
+{
+    return fMemoryManager;
+}
+
+XERCES_CPP_NAMESPACE_END
+
 // ---------------------------------------------------------------------------
 //
 //  Operator new.  Global overloaded version, lets any object be allocated on
 //                 the heap owned by a document.
 //
 // ---------------------------------------------------------------------------
-inline void * operator new(size_t amt, DOMDocument *doc, DOMDocumentImpl::NodeObjectType type)
+inline void * operator new(size_t amt, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl::NodeObjectType type)
 {
     // revist.  Probably should be a checked cast.
-    void *p = ((DOMDocumentImpl *)doc)->allocate(amt, type);
+    void *p = ((XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl *)doc)->allocate(amt, type);
     return p;
 }
 
-inline void * operator new(size_t amt, DOMDocument *doc)
+inline void * operator new(size_t amt, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc)
 {
     // revist.  Probably should be a checked cast.
-    void *p = ((DOMDocumentImpl *)doc)->allocate(amt);
+    void *p = ((XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl *)doc)->allocate(amt);
     return p;
 }
 
@@ -397,11 +411,11 @@ inline void * operator new(size_t amt, DOMDocument *doc)
 //    no matching operator delete found; memory will not be freed if initialization throws an exception
 // ---------------------------------------------------------------------------
 #if _MSC_VER >= 1200 /* VC++ 6.0 */
-inline void operator delete(void* ptr, DOMDocument *doc)
+inline void operator delete(void* ptr, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc)
 {
     return;
 }
-inline void operator delete(void* ptr, DOMDocument *doc, DOMDocumentImpl::NodeObjectType type)
+inline void operator delete(void* ptr, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl::NodeObjectType type)
 {
     return;
 }
