@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DOMElementImpl.cpp,v 1.12 2002/08/16 19:20:28 tng Exp $
+ * $Id: DOMElementImpl.cpp,v 1.20 2003/05/16 06:01:50 knoaman Exp $
  */
 
 #include "DOMElementImpl.hpp"
@@ -63,24 +63,29 @@
 #include <xercesc/dom/DOMAttr.hpp>
 #include <xercesc/dom/DOMDocument.hpp>
 #include <xercesc/dom/DOMException.hpp>
+#include <xercesc/util/XMLUniDefs.hpp>
+#include <xercesc/util/XMLUri.hpp>
 
 #include "DOMAttrMapImpl.hpp"
+#include "DOMAttrImpl.hpp"
 #include "DOMDocumentImpl.hpp"
 #include "DOMParentNode.hpp"
 #include "DOMStringPool.hpp"
 #include "DOMCasts.hpp"
 #include "DOMElementNSImpl.hpp"
-
+#include "DOMTypeInfoImpl.hpp"
 
 #include "DOMDeepNodeListImpl.hpp"
 #include "DOMDocumentTypeImpl.hpp"
 #include "DOMNamedNodeMapImpl.hpp"
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 class DOMAttr;
 
 DOMElementImpl::DOMElementImpl(DOMDocument *ownerDoc, const XMLCh *eName)
-    : fNode(ownerDoc), fParent(ownerDoc), fAttributes(0), fDefaultAttributes(0)
+    : fNode(ownerDoc), fParent(ownerDoc), fAttributes(0), fDefaultAttributes(0), fSchemaType(0)
 {
     DOMDocumentImpl *docImpl = (DOMDocumentImpl *)ownerDoc;
     fName = docImpl->getPooledString(eName);
@@ -98,10 +103,12 @@ DOMElementImpl::DOMElementImpl(DOMDocument *ownerDoc, const XMLCh *eName)
 DOMElementImpl::DOMElementImpl(const DOMElementImpl &other, bool deep)
     : fNode(other.getOwnerDocument()),
       fParent(other.getOwnerDocument()),
+      fSchemaType(other.fSchemaType),
       fAttributes(0),
       fDefaultAttributes(0)
 {
     fName = other.fName;
+
     if (deep)
         fParent.cloneChildren(&other);
 
@@ -129,6 +136,7 @@ DOMElementImpl::DOMElementImpl(const DOMElementImpl &other, bool deep)
             fAttributes = new (getOwnerDocument()) DOMAttrMapImpl(this, fDefaultAttributes);
         }
     }
+
 };
 
 
@@ -178,7 +186,6 @@ DOMAttr *DOMElementImpl::getAttributeNode(const XMLCh *nam) const
     return  (DOMAttr *)fAttributes->getNamedItem(nam);
 };
 
-#include "stdio.h"
 
 DOMNamedNodeMap *DOMElementImpl::getAttributes() const
 {
@@ -211,6 +218,7 @@ void DOMElementImpl::removeAttribute(const XMLCh *nam)
     if (i >= 0)
     {
         DOMNode *att = fAttributes->removeNamedItemAt(i);
+        ((DOMAttrImpl *)att)->removeAttrFromIDNodeMap();
         att->release();
     }
 };
@@ -236,8 +244,10 @@ DOMAttr *DOMElementImpl::removeAttributeNode(DOMAttr *oldAttr)
     if (i >= 0) {
         // If it is in fact the right object, remove it.
         found = fAttributes->item(i);
-        if (found == oldAttr)
+        if (found == oldAttr) {
             fAttributes->removeNamedItemAt(i);
+            ((DOMAttrImpl *)oldAttr)->removeAttrFromIDNodeMap();
+        }
         else
             throw DOMException(DOMException::NOT_FOUND_ERR, 0);
 
@@ -266,6 +276,54 @@ void DOMElementImpl::setAttribute(const XMLCh *nam, const XMLCh *val)
     newAttr->setNodeValue(val);
 };
 
+void DOMElementImpl::setIdAttribute(const XMLCh* name)
+{
+    if (fNode.isReadOnly())
+        throw DOMException(
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+
+    DOMAttr *attr = getAttributeNode(name);
+
+    if (!attr) 
+        throw DOMException(DOMException::NOT_FOUND_ERR, 0);
+
+    ((DOMAttrImpl *)attr)->addAttrToIDNodeMap();
+};
+
+void DOMElementImpl::setIdAttributeNS(const XMLCh* namespaceURI, const XMLCh* localName) {
+
+    if (fNode.isReadOnly())
+        throw DOMException(
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+
+    DOMAttr *attr = getAttributeNodeNS(namespaceURI, localName);
+
+    if (!attr) 
+        throw DOMException(DOMException::NOT_FOUND_ERR, 0);
+
+    ((DOMAttrImpl *)attr)->addAttrToIDNodeMap();
+
+};
+
+
+void DOMElementImpl::setIdAttributeNode(const DOMAttr *idAttr) {
+
+    if (fNode.isReadOnly())
+        throw DOMException(
+        DOMException::NO_MODIFICATION_ALLOWED_ERR, 0);
+
+    DOMAttr *attr;
+    const XMLCh* localName = idAttr->getLocalName();
+    if (localName)
+        attr = getAttributeNodeNS(idAttr->getNamespaceURI(), idAttr->getLocalName());
+    else 
+        attr = getAttributeNode(idAttr->getName());
+    
+    if(!attr) 
+        throw DOMException(DOMException::NOT_FOUND_ERR, 0);
+
+    ((DOMAttrImpl *)attr)->addAttrToIDNodeMap();
+};
 
 
 DOMAttr * DOMElementImpl::setAttributeNode(DOMAttr *newAttr)
@@ -305,7 +363,7 @@ const XMLCh * DOMElementImpl::getAttributeNS(const XMLCh *fNamespaceURI,
 {
     DOMAttr * attr=
       (DOMAttr *)(fAttributes->getNamedItemNS(fNamespaceURI, fLocalName));
-    return (attr==0) ? 0 : attr->getValue();
+    return (attr==0) ? XMLUni::fgZeroLenString : attr->getValue();
 }
 
 
@@ -465,6 +523,35 @@ void DOMElementImpl::release()
     }
 }
 
+const XMLCh* DOMElementImpl::getBaseURI() const
+{
+    const XMLCh* baseURI = fNode.fOwnerNode->getBaseURI();
+    if (fAttributes) {
+        const XMLCh xmlBaseString[] =
+        {
+            chLatin_x, chLatin_m, chLatin_l, chColon, chLatin_b, chLatin_a, chLatin_s, chLatin_e, chNull
+        };
+        DOMNode* attrNode = fAttributes->getNamedItem(xmlBaseString);
+        if (attrNode) {
+            const XMLCh* uri =  attrNode->getNodeValue();
+            if (uri && *uri) {// attribute value is always empty string
+                try {
+                    XMLUri temp(baseURI, ((DOMDocumentImpl *)this->getOwnerDocument())->getMemoryManager());
+                    XMLUri temp2(&temp, uri, ((DOMDocumentImpl *)this->getOwnerDocument())->getMemoryManager());
+                    uri = ((DOMDocumentImpl *)this->getOwnerDocument())->cloneString(temp2.getUriText());
+                }
+                catch (...){
+                    // REVISIT: what should happen in this case?
+                    return 0;
+                }
+                return uri;
+            }
+        }
+    }
+    return baseURI;
+}
+
+
 
 //
 //   Functions inherited from Node
@@ -477,7 +564,7 @@ void DOMElementImpl::release()
      const XMLCh*           DOMElementImpl::getNamespaceURI() const                 {return fNode.getNamespaceURI (); };
            DOMNode*         DOMElementImpl::getNextSibling() const                  {return fChild.getNextSibling (); };
      const XMLCh*           DOMElementImpl::getNodeValue() const                    {return fNode.getNodeValue (); };
-           DOMDocument*     DOMElementImpl::getOwnerDocument() const                {return fNode.getOwnerDocument (); };
+           DOMDocument*     DOMElementImpl::getOwnerDocument() const                {return fParent.fOwnerDocument; };
      const XMLCh*           DOMElementImpl::getPrefix() const                       {return fNode.getPrefix (); };
            DOMNode*         DOMElementImpl::getParentNode() const                   {return fChild.getParentNode (this); };
            DOMNode*         DOMElementImpl::getPreviousSibling() const              {return fChild.getPreviousSibling (this); };
@@ -491,12 +578,11 @@ void DOMElementImpl::release()
            bool             DOMElementImpl::isSupported(const XMLCh *feature, const XMLCh *version) const
                                                                                     {return fNode.isSupported (feature, version); };
            void             DOMElementImpl::setPrefix(const XMLCh  *prefix)         {fNode.setPrefix(prefix); };
-           bool             DOMElementImpl::isSameNode(const DOMNode* other)        {return fNode.isSameNode(other); };
+           bool             DOMElementImpl::isSameNode(const DOMNode* other) const  {return fNode.isSameNode(other); };
            void*            DOMElementImpl::setUserData(const XMLCh* key, void* data, DOMUserDataHandler* handler)
                                                                                     {return fNode.setUserData(key, data, handler); };
            void*            DOMElementImpl::getUserData(const XMLCh* key) const     {return fNode.getUserData(key); };
-           const XMLCh*     DOMElementImpl::getBaseURI() const                      {return fNode.getBaseURI(); };
-           short            DOMElementImpl::compareTreePosition(DOMNode* other)     {return fNode.compareTreePosition(other); };
+           short            DOMElementImpl::compareTreePosition(const DOMNode* other) const {return fNode.compareTreePosition(other); };
            const XMLCh*     DOMElementImpl::getTextContent() const                  {return fNode.getTextContent(); };
            void             DOMElementImpl::setTextContent(const XMLCh* textContent){fNode.setTextContent(textContent); };
            const XMLCh*     DOMElementImpl::lookupNamespacePrefix(const XMLCh* namespaceURI, bool useDefault) const  {return fNode.lookupNamespacePrefix(namespaceURI, useDefault); };
@@ -506,8 +592,12 @@ void DOMElementImpl::release()
 
 
 
-bool DOMElementImpl::isEqualNode(const DOMNode* arg)
+bool DOMElementImpl::isEqualNode(const DOMNode* arg) const
 {
+    if (isSameNode(arg)) {
+        return true;
+    }
+
     if (!fNode.isEqualNode(arg)) {
         return false;
     }
@@ -595,3 +685,18 @@ DOMNode* DOMElementImpl::rename(const XMLCh* namespaceURI, const XMLCh* name)
     }
 }
 
+const DOMTypeInfo *DOMElementImpl::getTypeInfo() const
+{
+    if(!fSchemaType) 
+        ((DOMElementImpl *)(this))->fSchemaType = new (getOwnerDocument()) DOMTypeInfoImpl(0, 0, (DOMDocumentImpl *)getOwnerDocument());
+    return fSchemaType;
+}
+
+
+void DOMElementImpl::setTypeInfo(const XMLCh* typeName, const XMLCh* typeURI) 
+{
+    fSchemaType = new (getOwnerDocument()) DOMTypeInfoImpl(typeName, typeURI, (DOMDocumentImpl *)getOwnerDocument());
+}
+
+XERCES_CPP_NAMESPACE_END
+
