@@ -55,8 +55,38 @@
  */
 
 /*
- * $Id: DOMWriterImpl.hpp,v 1.7 2002/06/25 16:17:16 tng Exp $
+ * $Id: DOMWriterImpl.hpp,v 1.16 2003/05/22 02:10:51 knoaman Exp $
  * $Log: DOMWriterImpl.hpp,v $
+ * Revision 1.16  2003/05/22 02:10:51  knoaman
+ * Default the memory manager.
+ *
+ * Revision 1.15  2003/05/15 18:25:54  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.14  2003/05/12 16:08:11  gareth
+ * fix to #18832. Corrected serilization with regards to namespace nodes. Patch by Alby Massari.
+ *
+ * Revision 1.13  2003/03/16 05:42:04  peiyongz
+ * Bug#17983 Formatter does not escape control characters
+ *
+ * Revision 1.12  2003/01/28 18:31:47  peiyongz
+ * Bug#13694: Allow Xerces to write the BOM to XML files
+ *
+ * Revision 1.11  2003/01/20 16:50:13  tng
+ * DOMWriter fix:
+ * 1. wrong wrong nested cdata message
+ * 2. pretty format the cdata section
+ * 3. do not increment error count if warning was issued
+ *
+ * Revision 1.10  2002/12/10 21:01:32  tng
+ * NLS: DOMWriter should use message loader to load message instead of using hardcoded static stirng
+ *
+ * Revision 1.9  2002/12/09 11:46:08  gareth
+ * More pretty pretty print feature. Patch by Kevin King. Closes bug #13840.
+ *
+ * Revision 1.8  2002/11/04 15:07:35  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.7  2002/06/25 16:17:16  tng
  * DOM L3: add release()
  *
@@ -302,6 +332,12 @@
 
 #include <xercesc/dom/DOM.hpp>
 #include <xercesc/dom/DOMWriter.hpp>
+#include <xercesc/util/XMLDOMMsg.hpp>
+#include <xercesc/util/RefHashTableOf.hpp>
+#include <xercesc/util/RefVectorOf.hpp>
+
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 class CDOM_EXPORT DOMWriterImpl:public DOMWriter {
 
@@ -313,7 +349,7 @@ public:
     /**
      * Constructor.
      */
-    DOMWriterImpl();
+    DOMWriterImpl(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
 
     /**
      * Destructor.
@@ -371,12 +407,17 @@ private:
     bool                          reportError(const DOMNode* const    errorNode
                                             , DOMError::ErrorSeverity errorType
                                             , const XMLCh*   const    errorMsg);
+    bool                          reportError(const DOMNode* const    errorNode
+                                            , DOMError::ErrorSeverity errorType
+                                            , XMLDOMMsg::Codes        toEmit);
 
     void                          procCdataSection(const XMLCh*   const nodeValue
-                                                 , const DOMNode* const nodeToWrite);
+                                                 , const DOMNode* const nodeToWrite
+                                                 , int level);
 
     void                          procUnrepCharInCdataSection(const XMLCh*   const nodeValue
-                                                            , const DOMNode* const nodeToWrite);
+                                                            , const DOMNode* const nodeToWrite
+                                                            , int level);
 
     bool                          canSetFeature(const int featureId
                                               , bool      val)     const;
@@ -384,9 +425,16 @@ private:
                                            , bool      val);
     bool                          getFeature(const int featureId) const;
 
-    void                          printNewLine() const;
+    void                          printNewLine();
     void                          setURCharRef();
 
+
+    void printIndent(int level) const;
+    //does the actual work for processNode while keeping track of the level
+    void processNode(const DOMNode* const nodeToWrite, int level);
+
+    void processBOM();
+
     // -----------------------------------------------------------------------
     //  Private data members
     //
@@ -404,6 +452,9 @@ private:
     //  fFilter
     //      don't own it
     //
+    //  fDocumentVersion
+    //      The XML Version of the document to be serialized.
+    // 
     //  fEncodingUsed (session var)
     //      the actual encoding used in WriteNode(),
     //      it does not own any data(memory).
@@ -421,6 +472,10 @@ private:
     //      treat as fatal. And the serializer will return true/false
     //      based on this value.
     //
+    //  fCurrentLine
+    //      the current line. Used to track the line number the current
+    //      node begins on
+    //
     // -----------------------------------------------------------------------
 
     int                           fFeatures;
@@ -428,12 +483,17 @@ private:
     XMLCh                        *fNewLine;
     DOMErrorHandler              *fErrorHandler;
     DOMWriterFilter              *fFilter;
+    const XMLCh                  *fDocumentVersion;
 
     //session vars
     const XMLCh                  *fEncodingUsed;
     const XMLCh                  *fNewLineUsed;
     XMLFormatter                 *fFormatter;
     int                           fErrorCount;
+    int                           fCurrentLine;
+
+    RefVectorOf< RefHashTableOf<XMLCh> >* fNamespaceStack;
+    MemoryManager*               fMemoryManager;
 };
 
 inline void DOMWriterImpl::setFeature(const int featureId
@@ -452,4 +512,6 @@ inline void DOMWriterImpl::setURCharRef()
     fFormatter->setUnRepFlags(XMLFormatter::UnRep_CharRef);
 }
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
