@@ -55,7 +55,7 @@
  */
 
 /**
- * $Id: XMLFormatter.cpp,v 1.1.1.1 2002/02/01 22:21:51 peiyongz Exp $
+ * $Id: XMLFormatter.cpp,v 1.12 2003/05/16 21:36:55 knoaman Exp $
  */
 
 
@@ -69,8 +69,12 @@
 #include <xercesc/util/TranscodingException.hpp>
 #include <xercesc/util/XMLExceptMsgs.hpp>
 #include <xercesc/framework/XMLFormatter.hpp>
+#include <xercesc/util/Janitor.hpp>
+#include <xercesc/util/XMLChar.hpp>
+
 #include <string.h>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Local data
@@ -123,8 +127,8 @@ static const XMLCh gEscapeChars[XMLFormatter::EscapeFlags_Count][kEscapeCount] =
 // ---------------------------------------------------------------------------
 //  Local methods
 // ---------------------------------------------------------------------------
-static inline bool inEscapeList(const   XMLFormatter::EscapeFlags escStyle
-                                , const XMLCh                     toCheck)
+bool XMLFormatter::inEscapeList(const XMLFormatter::EscapeFlags escStyle
+                              , const XMLCh                     toCheck)
 {
     const XMLCh* escList = gEscapeChars[escStyle];
     while (*escList)
@@ -132,7 +136,42 @@ static inline bool inEscapeList(const   XMLFormatter::EscapeFlags escStyle
         if (*escList++ == toCheck)
             return true;
     }
-    return false;
+
+    /***
+     *  XML1.1
+     *
+     *  Finally, there is considerable demand to define a standard representation of 
+     *  arbitrary Unicode characters in XML documents. Therefore, XML 1.1 allows the 
+     *  use of character references to the control characters #x1 through #x1F, 
+     *  most of which are forbidden in XML 1.0. For reasons of robustness, however, 
+     *  these characters still cannot be used directly in documents.
+     *  In order to improve the robustness of character encoding detection, the 
+     *  additional control characters #x7F through #x9F, which were freely allowed in 
+     *  XML 1.0 documents, now must also appear only as character references. 
+     *  (Whitespace characters are of course exempt.) The minor sacrifice of backward 
+     *  compatibility is considered not significant. 
+     *  Due to potential problems with APIs, #x0 is still forbidden both directly and 
+     *  as a character reference.
+     *
+    ***/
+    if (fIsXML11)
+    {
+        // for XML11
+        if ( XMLChar1_1::isControlChar(toCheck, 0) &&
+            !XMLChar1_1::isWhitespace(toCheck, 0)   )
+        {
+            return true;    
+        }
+        else
+        {    
+            return false;
+        }
+    }
+    else
+    {
+        return false;
+    }
+
 }
 
 
@@ -140,30 +179,31 @@ static inline bool inEscapeList(const   XMLFormatter::EscapeFlags escStyle
 //  XMLFormatter: Constructors and Destructor
 // ---------------------------------------------------------------------------
 XMLFormatter::XMLFormatter( const   char* const             outEncoding
+                            , const char* const             docVersion
                             ,       XMLFormatTarget* const  target
                             , const EscapeFlags             escapeFlags
-                            , const UnRepFlags              unrepFlags) :
-
-    fEscapeFlags(escapeFlags)
+                            , const UnRepFlags              unrepFlags
+                            ,       MemoryManager* const    manager)
+    : fEscapeFlags(escapeFlags)
     , fOutEncoding(0)
     , fTarget(target)
     , fUnRepFlags(unrepFlags)
-    , fXCoder(0)
-
+    , fXCoder(0)  
     , fAposRef(0)
-    , fAmpRef(0)
-    , fGTRef(0)
-    , fLTRef(0)
-    , fQuoteRef(0)
-
     , fAposLen(0)
-    , fAmpLen(0)
+    , fAmpRef(0)    
+    , fAmpLen(0)    
+    , fGTRef(0)
     , fGTLen(0)
+    , fLTRef(0)
     , fLTLen(0)
-    , fQuoteLen(0)
+    , fQuoteRef(0)
+    , fQuoteLen(0) 
+    , fIsXML11(false)
+    , fMemoryManager(manager)
 {
     // Transcode the encoding string
-    fOutEncoding = XMLString::transcode(outEncoding);
+    fOutEncoding = XMLString::transcode(outEncoding, fMemoryManager);
 
     // Try to create a transcoder for this encoding
     XMLTransService::Codes resCode;
@@ -172,12 +212,12 @@ XMLFormatter::XMLFormatter( const   char* const             outEncoding
         fOutEncoding
         , resCode
         , kTmpBufSize
+        , fMemoryManager
     );
 
     if (!fXCoder)
     {
-        delete [] fOutEncoding;
-        fOutEncoding = 0;
+        fMemoryManager->deallocate(fOutEncoding); //delete [] fOutEncoding;
         ThrowXML1
         (
             TranscodingException
@@ -185,41 +225,49 @@ XMLFormatter::XMLFormatter( const   char* const             outEncoding
             , outEncoding
         );
     }
+
+    XMLCh* const tmpDocVer = XMLString::transcode(docVersion, fMemoryManager);
+    ArrayJanitor<XMLCh> jname(tmpDocVer, fMemoryManager);
+    fIsXML11 = XMLString::equals(tmpDocVer, XMLUni::fgVersion1_1);
 }
 
 
 XMLFormatter::XMLFormatter( const   XMLCh* const            outEncoding
+                            , const XMLCh* const            docVersion
                             ,       XMLFormatTarget* const  target
                             , const EscapeFlags             escapeFlags
-                            , const UnRepFlags              unrepFlags) :
-    fEscapeFlags(escapeFlags)
+                            , const UnRepFlags              unrepFlags
+                            ,       MemoryManager* const    manager)
+    : fEscapeFlags(escapeFlags)
     , fOutEncoding(0)
     , fTarget(target)
     , fUnRepFlags(unrepFlags)
-    , fXCoder(0)
-
+    , fXCoder(0)  
     , fAposRef(0)
-    , fAmpRef(0)
+    , fAposLen(0)
+    , fAmpRef(0)    
+    , fAmpLen(0)    
     , fGTRef(0)
+    , fGTLen(0)
     , fLTRef(0)
+    , fLTLen(0)
     , fQuoteRef(0)
+    , fQuoteLen(0) 
+    , fIsXML11(false)
+    , fMemoryManager(manager)
 {
-    // Copy the encoding string
-    fOutEncoding = XMLString::replicate(outEncoding);
-
     // Try to create a transcoder for this encoding
     XMLTransService::Codes resCode;
     fXCoder = XMLPlatformUtils::fgTransService->makeNewTranscoderFor
     (
-        fOutEncoding
+        outEncoding
         , resCode
         , kTmpBufSize
+        , fMemoryManager
     );
 
     if (!fXCoder)
     {
-        delete [] fOutEncoding;
-        fOutEncoding = 0;
         ThrowXML1
         (
             TranscodingException
@@ -227,17 +275,22 @@ XMLFormatter::XMLFormatter( const   XMLCh* const            outEncoding
             , outEncoding
         );
     }
+
+    // Copy the encoding string
+    fOutEncoding = XMLString::replicate(outEncoding, fMemoryManager);
+
+
+    fIsXML11 = XMLString::equals(docVersion, XMLUni::fgVersion1_1);
 }
 
 XMLFormatter::~XMLFormatter()
 {
-    delete [] fAposRef;
-    delete [] fAmpRef;
-    delete [] fGTRef;
-    delete [] fLTRef;
-    delete [] fQuoteRef;
-
-    delete [] fOutEncoding;
+    fMemoryManager->deallocate(fAposRef); //delete [] fAposRef;
+    fMemoryManager->deallocate(fAmpRef); //delete [] fAmpRef;
+    fMemoryManager->deallocate(fGTRef); //delete [] fGTRef;
+    fMemoryManager->deallocate(fLTRef); //delete [] fLTRef;
+    fMemoryManager->deallocate(fQuoteRef); //delete [] fQuoteRef;
+    fMemoryManager->deallocate(fOutEncoding); //delete [] fOutEncoding;
     delete fXCoder;
 
     // We DO NOT own the target object!
@@ -275,71 +328,27 @@ XMLFormatter::formatBuf(const   XMLCh* const    toFormat
         return;
     }
 
-    //
-    //  Use that to figure out what I should pass to the transcoder. If we
-    //  are doing character references or failing for unrepresentable chars,
-    //  then we just throw, since we should never get a call for something
-    //  we cannot represent. Else, we tell it to just use the replacement
-    //  char.
-    //
-    const XMLTranscoder::UnRepOpts unRepOpts = (actualUnRep == UnRep_Replace)
-                                             ? XMLTranscoder::UnRep_RepChar
-                                             : XMLTranscoder::UnRep_Throw;
     //
     //  If we don't have any escape flags set, then we can do the most
     //  efficient loop, else we have to do it the hard way.
     //
     const XMLCh*    srcPtr = toFormat;
     const XMLCh*    endPtr = toFormat + count;
-    unsigned int    charsEaten;
     if (actualEsc == NoEscapes)
     {
         //
-        //  Just do a whole buffer at a time into the temp buffer, cap it
-        //  off, and send it to the target.
+        //  Just do a whole buffer at a time into the temp buffer, cap 
+        //  it off, and send it to the target. 
         //
-        while (srcPtr < endPtr)
-        {
-            const unsigned int srcCount = endPtr - srcPtr;
-            const unsigned srcChars = srcCount > kTmpBufSize ?
-                                      kTmpBufSize : srcCount;
-
-            const unsigned int outBytes = fXCoder->transcodeTo
-            (
-                srcPtr
-                , srcChars
-                , fTmpBuf
-                , kTmpBufSize
-                , charsEaten
-                , unRepOpts
-            );
-
-            #if defined(XML_DEBUG)
-            if ((outBytes > kTmpBufSize)
-            ||  (charsEaten > srcCount))
-            {
-                // <TBD> The transcoder is freakin out maaaannn
-            }
-            #endif
-
-            // If we get any bytes out, then write them
-            if (outBytes)
-            {
-                fTmpBuf[outBytes] = 0; fTmpBuf[outBytes + 1] = 0;
-                fTmpBuf[outBytes + 2] = 0; fTmpBuf[outBytes + 3] = 0;
-                fTarget->writeChars(fTmpBuf, outBytes, this);
-            }
-
-            // And bump up our pointer
-            srcPtr += charsEaten;
-        }
+        if (srcPtr < endPtr)
+           srcPtr += handleUnEscapedChars(srcPtr, endPtr - srcPtr, actualUnRep); 
     }
      else
     {
         //
-        //  Escap chars that require it according tot he scale flags we were
-        //  given. For the others, try to accumulate them and format them in
-        //  as big as bulk as we can.
+        //  Escape chars that require it according tot he scale flags 
+        //  we were given. For the others, try to accumulate them and 
+        //  format them in as big as bulk as we can. 
         //
         while (srcPtr < endPtr)
         {
@@ -357,77 +366,45 @@ XMLFormatter::formatBuf(const   XMLCh* const    toFormat
             //  out.
             //
             if (tmpPtr > srcPtr)
-            {
-                const unsigned int srcCount = tmpPtr - srcPtr;
-                const unsigned srcChars = srcCount > kTmpBufSize ?
-                                          kTmpBufSize : srcCount;
+               srcPtr += handleUnEscapedChars(srcPtr, tmpPtr - srcPtr,  
+                                              actualUnRep); 
 
-                const unsigned int outBytes = fXCoder->transcodeTo
-                (
-                    srcPtr
-                    , srcChars
-                    , fTmpBuf
-                    , kTmpBufSize
-                    , charsEaten
-                    , unRepOpts
-                );
-
-                #if defined(XML_DEBUG)
-                if ((outBytes > kTmpBufSize)
-                ||  (charsEaten > srcCount))
-                {
-                    // <TBD> The transcoder is freakin out maaaannn
-                }
-                #endif
-
-                // If we get any bytes out, then write them
-                if (outBytes)
-                {
-                    fTmpBuf[outBytes] = 0; fTmpBuf[outBytes + 1] = 0;
-                    fTmpBuf[outBytes + 2] = 0; fTmpBuf[outBytes + 3] = 0;
-                    fTarget->writeChars(fTmpBuf, outBytes, this);
-                }
-
-                // And bump up our pointer
-                srcPtr += charsEaten;
-            }
              else if (tmpPtr < endPtr)
             {
                 //
                 //  Ok, so we've hit a char that must be escaped. So do
                 //  this one specially.
                 //
-                const XMLByte * theChars;
-                unsigned int count = 0;
-                switch(*srcPtr)
-                {
+                const XMLByte * theChars;                
+                switch (*srcPtr) { 
                     case chAmpersand :
-                        theChars = getAmpRef(count);
-                        fTarget->writeChars(theChars, count, this);
+                        theChars = getCharRef(fAmpLen, fAmpRef, gAmpRef); 
+                        fTarget->writeChars(theChars, fAmpLen, this);
                         break;
 
                     case chSingleQuote :
-                        theChars = getAposRef(count);
-                        fTarget->writeChars(theChars, count, this);
+                        theChars = getCharRef(fAposLen, fAposRef, gAposRef); 
+                        fTarget->writeChars(theChars, fAposLen, this);
                         break;
 
                     case chDoubleQuote :
-                        theChars = getQuoteRef(count);
-                        fTarget->writeChars(theChars, count, this);
+                        theChars = getCharRef(fQuoteLen, fQuoteRef, gQuoteRef); 
+                        fTarget->writeChars(theChars, fQuoteLen, this);
                         break;
 
                     case chCloseAngle :
-                        theChars = getGTRef(count);
-                        fTarget->writeChars(theChars, count, this);
+                        theChars = getCharRef(fGTLen, fGTRef, gGTRef); 
+                        fTarget->writeChars(theChars, fGTLen, this);
                         break;
 
                     case chOpenAngle :
-                        theChars = getLTRef(count);
-                        fTarget->writeChars(theChars, count, this);
+                        theChars = getCharRef(fLTLen, fLTRef, gLTRef); 
+                        fTarget->writeChars(theChars, fLTLen, this);
                         break;
 
                     default:
-                        // <TBD> This is obviously an error
+                        // control characters
+                        writeCharRef(*srcPtr);
                         break;
                 }
                 srcPtr++;
@@ -436,6 +413,49 @@ XMLFormatter::formatBuf(const   XMLCh* const    toFormat
     }
 }
 
+
+unsigned int 
+XMLFormatter::handleUnEscapedChars(const XMLCh *                  srcPtr, 
+                                   const unsigned int             oCount, 
+                                   const UnRepFlags               actualUnRep) 
+{ 
+   //
+   //  Use that to figure out what I should pass to the transcoder. If we
+   //  are doing character references or failing for unrepresentable chars,
+   //  then we just throw, since we should never get a call for something
+   //  we cannot represent. Else, we tell it to just use the replacement
+   //  char.
+   //
+   const XMLTranscoder::UnRepOpts unRepOpts = (actualUnRep == UnRep_Replace)
+                                             ? XMLTranscoder::UnRep_RepChar
+                                             : XMLTranscoder::UnRep_Throw;
+                                             	
+   unsigned int charsEaten; 
+   unsigned int count = oCount; 
+ 
+   while (count) { 
+      const unsigned srcChars  
+         = count > kTmpBufSize ? kTmpBufSize : count; 
+ 
+      const unsigned int outBytes  
+         = fXCoder->transcodeTo(srcPtr, srcChars,  
+                                fTmpBuf, kTmpBufSize, 
+                                charsEaten, unRepOpts); 
+ 
+      if (outBytes) { 
+         fTmpBuf[outBytes]     = 0; fTmpBuf[outBytes + 1] = 0; 
+         fTmpBuf[outBytes + 2] = 0; fTmpBuf[outBytes + 3] = 0; 
+         fTarget->writeChars(fTmpBuf, outBytes, this); 
+      } 
+ 
+      srcPtr += charsEaten; 
+      count  -= charsEaten; 
+   } 
+    
+   return oCount; // This should be an assertion that count == 0. 
+} 
+ 
+ 
 XMLFormatter& XMLFormatter::operator<<(const XMLCh* const toFormat)
 {
     const unsigned int len = XMLString::stringLen(toFormat);
@@ -454,147 +474,69 @@ XMLFormatter& XMLFormatter::operator<<(const XMLCh toFormat)
     return *this;
 }
 
-
+/**
+ * the parameter, count, is needed since stringLen()
+ * does not work on a BOM like "0xFE0xFF0x000x00" or
+ * "0x000x000xFF0xFE"
+ **/
+void XMLFormatter::writeBOM(const XMLByte* const toFormat
+                          , const unsigned int   count)
+{
+    fTarget->writeChars(toFormat, count, this);    
+}
 
 // ---------------------------------------------------------------------------
 //  XMLFormatter: Private helper methods
 // ---------------------------------------------------------------------------
-const XMLByte* XMLFormatter::getAposRef(unsigned int & count)
+const void XMLFormatter::writeCharRef(const XMLCh &toWrite)
 {
-    if (fAposRef)
-    {
-        count = fAposLen;
-        return fAposRef;
-    }
+    XMLCh tmpBuf[32];
+    tmpBuf[0] = chAmpersand;
+    tmpBuf[1] = chPound;
+    tmpBuf[2] = chLatin_x;
 
-    unsigned int charsEaten;
-    const unsigned int outBytes = fXCoder->transcodeTo
-    (
-        gAposRef
-        , XMLString::stringLen(gAposRef)
-        , fTmpBuf
-        , kTmpBufSize
-        , charsEaten
-        , XMLTranscoder::UnRep_Throw
-    );
-    fTmpBuf[outBytes] = 0; fTmpBuf[outBytes + 1] = 0;
-    fTmpBuf[outBytes + 2] = 0; fTmpBuf[outBytes + 3] = 0;
+    // Build a char ref for the current char
+    XMLString::binToText(toWrite, &tmpBuf[3], 8, 16);
+    const unsigned int bufLen = XMLString::stringLen(tmpBuf);
+    tmpBuf[bufLen] = chSemiColon;
+    tmpBuf[bufLen+1] = chNull;
+
+    // write it out
+    formatBuf(tmpBuf
+            , bufLen + 1
+            , XMLFormatter::NoEscapes
+            , XMLFormatter::UnRep_Fail);
 
-    ((XMLFormatter*)this)->fAposRef = new XMLByte[outBytes + 4];
-    memcpy(fAposRef, fTmpBuf, outBytes + 4);
-    count = fAposLen = outBytes;
-    return fAposRef;
 }
 
-const XMLByte* XMLFormatter::getAmpRef(unsigned int & count)
+const XMLByte* XMLFormatter::getCharRef(unsigned int & count, 
+                                        XMLByte*       &ref, 
+                                        const XMLCh *  stdRef) 
 {
-    if (fAmpRef)
-    {
-        count = fAmpLen;
-        return fAmpRef;
-    }
+   if (!ref) { 
 
-    unsigned int charsEaten;
-    const unsigned int outBytes = fXCoder->transcodeTo
-    (
-        gAmpRef
-        , XMLString::stringLen(gAmpRef)
-        , fTmpBuf
-        , kTmpBufSize
-        , charsEaten
-        , XMLTranscoder::UnRep_Throw
-    );
-    fTmpBuf[outBytes] = 0; fTmpBuf[outBytes + 1] = 0;
-    fTmpBuf[outBytes + 2] = 0; fTmpBuf[outBytes + 3] = 0;
+       unsigned int charsEaten;
+       const unsigned int outBytes = 
+           fXCoder->transcodeTo(stdRef, XMLString::stringLen(stdRef), 
+                                fTmpBuf, kTmpBufSize, charsEaten, 
+                                XMLTranscoder::UnRep_Throw); 
 
-    ((XMLFormatter*)this)->fAmpRef = new XMLByte[outBytes + 4];
-    memcpy(fAmpRef, fTmpBuf, outBytes + 4);
-    count = fAmpLen = outBytes;
-    return fAmpRef;
+       fTmpBuf[outBytes] = 0; 
+       fTmpBuf[outBytes + 1] = 0;
+       fTmpBuf[outBytes + 2] = 0; 
+       fTmpBuf[outBytes + 3] = 0;
+
+       ref = (XMLByte*) fMemoryManager->allocate
+       (
+           (outBytes + 4) * sizeof(XMLByte)
+       );//new XMLByte[outBytes + 4]; 
+       memcpy(ref, fTmpBuf, outBytes + 4); 
+       count = outBytes; 
+   }
+
+   return ref; 
 }
 
-const XMLByte* XMLFormatter::getGTRef(unsigned int & count)
-{
-    if (fGTRef)
-    {
-        count = fGTLen;
-        return fGTRef;
-    }
-
-    unsigned int charsEaten;
-    const unsigned int outBytes = fXCoder->transcodeTo
-    (
-        gGTRef
-        , XMLString::stringLen(gGTRef)
-        , fTmpBuf
-        , kTmpBufSize
-        , charsEaten
-        , XMLTranscoder::UnRep_Throw
-    );
-    fTmpBuf[outBytes] = 0; fTmpBuf[outBytes + 1] = 0;
-    fTmpBuf[outBytes + 2] = 0; fTmpBuf[outBytes + 3] = 0;
-
-    ((XMLFormatter*)this)->fGTRef = new XMLByte[outBytes + 4];
-    memcpy(fGTRef, fTmpBuf, outBytes + 4);
-    count = fGTLen = outBytes;
-    return fGTRef;
-}
-
-const XMLByte* XMLFormatter::getLTRef(unsigned int & count)
-{
-    if (fLTRef)
-    {
-        count = fLTLen;
-        return fLTRef;
-    }
-
-    unsigned int charsEaten;
-    const unsigned int outBytes = fXCoder->transcodeTo
-    (
-        gLTRef
-        , XMLString::stringLen(gLTRef)
-        , fTmpBuf
-        , kTmpBufSize
-        , charsEaten
-        , XMLTranscoder::UnRep_Throw
-    );
-    fTmpBuf[outBytes] = 0; fTmpBuf[outBytes + 1] = 0;
-    fTmpBuf[outBytes + 2] = 0; fTmpBuf[outBytes + 3] = 0;
-
-    ((XMLFormatter*)this)->fLTRef = new XMLByte[outBytes + 4];
-    memcpy(fLTRef, fTmpBuf, outBytes + 4);
-    count = fLTLen = outBytes;
-    return fLTRef;
-}
-
-const XMLByte* XMLFormatter::getQuoteRef(unsigned int & count)
-{
-    if (fQuoteRef)
-    {
-        count = fQuoteLen;
-        return fQuoteRef;
-    }
-
-    unsigned int charsEaten;
-    const unsigned int outBytes = fXCoder->transcodeTo
-    (
-        gQuoteRef
-        , XMLString::stringLen(gQuoteRef)
-        , fTmpBuf
-        , kTmpBufSize
-        , charsEaten
-        , XMLTranscoder::UnRep_Throw
-    );
-    fTmpBuf[outBytes] = 0; fTmpBuf[outBytes + 1] = 0;
-    fTmpBuf[outBytes + 2] = 0; fTmpBuf[outBytes + 3] = 0;
-
-    ((XMLFormatter*)this)->fQuoteRef = new XMLByte[outBytes + 4];
-    memcpy(fQuoteRef, fTmpBuf, outBytes + 4);
-    count = fQuoteLen = outBytes;
-    return fQuoteRef;
-}
-
-
 void XMLFormatter::specialFormat(const  XMLCh* const    toFormat
                                 , const unsigned int    count
                                 , const EscapeFlags     escapeFlags)
@@ -610,12 +552,6 @@ void XMLFormatter::specialFormat(const  XMLCh* const    toFormat
     const XMLCh*    srcPtr = toFormat;
     const XMLCh*    endPtr = toFormat + count;
 
-    // Set up the common part of the buffer that we build char refs into
-    XMLCh tmpBuf[32];
-    tmpBuf[0] = chAmpersand;
-    tmpBuf[1] = chPound;
-    tmpBuf[2] = chLatin_x;
-
     while (srcPtr < endPtr)
     {
         const XMLCh* tmpPtr = srcPtr;
@@ -643,27 +579,14 @@ void XMLFormatter::specialFormat(const  XMLCh* const    toFormat
         }
          else
         {
-            //
+ 
             //  We hit something unrepresentable. So continue forward doing
             //  char refs until we hit something representable again or the
             //  end of input.
             //
             while (srcPtr < endPtr)
             {
-                // Build a char ref for the current char
-                XMLString::binToText(*srcPtr, &tmpBuf[3], 8, 16);
-                const unsigned int bufLen = XMLString::stringLen(tmpBuf);
-                tmpBuf[bufLen] = chSemiColon;
-                tmpBuf[bufLen+1] = chNull;
-
-                // And now call recursively back to our caller to format this
-                formatBuf
-                (
-                    tmpBuf
-                    , bufLen + 1
-                    , XMLFormatter::NoEscapes
-                    , XMLFormatter::UnRep_Fail
-                );
+                writeCharRef(*srcPtr);
 
                 // Move up the source pointer and break out if needed
                 srcPtr++;
@@ -674,3 +597,5 @@ void XMLFormatter::specialFormat(const  XMLCh* const    toFormat
     }
 }
 
+XERCES_CPP_NAMESPACE_END
+
