@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: ReaderMgr.cpp,v 1.4 2002/08/22 19:28:11 tng Exp $
+ * $Id: ReaderMgr.cpp,v 1.15 2003/05/18 14:02:04 knoaman Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -79,10 +79,12 @@
 #include <xercesc/internal/EndOfEntityException.hpp>
 #include <xercesc/internal/ReaderMgr.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  ReaderMgr: Constructors and Destructor
 // ---------------------------------------------------------------------------
-ReaderMgr::ReaderMgr() :
+ReaderMgr::ReaderMgr(MemoryManager* const manager) :
 
     fCurEntity(0)
     , fCurReader(0)
@@ -91,6 +93,9 @@ ReaderMgr::ReaderMgr() :
     , fNextReaderNum(1)
     , fReaderStack(0)
     , fThrowEOE(false)
+    , fXMLVersion(XMLReader::XMLV1_0)
+    , fStandardUriConformant(false)
+    , fMemoryManager(manager)
 {
 }
 
@@ -351,7 +356,7 @@ XMLCh ReaderMgr::skipUntilInOrWS(const XMLCh* const listToSkip)
         if (!nextCh)
             break;
 
-        if (XMLReader::isWhitespace(nextCh))
+        if (fCurReader->isWhitespace(nextCh))
             break;
 
         if (XMLString::indexOf(listToSkip, nextCh) != -1)
@@ -409,7 +414,8 @@ XMLReader* ReaderMgr::createReader( const   InputSource&        src
                                     , const bool                xmlDecl
                                     , const XMLReader::RefFrom  refFrom
                                     , const XMLReader::Types    type
-                                    , const XMLReader::Sources  source)
+                                    , const XMLReader::Sources  source
+                                    , const bool                calcSrcOfs)
 {
     //
     //  Ask the input source to create us an input stream. The particular
@@ -436,7 +442,7 @@ XMLReader* ReaderMgr::createReader( const   InputSource&        src
     try {
         if (src.getEncoding())
         {
-            retVal = new XMLReader
+            retVal = new (fMemoryManager) XMLReader
                 (
                 src.getPublicId()
                 , src.getSystemId()
@@ -445,11 +451,15 @@ XMLReader* ReaderMgr::createReader( const   InputSource&        src
                 , refFrom
                 , type
                 , source
+                , false
+                , calcSrcOfs
+                , fXMLVersion
+                , fMemoryManager
                 );
         }
         else
         {
-            retVal = new XMLReader
+            retVal = new (fMemoryManager) XMLReader
                 (
                 src.getPublicId()
                 , src.getSystemId()
@@ -457,6 +467,10 @@ XMLReader* ReaderMgr::createReader( const   InputSource&        src
                 , refFrom
                 , type
                 , source
+                , false
+                , calcSrcOfs
+                , fXMLVersion
+                , fMemoryManager
                 );
         }
     }
@@ -485,10 +499,11 @@ XMLReader* ReaderMgr::createReader( const   XMLCh* const        sysId
                                     , const XMLReader::RefFrom  refFrom
                                     , const XMLReader::Types    type
                                     , const XMLReader::Sources  source
-                                    ,       InputSource*&       srcToFill)
+                                    ,       InputSource*&       srcToFill
+                                    , const bool                calcSrcOfs)
 {
     // Create a buffer for expanding the system id
-    XMLBuffer expSysId;
+    XMLBuffer expSysId(1023, fMemoryManager);
 
     //
     //  Allow the entity handler to expand the system id if they choose
@@ -535,17 +550,25 @@ XMLReader* ReaderMgr::createReader( const   XMLCh* const        sysId
                     , XMLExcepts::URL_NoProtocolPresent
                 );
             }
-            srcToFill = new URLInputSource(urlTmp);
+            else {
+                if (fStandardUriConformant && urlTmp.hasInvalidChar())
+                    ThrowXML(MalformedURLException, XMLExcepts::URL_MalformedURL);
+                srcToFill = new (fMemoryManager) URLInputSource(urlTmp, fMemoryManager);
+            }
         }
 
-        catch(const MalformedURLException&)
+        catch(const MalformedURLException& e)
         {
-            // Its not a URL, so lets assume its a local file name.
-            srcToFill = new LocalFileInputSource
-            (
-                lastInfo.systemId
-                , expSysId.getRawBuffer()
-            );
+            // Its not a URL, so lets assume its a local file name if non-standard uri is allowed
+            if (!fStandardUriConformant)
+                srcToFill = new (fMemoryManager) LocalFileInputSource
+                (
+                    lastInfo.systemId
+                    , expSysId.getRawBuffer()
+                    , fMemoryManager
+                );
+            else
+                throw e;
         }
     }
 
@@ -563,6 +586,7 @@ XMLReader* ReaderMgr::createReader( const   XMLCh* const        sysId
         , refFrom
         , type
         , source
+        , calcSrcOfs
     );
 
     // Either way, we can release the input source now
@@ -585,10 +609,11 @@ XMLReader* ReaderMgr::createReader( const   XMLCh* const        baseURI
                                     , const XMLReader::RefFrom  refFrom
                                     , const XMLReader::Types    type
                                     , const XMLReader::Sources  source
-                                    ,       InputSource*&       srcToFill)
+                                    ,       InputSource*&       srcToFill
+                                    , const bool                calcSrcOfs)
 {
     // Create a buffer for expanding the system id
-    XMLBuffer expSysId;
+    XMLBuffer expSysId(1023, fMemoryManager);
 
     //
     //  Allow the entity handler to expand the system id if they choose
@@ -636,17 +661,25 @@ XMLReader* ReaderMgr::createReader( const   XMLCh* const        baseURI
                     , XMLExcepts::URL_NoProtocolPresent
                 );
             }
-            srcToFill = new URLInputSource(urlTmp);
+            else {
+                if (fStandardUriConformant && urlTmp.hasInvalidChar())
+                    ThrowXML(MalformedURLException, XMLExcepts::URL_MalformedURL);
+                srcToFill = new (fMemoryManager) URLInputSource(urlTmp, fMemoryManager);
+            }
         }
 
-        catch(const MalformedURLException&)
+        catch(const MalformedURLException& e)
         {
-            // Its not a URL, so lets assume its a local file name.
-            srcToFill = new LocalFileInputSource
-            (
-                lastInfo.systemId
-                , expSysId.getRawBuffer()
-            );
+            // Its not a URL, so lets assume its a local file name if non-standard uri is allowed
+            if (!fStandardUriConformant)
+                srcToFill = new (fMemoryManager) LocalFileInputSource
+                (
+                    lastInfo.systemId
+                    , expSysId.getRawBuffer()
+                    , fMemoryManager
+                );
+            else
+                throw e;
         }
     }
 
@@ -664,6 +697,7 @@ XMLReader* ReaderMgr::createReader( const   XMLCh* const        baseURI
         , refFrom
         , type
         , source
+        , calcSrcOfs
     );
 
     // Either way, we can release the input source now
@@ -685,7 +719,8 @@ ReaderMgr::createIntEntReader(  const   XMLCh* const        sysId
                                 , const XMLReader::Types    type
                                 , const XMLCh* const        dataBuf
                                 , const unsigned int        dataLen
-                                , const bool                copyBuf)
+                                , const bool                copyBuf
+                                , const bool                calcSrcOfs)
 {
     //
     //  This one is easy, we just create an input stream for the data and
@@ -695,25 +730,30 @@ ReaderMgr::createIntEntReader(  const   XMLCh* const        sysId
     //  as a 'do nothing' transcoder for the already internalized XMLCh
     //  data that makes up an internal entity.
     //
-    BinMemInputStream* newStream = new BinMemInputStream
+    BinMemInputStream* newStream = new (fMemoryManager) BinMemInputStream
                                    (
                                      (const XMLByte*)dataBuf
                                      , dataLen * sizeof(XMLCh)
                                      , copyBuf ? BinMemInputStream::BufOpt_Copy
-                                                 : BinMemInputStream::BufOpt_Reference
+                                               : BinMemInputStream::BufOpt_Reference
+                                     , fMemoryManager
                                    );
     if (!newStream)
         return 0;
 
-    XMLReader* retVal = new XMLReader
+    XMLReader* retVal = new (fMemoryManager) XMLReader
     (
         sysId
         , 0
         , newStream
-        , XMLUni::fgXMLChEncodingString
+        , XMLRecognizer::XERCES_XMLCH
         , refFrom
         , type
         , XMLReader::Source_Internal
+        , false
+        , calcSrcOfs
+        , fXMLVersion
+        , fMemoryManager
     );
 
     // If it failed for any reason, then return zero.
@@ -845,7 +885,7 @@ bool ReaderMgr::pushReader(         XMLReader* const        reader
             const XMLEntityDecl* curDecl = fEntityStack->elementAt(index);
             if (curDecl)
             {
-                if (!XMLString::compareString(theName, curDecl->getName()))
+                if (XMLString::equals(theName, curDecl->getName()))
                 {
                     // Oops, already there so delete reader and return
                     delete reader;
@@ -860,11 +900,11 @@ bool ReaderMgr::pushReader(         XMLReader* const        reader
     //  tell it it does own its elements.
     //
     if (!fReaderStack)
-        fReaderStack = new RefStackOf<XMLReader>(16, true);
+        fReaderStack = new (fMemoryManager) RefStackOf<XMLReader>(16, true, fMemoryManager);
 
     // And the entity stack, which does not own its elements
     if (!fEntityStack)
-        fEntityStack = new RefStackOf<XMLEntityDecl>(16, false);
+        fEntityStack = new (fMemoryManager) RefStackOf<XMLEntityDecl>(16, false, fMemoryManager);
 
     //
     //  Push the current reader and entity onto their respective stacks.
@@ -1074,3 +1114,5 @@ bool ReaderMgr::popReader()
     }
     return true;
 }
+
+XERCES_CPP_NAMESPACE_END
