@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 2002 The Apache Software Foundation.  All rights
+ * Copyright (c) 2002,2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,7 +59,7 @@
 *  handler with the scanner. In these handler methods, appropriate DOM nodes
 *  are created and added to the DOM tree.
 *
-* $Id: AbstractDOMParser.cpp,v 1.21 2002/08/19 14:39:57 tng Exp $
+* $Id: AbstractDOMParser.cpp,v 1.44 2003/05/21 21:18:14 peiyongz Exp $
 *
 */
 
@@ -68,13 +68,14 @@
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
+#include <xercesc/parsers/AbstractDOMParser.hpp>
+#include <xercesc/internal/XMLScannerResolver.hpp>
+#include <xercesc/internal/ElemStack.hpp>
 #include <xercesc/sax/EntityResolver.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/framework/XMLNotationDecl.hpp>
+#include <xercesc/framework/XMLValidator.hpp>
 #include <xercesc/util/IOException.hpp>
-#include <xercesc/internal/XMLScanner.hpp>
-#include <xercesc/validators/DTD/DTDValidator.hpp>
-#include <xercesc/parsers/AbstractDOMParser.hpp>
 #include <xercesc/dom/DOMImplementation.hpp>
 #include <xercesc/dom/DOMElement.hpp>
 #include <xercesc/dom/impl/DOMAttrImpl.hpp>
@@ -90,21 +91,26 @@
 #include <xercesc/dom/impl/DOMNotationImpl.hpp>
 #include <xercesc/dom/DOMNamedNodeMap.hpp>
 #include <xercesc/dom/DOMProcessingInstruction.hpp>
+#include <xercesc/dom/impl/DOMProcessingInstructionImpl.hpp>
 #include <xercesc/dom/impl/DOMNodeIDMap.hpp>
-
-
 #include <xercesc/validators/common/ContentSpecNode.hpp>
-#include <xercesc/validators/DTD/DTDAttDefList.hpp>
+#include <xercesc/validators/common/GrammarResolver.hpp>
+#include <xercesc/validators/schema/SchemaSymbols.hpp>
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 // ---------------------------------------------------------------------------
 //  AbstractDOMParser: Constructors and Destructor
 // ---------------------------------------------------------------------------
-AbstractDOMParser::AbstractDOMParser(XMLValidator* const valToAdopt) :
+AbstractDOMParser::AbstractDOMParser( XMLValidator* const valToAdopt
+                                    , MemoryManager* const manager) :
 
   fCreateEntityReferenceNodes(true)
 , fIncludeIgnorableWhitespace(true)
 , fWithinElement(false)
 , fParseInProgress(false)
+, fCreateCommentNodes(true)
+, fDocumentAdoptedByUser(false)
 , fScanner(0)
 , fCurrentParent(0)
 , fCurrentNode(0)
@@ -113,34 +119,67 @@ AbstractDOMParser::AbstractDOMParser(XMLValidator* const valToAdopt) :
 , fNodeStack(0)
 , fDocumentType(0)
 , fDocumentVector(0)
-, fCreateCommentNodes(true)
-, fDocumentAdoptedByUser(false)
+, fGrammarResolver(0)
+, fURIStringPool(0)
+, fValidator(valToAdopt)
+, fMemoryManager(manager)
+, fBufMgr(manager)
 , fInternalSubset(fBufMgr.bidOnBuffer())
 {
-    //
-    //  Create a scanner and tell it what validator to use. Then set us
-    //  as the document event handler so we can fill the DOM document.
-    //
-    fScanner = new XMLScanner(valToAdopt);
-    fScanner->setDocHandler(this);
-    fScanner->setDocTypeHandler(this);
-
-    fNodeStack = new ValueStackOf<DOMNode*>(64);
-    this->reset();
+    try
+    {
+        initialize();
+    }
+    catch(...)
+    {
+       cleanUp();
+       throw;
+    }
 }
 
 
 AbstractDOMParser::~AbstractDOMParser()
+{
+    cleanUp();
+}
+
+// ---------------------------------------------------------------------------
+//  AbstractDOMParser: Initialize/CleanUp methods
+// ---------------------------------------------------------------------------
+void AbstractDOMParser::initialize()
+{
+    //  Create grammar resolver and string pool to pass to the scanner
+    fGrammarResolver = new (fMemoryManager) GrammarResolver(fMemoryManager);
+    fURIStringPool = new (fMemoryManager) XMLStringPool(109, fMemoryManager);
+
+    //  Create a scanner and tell it what validator to use. Then set us
+    //  as the document event handler so we can fill the DOM document.
+    fScanner = XMLScannerResolver::getDefaultScanner(fValidator, fMemoryManager);
+    fScanner->setDocHandler(this);
+    fScanner->setDocTypeHandler(this);
+    fScanner->setGrammarResolver(fGrammarResolver);
+    fScanner->setURIStringPool(fURIStringPool);
+
+    fNodeStack = new (fMemoryManager) ValueStackOf<DOMNode*>(64, fMemoryManager);
+    this->reset();
+}
+
+void AbstractDOMParser::cleanUp()
 {
     if (fDocumentVector)
         delete fDocumentVector;
 
-    if (!fDocumentAdoptedByUser)
-        delete fDocument;
+    if (!fDocumentAdoptedByUser && fDocument)
+        fDocument->release();
+
     delete fNodeStack;
     delete fScanner;
-}
+    delete fGrammarResolver;
+    delete fURIStringPool;
 
+    if (fValidator)
+        delete fValidator;
+}
 
 // ---------------------------------------------------------------------------
 //  AbstractDOMParser: Utilities
@@ -151,7 +190,7 @@ void AbstractDOMParser::reset()
     if (fDocument && !fDocumentAdoptedByUser) {
         if (!fDocumentVector) {
             // allocate the vector if not exists yet
-            fDocumentVector  = new RefVectorOf<DOMDocumentImpl>(10, true) ;
+            fDocumentVector  = new (fMemoryManager) RefVectorOf<DOMDocumentImpl>(10, true, fMemoryManager) ;
         }
         fDocumentVector->addElement(fDocument);
     }
@@ -179,10 +218,16 @@ void AbstractDOMParser::resetPool()
     if (fDocumentVector)
         fDocumentVector->removeAllElements();
 
-    delete fDocument;
+    if (!fDocumentAdoptedByUser && fDocument)
+        fDocument->release();
+
     fDocument = 0;
 }
 
+bool AbstractDOMParser::isDocumentAdopted() const
+{
+    return fDocumentAdoptedByUser;
+}
 
 DOMDocument* AbstractDOMParser::adoptDocument()
 {
@@ -256,11 +301,26 @@ XMLCh* AbstractDOMParser::getExternalNoNamespaceSchemaLocation() const
     return fScanner->getExternalNoNamespaceSchemaLocation();
 }
 
+SecurityManager* AbstractDOMParser::getSecurityManager() const
+{
+    return fScanner->getSecurityManager();
+}
+
 bool AbstractDOMParser::getLoadExternalDTD() const
 {
     return fScanner->getLoadExternalDTD();
 }
 
+bool AbstractDOMParser::getCalculateSrcOfs() const
+{
+    return fScanner->getCalculateSrcOfs();
+}
+
+bool AbstractDOMParser::getStandardUriConformant() const
+{
+    return fScanner->getStandardUriConformant();
+}
+
 
 // ---------------------------------------------------------------------------
 //  AbstractDOMParser: Setter methods
@@ -318,11 +378,49 @@ void AbstractDOMParser::setExternalNoNamespaceSchemaLocation(const char* const n
     fScanner->setExternalNoNamespaceSchemaLocation(noNamespaceSchemaLocation);
 }
 
+void AbstractDOMParser::setSecurityManager(SecurityManager* const securityManager)
+{
+    // since this could impact various components, don't permit it to change
+    // during a parse
+    if (fParseInProgress)
+        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+
+    fScanner->setSecurityManager(securityManager);
+}
+
 void AbstractDOMParser::setLoadExternalDTD(const bool newState)
 {
     fScanner->setLoadExternalDTD(newState);
 }
 
+void AbstractDOMParser::setCalculateSrcOfs(const bool newState)
+{
+    fScanner->setCalculateSrcOfs(newState);
+}
+
+void AbstractDOMParser::setStandardUriConformant(const bool newState)
+{
+    fScanner->setStandardUriConformant(newState);
+}
+
+void AbstractDOMParser::useScanner(const XMLCh* const scannerName)
+{
+    XMLScanner* tempScanner = XMLScannerResolver::resolveScanner
+    (
+        scannerName
+        , fValidator
+        , fMemoryManager
+    );
+
+    if (tempScanner) {
+
+        tempScanner->setParseSettings(fScanner);
+        tempScanner->setGrammarResolver(fGrammarResolver);
+        tempScanner->setURIStringPool(fURIStringPool);
+        delete fScanner;
+        fScanner = tempScanner;
+    }
+}
 
 // ---------------------------------------------------------------------------
 //  AbstractDOMParser: Parsing methods
@@ -514,15 +612,82 @@ void AbstractDOMParser::docPI(  const   XMLCh* const    target
 
 void AbstractDOMParser::endEntityReference(const XMLEntityDecl& entDecl)
 {
-    if (fCreateEntityReferenceNodes == true)
-    {
-        if (fCurrentParent->getNodeType() == DOMNode::ENTITY_REFERENCE_NODE) {
-            DOMEntityReferenceImpl *erImpl = (DOMEntityReferenceImpl *) fCurrentParent;
-            erImpl->setReadOnly(true, true);
-        }
-        fCurrentParent = fNodeStack->pop();
-        fCurrentNode   = fCurrentParent;
+    DOMEntityReferenceImpl *erImpl = 0;
+    DOMNode* firstChild = 0;
+
+    if (fCurrentParent->getNodeType() == DOMNode::ENTITY_REFERENCE_NODE) {
+        erImpl = (DOMEntityReferenceImpl *) fCurrentParent;
+        firstChild = erImpl->getFirstChild();
     }
+
+    fCurrentParent = fNodeStack->pop();
+
+    if (!fCreateEntityReferenceNodes && erImpl && firstChild) {
+        DOMNode *kid, *next;
+        fCurrentNode   = fCurrentParent->getLastChild();
+
+        for (kid = firstChild; kid != 0; kid = next)
+        {
+            // If kid and fCurrentNode are both Text nodes (but _not_ CDATASection,
+            // which is a subclass of Text), they can be merged.
+            if (kid->getNodeType() == DOMNode::TEXT_NODE   &&
+                fCurrentNode &&
+                fCurrentNode->getNodeType() == DOMNode::TEXT_NODE )
+            {
+                ((DOMTextImpl *) fCurrentNode)->appendData(((DOMTextImpl *) kid)->getData());
+            }
+            else {
+                // append the child of erImpl to currentParent
+                fCurrentNode = kid->cloneNode(true);
+                fCurrentParent->appendChild(fCurrentNode);
+
+                if (erImpl->getBaseURI()) {
+                    /**
+                     * Record baseURI information for the Element (by adding xml:base attribute)
+                     * or for the ProcessingInstruction (by setting a baseURI field)
+                     */
+                    if (fCurrentNode->getNodeType() == DOMNode::ELEMENT_NODE) {
+                        // if an element already has xml:base attribute
+                        // do nothing
+                        const XMLCh baseString[] =
+                        {
+                            chLatin_b, chLatin_a, chLatin_s, chLatin_e, chNull
+                        };
+                        const XMLCh xmlBaseString[] =
+                        {
+                            chLatin_x, chLatin_m, chLatin_l, chColon, chLatin_b, chLatin_a, chLatin_s, chLatin_e, chNull
+                        };
+
+                        if (fScanner -> getDoNamespaces() && (((DOMElement*)fCurrentNode)->getAttributeNodeNS(DOMNodeImpl::getXmlURIString(), baseString) != 0)) {
+                            return;
+                        } else if (((DOMElement*)fCurrentNode)->getAttributeNode(xmlBaseString) != 0) {
+                            return;
+                        }
+
+                        // retrive the baseURI from the entity decl
+                        const XMLCh* baseURI = erImpl->getBaseURI();
+                        if (baseURI != 0 && !XMLString::equals(baseURI,fDocument->getDocumentURI())) {
+                            if (fScanner -> getDoNamespaces()) {
+                                ((DOMElement*)fCurrentNode)->setAttributeNS(DOMNodeImpl::getXmlURIString(), baseString, baseURI);
+                            } else {
+                                ((DOMElement*)fCurrentNode)->setAttribute(xmlBaseString, baseURI);
+                            }
+                        }
+                    }
+                    else if (fCurrentNode->getNodeType() == DOMNode::PROCESSING_INSTRUCTION_NODE) {
+                        ((DOMProcessingInstructionImpl*)fCurrentNode)->setBaseURI(erImpl->getBaseURI());
+                    }
+                }
+            }
+
+            next = kid->getNextSibling();
+        }
+    }
+    else
+        fCurrentNode   = fCurrentParent;
+
+    if (erImpl)
+        erImpl->setReadOnly(true, true);
 }
 
 
@@ -534,6 +699,10 @@ void AbstractDOMParser::endElement( const   XMLElementDecl&     elemDecl
     fCurrentNode   = fCurrentParent;
     fCurrentParent = fNodeStack->pop();
 
+    //validation is performed after the startElement event so we have to associate the info here
+    ((DOMElementImpl *)(fCurrentNode))->setTypeInfo(elemDecl.getDOMTypeInfoName(), elemDecl.getDOMTypeInfoUri());
+
+
     // If we've hit the end of content, clear the flag
     if (fNodeStack->empty())
         fWithinElement = false;
@@ -578,18 +747,17 @@ void AbstractDOMParser::resetDocument()
     //  from a previous use, in particular the DOM document itself.
     //
     this->reset();
-    fDocument = (DOMDocumentImpl *)DOMImplementation::getImplementation()->createDocument();
 }
 
 
 void AbstractDOMParser::startDocument()
 {
+    fDocument = (DOMDocumentImpl *)DOMImplementation::getImplementation()->createDocument();
     // Just set the document as the current parent and current node
     fCurrentParent = fDocument;
     fCurrentNode   = fDocument;
     // set DOM error checking off
     fDocument->setErrorChecking(false);
-
     fDocument->setDocumentURI(fScanner->getLocator()->getSystemId());
     fDocument->setActualEncoding(fScanner->getReaderMgr()->getCurrentEncodingStr());
 }
@@ -616,40 +784,55 @@ void AbstractDOMParser::startElement(const  XMLElementDecl&         elemDecl
 {
     DOMElement     *elem;
     DOMElementImpl *elemImpl;
+    static const XMLCh XMLNS[] = {
+    chLatin_x, chLatin_m, chLatin_l, chLatin_n, chLatin_s, chNull
+    };
+
+    static const XMLCh XSI[] = {
+    chLatin_x, chLatin_s, chLatin_i, chNull
+    };
+
+    //get the list for use in the loop
+    XMLAttDefList* defAttrs = 0;
+    if(elemDecl.hasAttDefs()) {
+        defAttrs = &elemDecl.getAttDefList();
+    }
 
     if (fScanner -> getDoNamespaces()) {    //DOM Level 2, doNamespaces on
 
-        XMLBufBid bbURI(&fBufMgr);
-        XMLBuffer& bufURI = bbURI.getBuffer();
-        XMLCh* namespaceURI = 0;
-
-        XMLBufBid elemQName(&fBufMgr);
+        const XMLCh* namespaceURI = 0;
 
         if (urlId != fScanner->getEmptyNamespaceId()) {  //TagName has a prefix
-            fScanner->getURIText(urlId, bufURI);   //get namespaceURI
-            namespaceURI = bufURI.getRawBuffer();
+
+            namespaceURI = fScanner->getURIText(urlId); //get namespaceURI
 
             if (elemPrefix && *elemPrefix) {
+
+                XMLBufBid elemQName(&fBufMgr);
+
                 elemQName.set(elemPrefix);
                 elemQName.append(chColon);
+                elemQName.append(elemDecl.getBaseName());
+                elem = createElementNSNode(namespaceURI, elemQName.getRawBuffer());
             }
+            else { 
+                elem = createElementNSNode(namespaceURI, elemDecl.getBaseName());
+            }
+        } 
+        else {
+            elem = createElementNSNode(namespaceURI, elemDecl.getBaseName());
         }
-        elemQName.append(elemDecl.getBaseName());
 
-        elem = createElementNSNode(namespaceURI, elemQName.getRawBuffer());
         elemImpl = (DOMElementImpl *) elem;
+
         for (unsigned int index = 0; index < attrCount; ++index) {
-            static const XMLCh XMLNS[] = {
-            chLatin_x, chLatin_m, chLatin_l, chLatin_n, chLatin_s, chNull
-            };
             const XMLAttr* oneAttrib = attrList.elementAt(index);
             unsigned int attrURIId = oneAttrib -> getURIId();
             namespaceURI = 0;
-            if (!XMLString::compareString(oneAttrib -> getName(), XMLNS))    //for xmlns=...
+            if (XMLString::equals(oneAttrib -> getName(), XMLNS))    //for xmlns=...
                 attrURIId = fScanner->getXMLNSNamespaceId();
             if (attrURIId != fScanner->getEmptyNamespaceId()) {  //TagName has a prefix
-                fScanner->getURIText(attrURIId, bufURI);   //get namespaceURI
-                namespaceURI = bufURI.getRawBuffer();
+                namespaceURI = fScanner->getURIText(attrURIId);   //get namespaceURI
             }
             //  revisit.  Optimize to init the named node map to the
             //            right size up front.
@@ -675,6 +858,33 @@ void AbstractDOMParser::startElement(const  XMLElementDecl&         elemDecl
             }
 
             attr->setSpecified(oneAttrib->getSpecified());
+
+            XMLAttDef *attDef = 0;
+            if(defAttrs != 0)
+                attDef = defAttrs->findAttDef(attrURIId, oneAttrib->getQName());
+
+            if(attDef != 0) {
+                attr->setTypeInfo(attDef->getDOMTypeInfoName(), attDef->getDOMTypeInfoUri());
+                attDef->reset();
+            }
+            else {
+                const XMLCh *name = oneAttrib->getName();
+                if (XMLString::equals(oneAttrib->getPrefix(), XSI)) {
+                    if(XMLString::equals(name, SchemaSymbols::fgXSI_TYPE)) {
+                        attr->setTypeInfo(SchemaSymbols::fgDT_QNAME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+                    }
+                    else if(XMLString::equals(name, SchemaSymbols::fgATT_NILL)) {
+                        attr->setTypeInfo(SchemaSymbols::fgDT_BOOLEAN, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+                    }
+                    else if(XMLString::equals(name, SchemaSymbols::fgXSI_NONAMESPACESCHEMALOCACTION)) {
+                        attr->setTypeInfo(SchemaSymbols::fgDT_ANYURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+                    }
+                }
+                else {
+                    //for normal ns attrs
+                    attr->setTypeInfo(SchemaSymbols::fgDT_ANYURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+                }
+            }
         }
     }
     else {    //DOM Level 1
@@ -701,13 +911,22 @@ void AbstractDOMParser::startElement(const  XMLElementDecl&         elemDecl
                 fDocument->fNodeIDMap->add(attr);
                 attr->fNode.isIdAttr(true);
             }
+
+            XMLAttDef *attDef = 0;
+            if(defAttrs != 0)
+                attDef = defAttrs->findAttDef(oneAttrib -> getURIId(), oneAttrib->getQName());
+
+            if(attDef != 0) {
+                attr->setTypeInfo(attDef->getDOMTypeInfoName(), attDef->getDOMTypeInfoUri());
+                attDef->reset();
+            }
+
         }
     }
 
     // set up the default attributes
-	if (elemDecl.hasAttDefs())
+	if (defAttrs != 0)
 	{		
-        XMLAttDefList* defAttrs = &elemDecl.getAttDefList();
         XMLAttDef* attr = 0;
 
         DOMAttrImpl * insertAttr = 0;
@@ -715,47 +934,69 @@ void AbstractDOMParser::startElement(const  XMLElementDecl&         elemDecl
         while (defAttrs->hasMoreElements())
         {
             attr = &defAttrs->nextElement();
-            const XMLAttDef::DefAttTypes defType = attr->getDefaultType();
 
+            const XMLAttDef::DefAttTypes defType = attr->getDefaultType();
             if ((defType == XMLAttDef::Default)
             ||  (defType == XMLAttDef::Fixed))
             {
-                if (attr->getValue() != 0)
+
+                if (fScanner->getDoNamespaces())
                 {
-                    if (fScanner->getDoNamespaces())
-                    {
-                        // DOM Level 2 wants all namespace declaration attributes
-                        // to be bound to "http://www.w3.org/2000/xmlns/"
-                        // So as long as the XML parser doesn't do it, it needs to
-                        // done here.
-                        const XMLCh* qualifiedName = attr->getFullName();
-                        XMLBufBid bbPrefixQName(&fBufMgr);
-                        XMLBufBid bbQName(&fBufMgr);
-                        XMLBuffer& prefixBuf = bbPrefixQName.getBuffer();
-                        XMLBuffer& nameBuf = bbQName.getBuffer();
-                        unsigned int uriId = fScanner->resolveQName(qualifiedName, nameBuf, prefixBuf, ElemStack::Mode_Attribute);
+                    // DOM Level 2 wants all namespace declaration attributes
+                    // to be bound to "http://www.w3.org/2000/xmlns/"
+                    // So as long as the XML parser doesn't do it, it needs to
+                    // done here.
+                    const XMLCh* qualifiedName = attr->getFullName();
+                    XMLBufBid bbPrefixQName(&fBufMgr);
+                    XMLBuffer& prefixBuf = bbPrefixQName.getBuffer();
+                    int colonPos = -1;
+                    unsigned int uriId = fScanner->resolveQName(qualifiedName, prefixBuf, ElemStack::Mode_Attribute, colonPos);
 
-                        insertAttr = (DOMAttrImpl *) fDocument->createAttributeNS(
-                           fScanner->getURIText(uriId),     // NameSpaceURI
-                           qualifiedName);   // qualified name
+                    const XMLCh* namespaceURI = 0;
+                    if (XMLString::equals(qualifiedName, XMLNS))    //for xmlns=...
+                        uriId = fScanner->getXMLNSNamespaceId();
+                    if (uriId != fScanner->getEmptyNamespaceId()) {  //TagName has a prefix
+                        namespaceURI = fScanner->getURIText(uriId);
+                    }
+
+                    insertAttr = (DOMAttrImpl *) fDocument->createAttributeNS(
+                                                                              namespaceURI,     // NameSpaceURI
+                                                                              qualifiedName);   // qualified name
 
-                        DOMNode* remAttr = elemImpl->setDefaultAttributeNodeNS(insertAttr);
-                        if (remAttr)
-                            remAttr->release();
-                    }
-                    else
+                    DOMAttr* remAttr = elemImpl->setDefaultAttributeNodeNS(insertAttr);
+                    if (remAttr)
+                        remAttr->release();
+
+                    if (attr->getValue() != 0)
                     {
-                        // Namespaces is turned off...
-                        insertAttr = (DOMAttrImpl *) fDocument->createAttribute(attr->getFullName());
-                        DOMNode* remAttr = elemImpl->setDefaultAttributeNode(insertAttr);
-                        if (remAttr)
-                            remAttr->release();
+                        insertAttr->setValue(attr->getValue());
+                        insertAttr->setSpecified(false);
                     }
 
-                    insertAttr->setValue(attr->getValue());
-                    insertAttr->setSpecified(false);
                 }
+                else
+                {
+                    // Namespaces is turned off...
+                    insertAttr = (DOMAttrImpl *) fDocument->createAttribute(attr->getFullName());
+
+                    DOMNode* remAttr = elemImpl->setDefaultAttributeNode(insertAttr);
+                    if (remAttr)
+                        remAttr->release();
+
+                    //need to do this before the get as otherwise we overwrite any value in the attr
+                    if (attr->getValue() != 0)
+                    {
+                        insertAttr->setValue(attr->getValue());
+                        insertAttr->setSpecified(false);
+                    }
+                }
+
+                insertAttr->setTypeInfo(attr->getDOMTypeInfoName(), attr->getDOMTypeInfoUri());
+
             }
+
+            insertAttr = 0;
+            attr->reset();
         }
     }
 
@@ -782,26 +1023,27 @@ void AbstractDOMParser::startEntityReference(const XMLEntityDecl& entDecl)
         entity->setActualEncoding(fScanner->getReaderMgr()->getCurrentEncodingStr());
     fCurrentEntity = entity;
 
+    DOMEntityReference *er = fDocument->createEntityReferenceByParser(entName);
+
+    //set the readOnly flag to false before appending node, will be reset in endEntityReference
+    DOMEntityReferenceImpl *erImpl = (DOMEntityReferenceImpl *) er;
+    erImpl->setReadOnly(false, true);
+
     if (fCreateEntityReferenceNodes == true)
     {
-        DOMEntityReference *er = fDocument->createEntityReference(entName);
-
-        //set the readOnly flag to false before appending node, will be reset in endEntityReference
-        DOMEntityReferenceImpl *erImpl = (DOMEntityReferenceImpl *) er;
-        erImpl->setReadOnly(false, true);
-
         fCurrentParent->appendChild(er);
-        fNodeStack->push(fCurrentParent);
-        fCurrentParent = er;
-        fCurrentNode = er;
-
-        // this entityRef needs to be stored in Entity map too.
-        // We'd decide later whether the entity nodes should be created by a
-        // separated method in parser or not. For now just stick it in if
-        // the ref nodes are created
-        if (entity)
-            entity->setEntityRef(er);
     }
+
+    fNodeStack->push(fCurrentParent);
+    fCurrentParent = er;
+    fCurrentNode = er;
+
+    // this entityRef needs to be stored in Entity map too.
+    // We'd decide later whether the entity nodes should be created by a
+    // separated method in parser or not. For now just stick it in if
+    // the ref nodes are created
+    if (entity)
+        entity->setEntityRef(er);
 }
 
 
@@ -810,7 +1052,7 @@ void AbstractDOMParser::XMLDecl(const   XMLCh* const version
                                 , const XMLCh* const standalone
                                 , const XMLCh* const actualEncStr)
 {
-    fDocument->setStandalone(!XMLString::compareString(XMLUni::fgYesString, standalone));
+    fDocument->setStandalone(XMLString::equals(XMLUni::fgYesString, standalone));
 
     fDocument->setVersion(version);
     fDocument->setEncoding(encoding);
@@ -1092,23 +1334,26 @@ void AbstractDOMParser::endAttList
                         XMLCh temp[1000];
 
                         if (index > 999)
-                            prefix = new XMLCh[index+1];
+                            prefix = (XMLCh*) fMemoryManager->allocate
+                            (
+                                (index + 1) * sizeof(XMLCh)
+                            );//new XMLCh[index+1];
                         else
                             prefix = temp;
 
                         XMLString::subString(prefix ,qualifiedName, 0, index);
 
-                        if (!XMLString::compareString(prefix,XMLNS))
+                        if (XMLString::equals(prefix,XMLNS))
                             buf.append(XMLUni::fgXMLNSURIName);
                         else
                             buf.append(XMLUni::fgXMLURIName);
 
                         if (index > 999)
-                            delete prefix;
+                            fMemoryManager->deallocate(prefix);//delete [] prefix;
                     }
                     else {
                         //   No prefix
-                        if (!XMLString::compareString(qualifiedName,XMLNS))
+                        if (XMLString::equals(qualifiedName,XMLNS))
                             buf.append(XMLUni::fgXMLNSURIName);
                     }
 
@@ -1142,8 +1387,9 @@ void AbstractDOMParser::endAttList
 void AbstractDOMParser::endIntSubset()
 {
     fDocumentType->setInternalSubset(fInternalSubset.getRawBuffer());
-    fBufMgr.releaseBuffer(fInternalSubset);
-    fDocumentType->intSubsetReading = false;
+    // the buffer shouldn't be released as it is reused in the next parse
+    // fBufMgr.releaseBuffer(fInternalSubset);
+    fDocumentType->fIntSubsetReading = false;
 }
 
 void AbstractDOMParser::endExtSubset()
@@ -1162,6 +1408,7 @@ void AbstractDOMParser::entityDecl
     entity->setPublicId(entityDecl.getPublicId());
     entity->setSystemId(entityDecl.getSystemId());
     entity->setNotationName(entityDecl.getNotationName());
+    entity->setBaseURI(entityDecl.getBaseURI());
 
     DOMEntityImpl *previousDef = (DOMEntityImpl *)
 	    fDocumentType->getEntities()->setNamedItem( entity );
@@ -1232,6 +1479,7 @@ void AbstractDOMParser::notationDecl
     DOMNotationImpl* notation = (DOMNotationImpl *)fDocument->createNotation(notDecl.getName());
     notation->setPublicId(notDecl.getPublicId());
     notation->setSystemId(notDecl.getSystemId());
+    notation->setBaseURI(notDecl.getBaseURI());
 
     DOMNode* rem = fDocumentType->getNotations()->setNamedItem( notation );
     if (rem)
@@ -1287,7 +1535,7 @@ void AbstractDOMParser::startAttList
 
 void AbstractDOMParser::startIntSubset()
 {
-	fDocumentType->intSubsetReading = true;
+	fDocumentType->fIntSubsetReading = true;
 }
 
 void AbstractDOMParser::startExtSubset()
@@ -1305,3 +1553,6 @@ void AbstractDOMParser::TextDecl
         fCurrentEntity->setEncoding(encodingStr);
     }
 }
+
+XERCES_CPP_NAMESPACE_END
+
