@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DOMBuilderImpl.hpp,v 1.9 2002/07/15 18:25:24 tng Exp $
+ * $Id: DOMBuilderImpl.hpp,v 1.13 2003/05/15 18:26:50 knoaman Exp $
  *
  */
 
@@ -65,6 +65,10 @@
 
 #include <xercesc/parsers/AbstractDOMParser.hpp>
 #include <xercesc/dom/DOMBuilder.hpp>
+#include <xercesc/util/XercesDefs.hpp>
+
+
+XERCES_CPP_NAMESPACE_BEGIN
 
  /**
   * Introduced in DOM Level 3
@@ -92,7 +96,11 @@ public :
       * @param valToAdopt Pointer to the validator instance to use. The
       *                   parser is responsible for freeing the memory.
       */
-    DOMBuilderImpl(XMLValidator* const valToAdopt = 0);
+    DOMBuilderImpl
+    (
+          XMLValidator* const  valToAdopt = 0
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager
+    );
 
     /**
       * Destructor
@@ -634,6 +642,13 @@ public :
       */
     virtual void resetCachedGrammarPool();
 
+    /**
+      * Returns the current src offset within the input source.
+      *
+      * @return offset within the input source
+      */
+    virtual unsigned int getSrcOffset() const;
+
     //@}
 
     // -----------------------------------------------------------------------
@@ -803,6 +818,11 @@ private :
     //
     //  fCharsetOverridesXMLEncoding
     //      Indicates if the "charset-overrides-xml-encoding" is set or not
+    //
+    //  fUserAdoptsDocument
+    //      The DOMDocument ownership has been transferred to application
+    //      If set to true, the parser does not own the document anymore
+    //      and thus will not release its memory.
     //-----------------------------------------------------------------------
     bool                        fAutoValidation;
     bool                        fValidation;
@@ -810,6 +830,7 @@ private :
     DOMErrorHandler*            fErrorHandler;
     DOMBuilderFilter*           fFilter;
     bool                        fCharsetOverridesXMLEncoding;
+    bool                        fUserAdoptsDocument;
 };
 
 
@@ -873,4 +894,6 @@ inline const DOMBuilderFilter* DOMBuilderImpl::getFilter() const
 }
 
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
