@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,6 +56,59 @@
 
 /*
  * $Log: SAXParser.cpp,v $
+ * Revision 1.19  2003/05/18 14:02:05  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.18  2003/05/16 21:36:59  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.17  2003/05/15 18:26:50  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.16  2003/04/17 21:58:50  neilg
+ * Adding a new property,
+ * http://apache.org/xml/properties/security-manager, with
+ * appropriate getSecurityManager/setSecurityManager methods on DOM
+ * and SAX parsers.  Also adding a new SecurityManager class.
+ *
+ * The purpose of these modifications is to permit applications a
+ * means to have the parser reject documents whose processing would
+ * otherwise consume large amounts of system resources.  Malicious
+ * use of such documents could be used to launch a denial-of-service
+ * attack against a system running the parser.  Initially, the
+ * SecurityManager only knows about attacks that can result from
+ * exponential entity expansion; this is the only known attack that
+ * involves processing a single XML document.  Other, simlar attacks
+ * can be launched if arbitrary schemas may be parsed; there already
+ * exist means (via use of the EntityResolver interface) by which
+ * applications can deny processing of untrusted schemas.  In future,
+ * the SecurityManager will be expanded to take these other exploits
+ * into account.
+ *
+ * add security manager
+ * 
+ * Revision 1.15  2003/02/04 19:27:43  knoaman
+ * Performance: use global buffer to eliminate repetitive memory creation/deletion.
+ *
+ * Revision 1.14  2003/01/09 19:07:08  tng
+ * [Bug 15802] Add "const" qualifier to getURIText.
+ *
+ * Revision 1.13  2003/01/03 20:09:36  tng
+ * New feature StandardUriConformant to force strict standard uri conformance.
+ *
+ * Revision 1.12  2002/12/27 16:16:51  knoaman
+ * Set scanner options and handlers.
+ *
+ * Revision 1.11  2002/12/23 15:23:18  knoaman
+ * Added a public api to various parsers to return the src offset within the input
+ * source.
+ *
+ * Revision 1.10  2002/12/04 01:57:09  knoaman
+ * Scanner re-organization.
+ *
+ * Revision 1.9  2002/11/04 14:57:03  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.8  2002/08/14 15:20:38  knoaman
  * [Bug 3111] Problem with LexicalHandler::startDTD() and LexicalHandler::endDTD().
  *
@@ -180,48 +233,92 @@
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
+#include <xercesc/parsers/SAXParser.hpp>
+#include <xercesc/internal/XMLScannerResolver.hpp>
+#include <xercesc/framework/XMLValidator.hpp>
 #include <xercesc/util/IOException.hpp>
 #include <xercesc/sax/DocumentHandler.hpp>
 #include <xercesc/sax/DTDHandler.hpp>
 #include <xercesc/sax/ErrorHandler.hpp>
 #include <xercesc/sax/EntityResolver.hpp>
 #include <xercesc/sax/SAXParseException.hpp>
-#include <xercesc/internal/XMLScanner.hpp>
-#include <xercesc/parsers/SAXParser.hpp>
+#include <xercesc/validators/common/GrammarResolver.hpp>
 #include <string.h>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 
 // ---------------------------------------------------------------------------
 //  SAXParser: Constructors and Destructor
 // ---------------------------------------------------------------------------
-SAXParser::SAXParser(XMLValidator* const valToAdopt) :
+SAXParser::SAXParser( XMLValidator* const  valToAdopt
+                    , MemoryManager* const manager) :
 
-    fDocHandler(0)
-    , fDTDHandler(0)
+    fParseInProgress(false)
     , fElemDepth(0)
+    , fAdvDHCount(0)
+    , fAdvDHListSize(32)
+    , fDocHandler(0)
+    , fDTDHandler(0)
     , fEntityResolver(0)
     , fErrorHandler(0)
-    , fAdvDHCount(0)
     , fAdvDHList(0)
-    , fAdvDHListSize(32)
-    , fParseInProgress(false)
     , fScanner(0)
+    , fGrammarResolver(0)
+    , fURIStringPool(0)
+    , fValidator(valToAdopt)
+    , fMemoryManager(manager)
+    , fElemQNameBuf(1023, manager)
 {
-    // Create our scanner and tell it what validator to use
-    fScanner = new XMLScanner(valToAdopt);
-
-    // Create the initial advanced handler list array and zero it out
-    fAdvDHList = new XMLDocumentHandler*[fAdvDHListSize];
-    memset(fAdvDHList, 0, sizeof(void*) * fAdvDHListSize);
+    try
+    {
+        initialize();
+    }
+    catch(...)
+    {
+        cleanUp();
+        throw;
+    }
 }
 
 
 SAXParser::~SAXParser()
 {
-    delete [] fAdvDHList;
-    delete fScanner;
+    cleanUp();
 }
 
+// ---------------------------------------------------------------------------
+//  SAXParser: Initialize/CleanUp methods
+// ---------------------------------------------------------------------------
+void SAXParser::initialize()
+{
+    // Create grammar resolver and string pool to pass to scanner
+    fGrammarResolver = new (fMemoryManager) GrammarResolver(fMemoryManager);
+    fURIStringPool = new (fMemoryManager) XMLStringPool(109, fMemoryManager);
+
+    // Create our scanner and tell it what validator to use
+    fScanner = XMLScannerResolver::getDefaultScanner(fValidator,fMemoryManager);
+    fScanner->setGrammarResolver(fGrammarResolver);
+    fScanner->setURIStringPool(fURIStringPool);
+
+    // Create the initial advanced handler list array and zero it out
+    fAdvDHList = (XMLDocumentHandler**) fMemoryManager->allocate
+    (
+        fAdvDHListSize * sizeof(XMLDocumentHandler*)
+    );//new XMLDocumentHandler*[fAdvDHListSize];
+    memset(fAdvDHList, 0, sizeof(void*) * fAdvDHListSize);
+}
+
+void SAXParser::cleanUp()
+{
+    fMemoryManager->deallocate(fAdvDHList);//delete [] fAdvDHList;
+    delete fScanner;
+    delete fGrammarResolver;
+    delete fURIStringPool;
+
+    if (fValidator)
+        delete fValidator;
+}
 
 
 // ---------------------------------------------------------------------------
@@ -234,7 +331,10 @@ void SAXParser::installAdvDocHandler(XMLDocumentHandler* const toInstall)
     {
         // Calc a new size and allocate the new temp buffer
         const unsigned int newSize = (unsigned int)(fAdvDHListSize * 1.5);
-        XMLDocumentHandler** newList = new XMLDocumentHandler*[newSize];
+        XMLDocumentHandler** newList = (XMLDocumentHandler**) fMemoryManager->allocate
+        (
+            newSize * sizeof(XMLDocumentHandler*)
+        );//new XMLDocumentHandler*[newSize];
 
         // Copy over the old data to the new list and zero out the rest
         memcpy(newList, fAdvDHList, sizeof(void*) * fAdvDHListSize);
@@ -246,7 +346,7 @@ void SAXParser::installAdvDocHandler(XMLDocumentHandler* const toInstall)
         );
 
         // And now clean up the old array and store the new stuff
-        delete [] fAdvDHList;
+        fMemoryManager->deallocate(fAdvDHList);//delete [] fAdvDHList;
         fAdvDHList = newList;
         fAdvDHListSize = newSize;
     }
@@ -376,6 +476,11 @@ XMLCh* SAXParser::getExternalNoNamespaceSchemaLocation() const
     return fScanner->getExternalNoNamespaceSchemaLocation();
 }
 
+SecurityManager* SAXParser::getSecurityManager() const
+{
+    return fScanner->getSecurityManager();
+}
+
 bool SAXParser::getLoadExternalDTD() const
 {
     return fScanner->getLoadExternalDTD();
@@ -391,9 +496,19 @@ bool SAXParser::isUsingCachedGrammarInParse() const
     return fScanner->isUsingCachedGrammarInParse();
 }
 
+bool SAXParser::getCalculateSrcOfs() const
+{
+    return fScanner->getCalculateSrcOfs();
+}
+
+bool SAXParser::getStandardUriConformant() const
+{
+    return fScanner->getStandardUriConformant();
+}
+
 Grammar* SAXParser::getGrammar(const XMLCh* const nameSpaceKey)
 {
-    return fScanner->getGrammar(nameSpaceKey);
+    return fGrammarResolver->getGrammar(nameSpaceKey);
 }
 
 Grammar* SAXParser::getRootGrammar()
@@ -401,11 +516,16 @@ Grammar* SAXParser::getRootGrammar()
     return fScanner->getRootGrammar();
 }
 
-const XMLCh* SAXParser::getURIText(unsigned int uriId)
+const XMLCh* SAXParser::getURIText(unsigned int uriId) const
 {
     return fScanner->getURIText(uriId);
 }
 
+unsigned int SAXParser::getSrcOffset() const
+{
+    return fScanner->getSrcOffset();
+}
+
 // ---------------------------------------------------------------------------
 //  SAXParser: Setter methods
 // ---------------------------------------------------------------------------
@@ -465,6 +585,16 @@ void SAXParser::setExternalNoNamespaceSchemaLocation(const char* const noNamespa
     fScanner->setExternalNoNamespaceSchemaLocation(noNamespaceSchemaLocation);
 }
 
+void SAXParser::setSecurityManager(SecurityManager* const securityManager)
+{
+    // since this could impact various components, don't permit it to change
+    // during a parse
+    if (fParseInProgress)
+        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+
+    fScanner->setSecurityManager(securityManager);
+}
+
 void SAXParser::setLoadExternalDTD(const bool newState)
 {
     fScanner->setLoadExternalDTD(newState);
@@ -484,6 +614,35 @@ void SAXParser::useCachedGrammarInParse(const bool newState)
         fScanner->useCachedGrammarInParse(newState);
 }
 
+void SAXParser::setCalculateSrcOfs(const bool newState)
+{
+    fScanner->setCalculateSrcOfs(newState);
+}
+
+void SAXParser::setStandardUriConformant(const bool newState)
+{
+    fScanner->setStandardUriConformant(newState);
+}
+
+void SAXParser::useScanner(const XMLCh* const scannerName)
+{
+    XMLScanner* tempScanner = XMLScannerResolver::resolveScanner
+    (
+        scannerName
+        , fValidator
+        , fMemoryManager
+    );
+
+    if (tempScanner) {
+
+        tempScanner->setParseSettings(fScanner);
+        tempScanner->setGrammarResolver(fGrammarResolver);
+        tempScanner->setURIStringPool(fURIStringPool);
+        delete fScanner;
+        fScanner = tempScanner;
+    }
+}
+
 
 // ---------------------------------------------------------------------------
 //  SAXParser: Overrides of the SAX Parser interface
@@ -758,13 +917,17 @@ void SAXParser::endElement( const   XMLElementDecl& elemDecl
     // Just map to the SAX document handler
     if (fDocHandler) {
         if (fScanner->getDoNamespaces()) {
-            XMLBuffer elemQName;
+
             if (elemPrefix && *elemPrefix) {
-                elemQName.set(elemPrefix);
-                elemQName.append(chColon);
+
+                fElemQNameBuf.set(elemPrefix);
+                fElemQNameBuf.append(chColon);
+                fElemQNameBuf.append(elemDecl.getBaseName());
+                fDocHandler->endElement(fElemQNameBuf.getRawBuffer());
+            }
+            else {
+                fDocHandler->endElement(elemDecl.getBaseName());
             }
-            elemQName.append(elemDecl.getBaseName());
-            fDocHandler->endElement(elemQName.getRawBuffer());
         }
         else
             fDocHandler->endElement(elemDecl.getFullName());
@@ -872,18 +1035,26 @@ startElement(   const   XMLElementDecl&         elemDecl
     {
         fAttrList.setVector(&attrList, attrCount);
         if (fScanner->getDoNamespaces()) {
-            XMLBuffer elemQName;
+
             if (elemPrefix && *elemPrefix) {
-                elemQName.set(elemPrefix);
-                elemQName.append(chColon);
+
+                fElemQNameBuf.set(elemPrefix);
+                fElemQNameBuf.append(chColon);
+                fElemQNameBuf.append(elemDecl.getBaseName());
+                fDocHandler->startElement(fElemQNameBuf.getRawBuffer(), fAttrList);
+
+                // If its empty, send the end tag event now
+                if (isEmpty)
+                    fDocHandler->endElement(fElemQNameBuf.getRawBuffer());
             }
-            elemQName.append(elemDecl.getBaseName());
+            else {
 
-            fDocHandler->startElement(elemQName.getRawBuffer(), fAttrList);
+                fDocHandler->startElement(elemDecl.getBaseName(), fAttrList);
 
-            // If its empty, send the end tag event now
-            if (isEmpty)
-                fDocHandler->endElement(elemQName.getRawBuffer());
+                // If its empty, send the end tag event now
+                if (isEmpty)
+                    fDocHandler->endElement(elemDecl.getBaseName());
+            }
         }
         else {
             fDocHandler->startElement(elemDecl.getFullName(), fAttrList);
@@ -1253,5 +1424,8 @@ Grammar* SAXParser::loadGrammar(const InputSource& source,
 
 void SAXParser::resetCachedGrammarPool()
 {
-    fScanner->resetCachedGrammarPool();
+    fGrammarResolver->resetCachedGrammar();
 }
+
+XERCES_CPP_NAMESPACE_END
+
