@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: XercesDOMParser.hpp,v 1.10 2002/07/11 18:27:03 knoaman Exp $
+ * $Id: XercesDOMParser.hpp,v 1.14 2003/05/15 18:26:50 knoaman Exp $
  *
  */
 
@@ -65,6 +65,9 @@
 
 #include <xercesc/parsers/AbstractDOMParser.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 class EntityResolver;
 class ErrorHandler;
 class Grammar;
@@ -98,7 +101,11 @@ public :
       * @param valToAdopt Pointer to the validator instance to use. The
       *                   parser is responsible for freeing the memory.
       */
-    XercesDOMParser(XMLValidator* const valToAdopt = 0);
+    XercesDOMParser
+    (
+          XMLValidator* const  valToAdopt = 0
+        , MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager
+    );
 
     /**
       * Destructor
@@ -195,8 +202,15 @@ public :
      *
      * @param uriId id of the string in the URI string pool.
      * @return URI string corresponding to the URI id.
-     */    
-    const XMLCh* getURIText(unsigned int uriId);
+     */
+    const XMLCh* getURIText(unsigned int uriId) const;
+
+    /**
+     * Returns the current src offset within the input source.
+     *
+     * @return offset within the input source
+     */
+    unsigned int getSrcOffset() const;
 
     //@}
 
@@ -632,4 +646,6 @@ inline const EntityResolver* XercesDOMParser::getEntityResolver() const
 }
 
 
+XERCES_CPP_NAMESPACE_END
+
 #endif
