@@ -1,37 +1,37 @@
 /*
  * The Apache Software License, Version 1.1
- * 
+ *
  * Copyright (c) 2001 The Apache Software Foundation.  All rights
  * reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- * 
+ *    notice, this list of conditions and the following disclaimer.
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
- * 
+ *
  * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
+ *    if any, must include the following acknowledgment:
  *       "This product includes software developed by the
  *        Apache Software Foundation (http://www.apache.org/)."
  *    Alternately, this acknowledgment may appear in the software itself,
  *    if and wherever such third-party acknowledgments normally appear.
- * 
+ *
  * 4. The names "Xerces" and "Apache Software Foundation" must
  *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
+ *    software without prior written permission. For written
  *    permission, please contact apache\@apache.org.
- * 
+ *
  * 5. Products derived from this software may not be called "Apache",
  *    nor may "Apache" appear in their name, without prior written
  *    permission of the Apache Software Foundation.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
  * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
  * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -45,7 +45,7 @@
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  * ====================================================================
- * 
+ *
  * This software consists of voluntary contributions made by many
  * individuals on behalf of the Apache Software Foundation, and was
  * originally based on software copyright (c) 2001, International
@@ -56,6 +56,21 @@
 
 /*
  * $Log: Base64.cpp,v $
+ * Revision 1.10  2003/05/20 22:10:02  peiyongz
+ * To differentiate external/internal memory
+ *
+ * Revision 1.8  2003/01/27 21:15:56  peiyongz
+ * only zero or one space allowed in between B64 character.
+ *
+ * Revision 1.7  2002/12/20 22:10:20  tng
+ * XML 1.1
+ *
+ * Revision 1.6  2002/11/25 18:14:35  peiyongz
+ * Schema Errata: E2-9 Base64
+ *
+ * Revision 1.5  2002/11/04 15:22:03  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.4  2002/03/19 17:01:20  peiyongz
  * Fix to Bug#7243 Base64 encoding is not working.
  *
@@ -94,13 +109,14 @@
 #include <xercesc/util/Base64.hpp>
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/Janitor.hpp>
-#include <xercesc/util/PlatformUtils.hpp>
-#include <xercesc/util/TransService.hpp>
+#include <xercesc/internal/XMLReader.hpp>
+
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  constants
 // ---------------------------------------------------------------------------
-static const int BASELENGTH = 255;       
+static const int BASELENGTH = 255;
 static const int FOURBYTE   = 4;
 
 // ---------------------------------------------------------------------------
@@ -113,14 +129,14 @@ const XMLByte Base64::base64Alphabet[] = {
     chLatin_F, chLatin_G, chLatin_H, chLatin_I, chLatin_J,
     chLatin_K, chLatin_L, chLatin_M, chLatin_N, chLatin_O,
     chLatin_P, chLatin_Q, chLatin_R, chLatin_S, chLatin_T,
-    chLatin_U, chLatin_V, chLatin_W, chLatin_X, chLatin_Y, chLatin_Z, 
-	chLatin_a, chLatin_b, chLatin_c, chLatin_d, chLatin_e, 
-	chLatin_f, chLatin_g, chLatin_h, chLatin_i, chLatin_j, 
-	chLatin_k, chLatin_l, chLatin_m, chLatin_n, chLatin_o, 
-	chLatin_p, chLatin_q, chLatin_r, chLatin_s, chLatin_t, 
-	chLatin_u, chLatin_v, chLatin_w, chLatin_x, chLatin_y, chLatin_z, 
-	chDigit_0, chDigit_1, chDigit_2, chDigit_3, chDigit_4, 
-	chDigit_5, chDigit_6, chDigit_7, chDigit_8, chDigit_9, 
+    chLatin_U, chLatin_V, chLatin_W, chLatin_X, chLatin_Y, chLatin_Z,
+	chLatin_a, chLatin_b, chLatin_c, chLatin_d, chLatin_e,
+	chLatin_f, chLatin_g, chLatin_h, chLatin_i, chLatin_j,
+	chLatin_k, chLatin_l, chLatin_m, chLatin_n, chLatin_o,
+	chLatin_p, chLatin_q, chLatin_r, chLatin_s, chLatin_t,
+	chLatin_u, chLatin_v, chLatin_w, chLatin_x, chLatin_y, chLatin_z,
+	chDigit_0, chDigit_1, chDigit_2, chDigit_3, chDigit_4,
+	chDigit_5, chDigit_6, chDigit_7, chDigit_8, chDigit_9,
 	chPlus, chForwardSlash, chNull
 };
 
@@ -130,17 +146,78 @@ const XMLByte Base64::base64Padding = chEqual;
 
 bool Base64::isInitialized = false;
 
+/***
+ *
+ * Memory Management Issue:
+ *
+ * . For memory allocated for result returned to caller (external memory), 
+ *   the plugged memory manager is used if it is provided, otherwise global 
+ *   new used to retain the pre-memory-manager behaviour.
+ *
+ * . For memory allocated for temperary buffer (internal memory), 
+ *   XMLPlatformUtils::fgMemoryManager is used.
+ *
+ */
+
+static inline void* getExternalMemory(MemoryManager* const allocator
+                                    , unsigned int const   sizeToAllocate)
+{
+   return allocator ? allocator->allocate(sizeToAllocate)
+       : ::operator new(sizeToAllocate);
+}
+
+static inline void* getInternalMemory(unsigned int const   sizeToAllocate)
+{
+    return XMLPlatformUtils::fgMemoryManager->allocate(sizeToAllocate);
+}
+
+/***
+ * internal memory is deallocated by janitorArray
+ */ 
+static inline void returnExternalMemory(MemoryManager* const allocator
+                                      , void*                buffer)
+{
+    allocator ? allocator->deallocate(buffer)
+        : ::operator delete(buffer);
+}
+
+/**
+ *     E2-9
+ *
+ *     Canonical-base64Binary ::= (B64line* B64lastline)?
+ *
+ *      B64line ::= B64x15 B64x15 B64x15 B64x15 B64x15 B64 #xA
+ *                  76 Base64 characters followed by newline
+ *
+ *      B64x15  ::= B64 B64 B64 B64 B64
+ *                  B64 B64 B64 B64 B64
+ *                  B64 B64 B64 B64 B64
+ *
+ *      B64lastline ::= B64x4? B64x4? B64x4? B64x4?
+ *                      B64x4? B64x4? B64x4? B64x4?
+ *                      B64x4? B64x4? B64x4? B64x4?
+ *                      B64x4? B64x4? B64x4? B64x4?
+ *                      B64x4? B64x4?
+ *                      (B64x4 | (B64 B64 B16 '=') | (B64 B04 '=='))
+ *                      #xA
+ *
+ *      B64x4   ::= B64 B64 B64 B64
+ *      B04     ::= [AQgw]
+ *      B16     ::= [AEIMQUYcgkosw048]
+*/
+
 // number of quadruplets per one line ( must be >1 and <19 )
 const unsigned int Base64::quadsPerLine = 15;
 
-XMLByte* Base64::encode(const XMLByte* const inputData,
-                        const unsigned int   inputLength,
-                        unsigned int*        outputLength)
+XMLByte* Base64::encode(const XMLByte* const inputData
+                      , const unsigned int   inputLength
+                      , unsigned int*        outputLength
+                      , MemoryManager* const memMgr)
 {
     if (!isInitialized)
         init();
 
-    if (!inputData)
+    if (!inputData || !outputLength)
         return 0;
 
     int quadrupletCount = ( inputLength + 2 ) / 3;
@@ -157,7 +234,7 @@ XMLByte* Base64::encode(const XMLByte* const inputData,
 
     unsigned int inputIndex = 0;
     unsigned int outputIndex = 0;
-    XMLByte *encodedData = new XMLByte[ quadrupletCount*FOURBYTE + lineCount + 1 ];
+    XMLByte *encodedData = (XMLByte*) getExternalMemory(memMgr, (quadrupletCount*FOURBYTE+lineCount+1) * sizeof(XMLByte));
 
     //
     // Process all quadruplet(s) except the last
@@ -223,8 +300,7 @@ XMLByte* Base64::encode(const XMLByte* const inputData,
     // write out end of string
     encodedData[ outputIndex ] = 0;
 
-    if( outputLength != 0 )
-      (*outputLength) = outputIndex;
+    *outputLength = outputIndex;
 
     return encodedData;
 }
@@ -235,14 +311,14 @@ XMLByte* Base64::encode(const XMLByte* const inputData,
 //
 // In previous version, we use XMLString::strLen(decodedData)
 // to get the length, this will fail for test case containing
-// consequtive "A", such "AAFF", or "ab56AA56". Instead of 
+// consequtive "A", such "AAFF", or "ab56AA56". Instead of
 // returning 3/6, we have 0 and 3, indicating that "AA", after
 // decoded, is interpreted as <null> by the strLen().
 //
 // Since decode() has track of length of the decoded data, we
 // will get this length from decode(), instead of strLen().
 //
-int Base64::getDataLength( const XMLCh* const inputData )
+int Base64::getDataLength(const XMLCh* const inputData)
 {
     unsigned int    retLen = 0;
     XMLCh* decodedData = decode(inputData, &retLen);
@@ -251,7 +327,7 @@ int Base64::getDataLength( const XMLCh* const inputData )
         return -1;
     else
     {
-        delete[] decodedData;
+        returnExternalMemory(0, decodedData);
         return retLen;
     }
 }
@@ -260,12 +336,29 @@ int Base64::getDataLength( const XMLCh* const inputData )
 // return 0(null) if invalid data found.
 // return the buffer containning decoded data otherwise
 // the caller is responsible for the de-allocation of the
-// buffer returned. 
+// buffer returned.
 //
 // temporary data, rawInputData, is ALWAYS released by this function.
 //
-XMLByte* Base64::decode(const XMLByte* const inputData,
-                        unsigned int*        outputLength)
+
+/***
+ *     E2-9
+ *
+ *     Base64Binary ::= S? B64quartet* B64final?
+ *
+ *     B64quartet   ::= B64 S? B64 S? B64 S? B64 S?
+ *
+ *     B64final     ::= B64 S? B04 S? '=' S? '=' S?
+ *                    | B64 S? B64 S? B16 S? '=' S?
+ *
+ *     B04          ::= [AQgw]
+ *     B16          ::= [AEIMQUYcgkosw048]
+ *     B64          ::= [A-Za-z0-9+/]
+*/
+
+XMLByte* Base64::decode(const XMLByte* const inputData
+                      , unsigned int*        decodedLength
+                      , MemoryManager* const memMgr)
 {
     if (!isInitialized)
         init();
@@ -274,28 +367,38 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
         return 0;
 
     //
-    // remove all whitespaces from the base64Data
+    // remove all XML whitespaces from the base64Data
     //
-
     int inputLength = XMLString::stringLen( (const char* const)inputData );
-    XMLByte* rawInputData = new XMLByte[ inputLength + 1 ];
-    ArrayJanitor<XMLByte> jan(rawInputData);
+    XMLByte* rawInputData = (XMLByte*) getInternalMemory((inputLength+1) * sizeof(XMLByte));
+    ArrayJanitor<XMLByte> jan(rawInputData, XMLPlatformUtils::fgMemoryManager);
 
     int inputIndex = 0;
     int rawInputLength = 0;
+    bool inWhiteSpace = false;
+
     while ( inputIndex < inputLength )
     {
-//        if( !isspace( inputData[ inputIndex ] ))
-        if (!XMLPlatformUtils::fgTransService->isSpace(inputData[inputIndex]))
+        if (!XMLChar1_0::isWhitespace(inputData[inputIndex]))
+        {
             rawInputData[ rawInputLength++ ] = inputData[ inputIndex ];
+            inWhiteSpace = false;
+        }
+        else
+        {
+            if (inWhiteSpace)
+                return 0; // more than 1 whitespaces encountered
+            else
+                inWhiteSpace = true;
+        }
 
         inputIndex++;
     }
     rawInputData[ rawInputLength ] = 0;
 
     // the length of raw data should be divisible by four
-    if (( rawInputLength % FOURBYTE ) != 0 ) 
-        return 0;  
+    if (( rawInputLength % FOURBYTE ) != 0 )
+        return 0;
 
     int quadrupletCount = rawInputLength / FOURBYTE;
     if ( quadrupletCount == 0 )
@@ -309,7 +412,7 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
 
     int rawInputIndex  = 0;
     int outputIndex    = 0;
-    XMLByte *decodedData = new XMLByte[ quadrupletCount*3 + 1 ];
+    XMLByte *decodedData = (XMLByte*) getExternalMemory(memMgr, (quadrupletCount*3+1) * sizeof(XMLByte));
 
     //
     // Process all quadruplet(s) except the last
@@ -324,7 +427,7 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
             !isData( (d4 = rawInputData[ rawInputIndex++ ]) ))
         {
             // if found "no data" just return NULL
-            delete[] decodedData;
+            returnExternalMemory(memMgr, decodedData);
             return 0;
         }
 
@@ -347,53 +450,53 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
         !isData( (d2 = rawInputData[ rawInputIndex++ ]) ))
     {
         // if found "no data" just return NULL
-        delete[] decodedData;
+        returnExternalMemory(memMgr, decodedData);
         return 0;
     }
 
-    b1 = base64Inverse[ d1 ]; 
+    b1 = base64Inverse[ d1 ];
     b2 = base64Inverse[ d2 ];
 
     // try to process last two octets
     d3 = rawInputData[ rawInputIndex++ ];
     d4 = rawInputData[ rawInputIndex++ ];
 
-    if (!isData( d3 ) || !isData( d4 )) 
+    if (!isData( d3 ) || !isData( d4 ))
     {
         // check if last two are PAD characters
-        if (isPad( d3 ) && isPad( d4 )) 
+        if (isPad( d3 ) && isPad( d4 ))
         {
             // two PAD e.g. 3c==
             if ((b2 & 0xf) != 0) // last 4 bits should be zero
             {
-                delete[] decodedData;
+                returnExternalMemory(memMgr, decodedData);
                 return 0;
             }
 
             decodedData[ outputIndex++ ] = set1stOctet(b1, b2);
         }
-        else if (!isPad( d3 ) && isPad( d4 )) 
-        {               
+        else if (!isPad( d3 ) && isPad( d4 ))
+        {
             // one PAD e.g. 3cQ=
             b3 = base64Inverse[ d3 ];
             if (( b3 & 0x3 ) != 0 ) // last 2 bits should be zero
             {
-                delete[] decodedData;
+                returnExternalMemory(memMgr, decodedData);
                 return 0;
             }
 
             decodedData[ outputIndex++ ] = set1stOctet( b1, b2 );
             decodedData[ outputIndex++ ] = set2ndOctet( b2, b3 );
-        } 
-        else 
+        }
+        else
         {
-            // an error like "3c[Pad]r", "3cdX", "3cXd", "3cXX" where X is non data 
-            delete[] decodedData;
+            // an error like "3c[Pad]r", "3cdX", "3cXd", "3cXX" where X is non data
+            returnExternalMemory(memMgr, decodedData);
             return 0;
-        }       
-    } 
-    else 
-    { 
+        }
+    }
+    else
+    {
         // no PAD e.g 3cQl
         b3 = base64Inverse[ d3 ];
         b4 = base64Inverse[ d4 ];
@@ -403,44 +506,56 @@ XMLByte* Base64::decode(const XMLByte* const inputData,
     }
 
     // write out the end of string
-    decodedData[ outputIndex ] = 0; 
-    *outputLength = outputIndex;
+    decodedData[ outputIndex ] = 0;
+    *decodedLength = outputIndex;
 
     return decodedData;
 }
 
-XMLCh* Base64::decode(const XMLCh* const inputData,
-                      unsigned int*      outputLength)
+XMLCh* Base64::decode(const XMLCh* const   inputData
+                    , unsigned int*        decodedLen
+                    , MemoryManager* const memMgr)
 {
 	if (!inputData)
 		return 0;
 
+    /***
+     *  Move input data to a XMLByte buffer
+     */
 	unsigned int srcLen = XMLString::stringLen(inputData);
-    XMLByte *toFill = new XMLByte[srcLen+1];
-    ArrayJanitor<XMLByte> janFill(toFill);
+    XMLByte *dataInByte = (XMLByte*) getInternalMemory((srcLen+1) * sizeof(XMLByte));
+    ArrayJanitor<XMLByte> janFill(dataInByte, XMLPlatformUtils::fgMemoryManager);
 
     for (unsigned int i = 0; i < srcLen; i++)
-		toFill[i] = (XMLByte)inputData[i];
+		dataInByte[i] = (XMLByte)inputData[i];
 
-	toFill[srcLen] = 0;
+	dataInByte[srcLen] = 0;
 
-	unsigned int      decodedLen = 0;
-	XMLByte *DecodedBuf = decode(toFill, &decodedLen);
+    /***
+     * Forward to the actual decoding method to do the decoding
+     */
+	*decodedLen = 0;
+	XMLByte *DecodedBuf = decode(dataInByte, decodedLen, memMgr);
 
 	if (!DecodedBuf)
 		return 0;
 
-    XMLCh *toRet = new XMLCh[decodedLen+1];
-
-    for (unsigned int j = 0; j < decodedLen; j++)
+    /***
+     * Move decoded data to a XMLCh buffer to return
+     */
+    XMLCh *toRet = (XMLCh*) getExternalMemory(memMgr, (*decodedLen+1) * sizeof(XMLCh));
+               
+    for (unsigned int j = 0; j < *decodedLen; j++)
 		toRet[j] = (XMLCh)DecodedBuf[j];
 
-	toRet[decodedLen] = 0;
+	toRet[*decodedLen] = 0;
 
-	*outputLength = decodedLen;
-	delete[] DecodedBuf;
+    /***
+     * Release the memory allocated in the actual decoding method
+     */ 
+    returnExternalMemory(memMgr, DecodedBuf);
 
-	return toRet;
+    return toRet;
 
 }
 
@@ -460,15 +575,17 @@ void Base64::init()
 
     int i;
     // set all fields to -1
-    for ( i = 0; i < BASELENGTH; i++ ) 
-        base64Inverse[i] = (XMLByte)-1; 
+    for ( i = 0; i < BASELENGTH; i++ )
+        base64Inverse[i] = (XMLByte)-1;
 
     // compute inverse table
-    for ( i = 0; i < 64; i++ ) 
+    for ( i = 0; i < 64; i++ )
         base64Inverse[ base64Alphabet[i] ] = (XMLByte)i;
 }
 
-bool Base64::isData(const XMLByte& octet) 
+bool Base64::isData(const XMLByte& octet)
 {
     return (base64Inverse[octet]!=(XMLByte)-1);
 }
+
+XERCES_CPP_NAMESPACE_END
