@@ -56,6 +56,15 @@
 
 /*
  * $Log: BinHTTPURLInputStream.cpp,v $
+ * Revision 1.4  2003/05/17 05:54:18  knoaman
+ * Update NetAccessors to use the memory manager.
+ *
+ * Revision 1.3  2002/12/06 16:43:55  tng
+ * Fix the error messages thrown from net accessor module.
+ *
+ * Revision 1.2  2002/11/04 15:11:39  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:23  peiyongz
  * sane_include
  *
@@ -130,6 +139,7 @@
 #include <xercesc/util/Janitor.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 HMODULE gWinsockLib = NULL;
 LPFN_GETHOSTBYNAME gWSgethostbyname = NULL;
@@ -306,24 +316,24 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     //   and transcode them back to ASCII.
     //
     const XMLCh*        hostName = urlSource.getHost();
-    char*               hostNameAsCharStar = XMLString::transcode(hostName);
-    ArrayJanitor<char>  janBuf1(hostNameAsCharStar);
+    char*               hostNameAsCharStar = XMLString::transcode(hostName, urlSource.getMemoryManager());
+    ArrayJanitor<char>  janBuf1(hostNameAsCharStar, urlSource.getMemoryManager());
 
     const XMLCh*        path = urlSource.getPath();
-    char*               pathAsCharStar = XMLString::transcode(path);
-    ArrayJanitor<char>  janBuf2(pathAsCharStar);
+    char*               pathAsCharStar = XMLString::transcode(path, urlSource.getMemoryManager());
+    ArrayJanitor<char>  janBuf2(pathAsCharStar, urlSource.getMemoryManager());
 
     const XMLCh*        fragment = urlSource.getFragment();
     char*               fragmentAsCharStar = 0;
     if (fragment)
-        fragmentAsCharStar = XMLString::transcode(fragment);
-    ArrayJanitor<char>  janBuf3(fragmentAsCharStar);
+        fragmentAsCharStar = XMLString::transcode(fragment, urlSource.getMemoryManager());
+    ArrayJanitor<char>  janBuf3(fragmentAsCharStar, urlSource.getMemoryManager());
 
     const XMLCh*        query = urlSource.getQuery();
     char*               queryAsCharStar = 0;
     if (query)
-        queryAsCharStar = XMLString::transcode(query);
-    ArrayJanitor<char>  janBuf4(queryAsCharStar);		
+        queryAsCharStar = XMLString::transcode(query, urlSource.getMemoryManager());
+    ArrayJanitor<char>  janBuf4(queryAsCharStar, urlSource.getMemoryManager());		
 
     unsigned short      portNumber = (unsigned short) urlSource.getPortNum();
 
@@ -340,16 +350,16 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
         if (numAddress == INADDR_NONE)
         {
             // Call WSAGetLastError() to get the error number.
-            ThrowXML(NetAccessorException,
-                     XMLExcepts::NetAcc_TargetResolution);
+            ThrowXML1(NetAccessorException,
+                     XMLExcepts::NetAcc_TargetResolution, hostName);
         }
         if ((hostEntPtr =
                 gethostbyaddr((const char *) &numAddress,
                               sizeof(unsigned long), AF_INET)) == NULL)
         {
             // Call WSAGetLastError() to get the error number.
-            ThrowXML(NetAccessorException,
-                     XMLExcepts::NetAcc_TargetResolution);
+            ThrowXML1(NetAccessorException,
+                     XMLExcepts::NetAcc_TargetResolution, hostName);
         }
     }
 
@@ -362,15 +372,15 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     if (s == INVALID_SOCKET)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML(NetAccessorException,
-                 XMLExcepts::NetAcc_CreateSocket);
+        ThrowXML1(NetAccessorException,
+                 XMLExcepts::NetAcc_CreateSocket, urlSource.getURLText());
     }
 
     if (connect(s, (struct sockaddr *) &sa, sizeof(sa)) == SOCKET_ERROR)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML(NetAccessorException,
-                 XMLExcepts::NetAcc_ConnSocket);
+        ThrowXML1(NetAccessorException,
+                 XMLExcepts::NetAcc_ConnSocket, urlSource.getURLText());
     }
 
 
@@ -417,8 +427,8 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     if ((aLent = send(s, fBuffer, lent, 0)) != lent)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML(NetAccessorException,
-                 XMLExcepts::NetAcc_WriteSocket);
+        ThrowXML1(NetAccessorException,
+                 XMLExcepts::NetAcc_WriteSocket, urlSource.getURLText());
     }
 
 
@@ -430,7 +440,7 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     if (aLent == SOCKET_ERROR || aLent == 0)
     {
         // Call WSAGetLastError() to get the error number.
-        ThrowXML(NetAccessorException, XMLExcepts::NetAcc_ReadSocket);
+        ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
     }
 
     fBufferEnd = fBuffer+aLent;
@@ -465,7 +475,7 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
                 if (aLent == SOCKET_ERROR || aLent == 0)
                 {
                     // Call WSAGetLastError() to get the error number.
-                    ThrowXML(NetAccessorException, XMLExcepts::NetAcc_ReadSocket);
+                    ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
                 }
                 fBufferEnd = fBufferEnd + aLent;
                 *fBufferEnd = 0;
@@ -478,13 +488,13 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
     char *p = strstr(fBuffer, "HTTP");
     if (p == 0)
     {
-        ThrowXML(NetAccessorException, XMLExcepts::NetAcc_ReadSocket);
+        ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
     }
 
     p = strchr(p, ' ');
     if (p == 0)
     {
-        ThrowXML(NetAccessorException, XMLExcepts::NetAcc_ReadSocket);
+        ThrowXML1(NetAccessorException, XMLExcepts::NetAcc_ReadSocket, urlSource.getURLText());
     }
 
     int httpResponse = atoi(p);
@@ -493,7 +503,7 @@ BinHTTPURLInputStream::BinHTTPURLInputStream(const XMLURL& urlSource)
         // Most likely a 404 Not Found error.
         //   Should recognize and handle the forwarding responses.
         //
-        ThrowXML(NetAccessorException, XMLExcepts::File_CouldNotOpenFile);
+        ThrowXML1(NetAccessorException, XMLExcepts::File_CouldNotOpenFile, urlSource.getURLText());
     }
 
     fSocketHandle = (unsigned int) s;
@@ -541,3 +551,5 @@ unsigned int BinHTTPURLInputStream::readBytes(XMLByte* const    toFill
     return len;
 }
 
+
+XERCES_CPP_NAMESPACE_END
