@@ -56,6 +56,27 @@
 
 /*
  * $Log: HPPlatformUtils.cpp,v $
+ * Revision 1.13  2003/05/15 18:37:48  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.12  2003/04/25 17:19:26  peiyongz
+ * throw exception if getcwd() fails
+ *
+ * Revision 1.11  2003/04/24 02:53:52  peiyongz
+ * Logical Path Resolution
+ *
+ * Revision 1.10  2003/03/09 16:53:55  peiyongz
+ * PanicHandler
+ *
+ * Revision 1.9  2002/12/02 19:17:00  tng
+ * [Bug 14723] Memory leak in atomicOpsMutex.  Patch from Adam Zell.
+ *
+ * Revision 1.8  2002/11/07 22:37:51  peiyongz
+ * build -miconv on hpux and linux
+ *
+ * Revision 1.7  2002/11/04 15:13:00  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.6  2002/07/15 21:52:48  peiyongz
  * CouldNotWriteToFile
  *
@@ -168,7 +189,7 @@
 #include    <xercesc/util/XMLString.hpp>
 #include    <xercesc/util/XMLUniDefs.hpp>
 #include    <xercesc/util/XMLUni.hpp>
-
+#include    <xercesc/util/PanicHandler.hpp>
 
 #if defined(XML_USE_ICU_TRANSCODER)
     #include <xercesc/util/Transcoders/ICU/ICUTransService.hpp>
@@ -180,7 +201,8 @@
 
 #if defined(XML_USE_ICU_MESSAGELOADER)
     #include <xercesc/util/MsgLoaders/ICU/ICUMsgLoader.hpp>
-
+#elif defined (XML_USE_ICONV_MESSAGELOADER)
+    #include <xercesc/util/MsgLoaders/MsgCatalog/MsgCatalogLoader.hpp>
 #else
     // Same as -DXML_USE_INMEM_MESSAGELOADER
     #include <xercesc/util/MsgLoaders/InMemory/InMemMsgLoader.hpp>
@@ -199,8 +221,7 @@
 #endif
 
 
-
-
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Private Static Methods
@@ -235,6 +256,8 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
     {
 #if defined (XML_USE_ICU_MESSAGELOADER)
         retVal = new ICUMsgLoader(msgDomain);
+#elif defined (XML_USE_ICONV_MESSAGELOADER)
+        retVal = new MsgCatalogLoader(msgDomain);
 #else
         // same as -DXML_USE_INMEM_MESSAGELOADER
         retVal = new InMemMsgLoader(msgDomain);
@@ -242,7 +265,7 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
     }
     catch(...)
     {
-        panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
+        panic(PanicHandler::Panic_CantLoadMsgDomain);
     }
     return retVal;
 }
@@ -274,27 +297,9 @@ XMLTransService* XMLPlatformUtils::makeTransService()
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: The panic method
 // ---------------------------------------------------------------------------
-void XMLPlatformUtils::panic(const PanicReasons reason)
+void XMLPlatformUtils::panic(const PanicHandler::PanicReasons reason)
 {
-     const char* reasonStr = "Unknown reason";
-    if (reason == Panic_NoTransService)
-        reasonStr = "Could not load a transcoding service";
-    else if (reason == Panic_NoDefTranscoder)
-        reasonStr = "Could not load a local code page transcoder";
-    else if (reason == Panic_CantFindLib)
-        reasonStr = "Could not find the xerces-c DLL";
-    else if (reason == Panic_UnknownMsgDomain)
-        reasonStr = "Unknown message domain";
-    else if (reason == Panic_CantLoadMsgDomain)
-        reasonStr = "Cannot load message domain";
-    else if (reason == Panic_SynchronizationErr)
-        reasonStr = "Cannot synchronize system or mutex";
-    else if (reason == Panic_SystemInit)
-        reasonStr = "Cannot initialize the system or mutex";
-
-    fprintf(stderr, "%s\n", reasonStr);
-
-    exit(-1);
+    fgUserPanicHandler? fgUserPanicHandler->panic(reason) : fgDefaultPanicHandler->panic(reason);	
 }
 
 
@@ -360,16 +365,16 @@ FileHandle XMLPlatformUtils::openFile(const char* const fileName)
 
 FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
 {
-    char* tmpFileName = XMLString::transcode(fileName);
+    char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
     FileHandle retVal = (FILE*) fopen(tmpFileName , "rb");
-    delete [] tmpFileName;
+    fgMemoryManager->deallocate(tmpFileName);//delete [] tmpFileName;
     return retVal;
 }
 
 FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
 {
-    const char* tmpFileName = XMLString::transcode(fileName);
-    ArrayJanitor<char> janText((char*)tmpFileName);
+    const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
+    ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
     return fopen( tmpFileName , "wb" );
 }
 
@@ -403,7 +408,7 @@ XMLPlatformUtils::readFileBuffer(       FileHandle      theFile
 void
 XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
                                    , long                  toWrite
-                                   , const XMLByte* const  toFlush)                                   
+                                   , const XMLByte* const  toFlush)
 {
     if (!theFile        ||
         (toWrite <= 0 ) ||
@@ -447,7 +452,8 @@ void XMLPlatformUtils::resetFile(FileHandle theFile)
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: File system methods
 // ---------------------------------------------------------------------------
-XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
+XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
+                                     MemoryManager* const manager)
 {
 
 
@@ -456,11 +462,11 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
     //  so we know that its not some pathological freaky path. It comes in
     //  in native format, and goes out as Unicode always
     //
-    char* newSrc = XMLString::transcode(srcPath);
-    ArrayJanitor<char> janText(newSrc);
+    char* newSrc = XMLString::transcode(srcPath, fgMemoryManager);
+    ArrayJanitor<char> janText(newSrc, fgMemoryManager);
 
     // Use a local buffer that is big enough for the largest legal path
-    char absPath[PATH_MAX];
+    char absPath[PATH_MAX + 1];
     //get the absolute path
     char* retPath = realpath(newSrc, &absPath[0]);
 
@@ -468,7 +474,7 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
     {
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName);
     }
-    return XMLString::transcode(absPath);
+    return XMLString::transcode(absPath, manager);
 }
 
 
@@ -490,112 +496,23 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
     return true;
 }
 
-XMLCh* XMLPlatformUtils::weavePaths
-    (
-        const   XMLCh* const    basePath
-        , const XMLCh* const    relativePath
-    )
+XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
-// Create a buffer as large as both parts and empty it
-    XMLCh* tmpBuf = new XMLCh[XMLString::stringLen(basePath)
-                              + XMLString::stringLen(relativePath)
-                              + 2];
-    *tmpBuf = 0;
+    char  dirBuf[PATH_MAX + 1];
+    char  *curDir = getcwd(&dirBuf[0], PATH_MAX + 1);
 
-    //
-    //  If we have no base path, then just take the relative path as
-    //  is.
-    //
-    if (!basePath)
+    if (!curDir)
     {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetBasePathName);
     }
 
-    if (!*basePath)
-    {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
-    }
+    return XMLString::transcode(curDir, manager);
+}
 
-    const XMLCh* basePtr = basePath + (XMLString::stringLen(basePath) - 1);
-    if ((*basePtr != chForwardSlash)
-    &&  (*basePtr != chBackSlash))
-    {
-        while ((basePtr >= basePath)
-        &&     ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
-        {
-            basePtr--;
-        }
-    }
-
-    // There is no relevant base path, so just take the relative part
-    if (basePtr < basePath)
-    {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
-    }
-
-    // After this, make sure the buffer gets handled if we exit early
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
-
-    //
-    //  We have some path part, so we need to check to see if we ahve to
-    //  weave any of the parts together.
-    //
-    const XMLCh* pathPtr = relativePath;
-    while (true)
-    {
-        // If it does not start with some period, then we are done
-        if (*pathPtr != chPeriod)
-            break;
-
-        unsigned int periodCount = 1;
-        pathPtr++;
-        if (*pathPtr == chPeriod)
-        {
-            pathPtr++;
-            periodCount++;
-        }
-
-        // Has to be followed by a \ or / or the null to mean anything
-        if ((*pathPtr != chForwardSlash) && (*pathPtr != chBackSlash)
-        &&  *pathPtr)
-        {
-            break;
-        }
-        if (*pathPtr)
-            pathPtr++;
-
-        // If its one period, just eat it, else move backwards in the base
-        if (periodCount == 2)
-        {
-            basePtr--;
-            while ((basePtr >= basePath)
-            &&     ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
-            {
-                basePtr--;
-            }
-
-            // The base cannot provide enough levels, so its in error/
-            if (basePtr < basePath)
-                ThrowXML(XMLPlatformUtilsException,
-                         XMLExcepts::File_BasePathUnderflow);
-        }
-    }
-
-    // Copy the base part up to the base pointer
-    XMLCh* bufPtr = tmpBuf;
-    const XMLCh* tmpPtr = basePath;
-    while (tmpPtr <= basePtr)
-        *bufPtr++ = *tmpPtr++;
-
-    // And then copy on the rest of our path
-    XMLString::copyString(bufPtr, pathPtr);
-
-    // Orphan the buffer and return it
-    janBuf.orphan();
-    return tmpBuf;
+inline bool XMLPlatformUtils::isAnySlash(XMLCh c) 
+{
+    return ( chBackSlash == c || chForwardSlash == c);
 }
 
 // ---------------------------------------------------------------------------
@@ -634,7 +551,8 @@ void XMLPlatformUtils::platformInit()
     // Normally, mutexes are created on first use, but there is a
     // circular dependency between compareAndExchange() and
     // mutex creation that must be broken.
-    atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
+    if (atomicOpsMutex.fHandle == 0)
+        atomicOpsMutex.fHandle = XMLPlatformUtils::makeMutex();
 }
 
 void* XMLPlatformUtils::makeMutex()
@@ -788,3 +706,6 @@ void XMLPlatformUtils::platformTerm()
 #endif
 }
 
+#include <xercesc/util/LogicalPath.c>
+
+XERCES_CPP_NAMESPACE_END
