@@ -56,6 +56,24 @@
 
 /*
  * $Log: OpenServerPlatformUtils.cpp,v $
+ * Revision 1.8  2003/05/15 18:37:49  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.7  2003/04/25 17:20:11  peiyongz
+ * throw exception if getcwd() fails
+ *
+ * Revision 1.6  2003/04/24 02:55:58  peiyongz
+ * Logical Path Resolution
+ *
+ * Revision 1.5  2003/03/09 16:57:18  peiyongz
+ * PanicHandler
+ *
+ * Revision 1.4  2002/11/19 13:04:32  gareth
+ * Bug# 14661 Caldera implemented openFileToWrite and writeBufferToFile. Patch from Cameron Dorrat.
+ *
+ * Revision 1.3  2002/11/04 15:13:01  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.2  2002/05/21 20:31:47  tng
  * Minor update: Remove obsolete code
  *
@@ -89,6 +107,7 @@
 #include    <xercesc/util/XMLString.hpp>
 #include    <xercesc/util/XMLUniDefs.hpp>
 #include    <xercesc/util/XMLUni.hpp>
+#include    <xercesc/util/PanicHandler.hpp>
 
 #if defined (XML_USE_ICU_TRANSCODER)
 #   include <xercesc/util/Transcoders/ICU/ICUTransService.hpp>
@@ -115,6 +134,7 @@
 #   include <xercesc/util/NetAccessors/Socket/SocketNetAccessor.hpp>
 #endif
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Private Static Methods
@@ -139,7 +159,7 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
     }
     catch(...)
     {
-        panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
+        panic(PanicHandler::Panic_CantLoadMsgDomain);
     }
     return retVal;
 }
@@ -175,27 +195,9 @@ XMLTransService* XMLPlatformUtils::makeTransService()
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: The panic method
 // ---------------------------------------------------------------------------
-void XMLPlatformUtils::panic(const PanicReasons reason)
+void XMLPlatformUtils::panic(const PanicHandler::PanicReasons reason)
 {
-	const char* reasonStr = "Unknown reason";
-    if (reason == Panic_NoTransService)
-        reasonStr = "Could not load a transcoding service";
-    else if (reason == Panic_NoDefTranscoder)
-        reasonStr = "Could not load a local code page transcoder";
-    else if (reason == Panic_CantFindLib)
-        reasonStr = "Could not find the xerces-c DLL";
-    else if (reason == Panic_UnknownMsgDomain)
-        reasonStr = "Unknown message domain";
-    else if (reason == Panic_CantLoadMsgDomain)
-        reasonStr = "Cannot load message domain";
-    else if (reason == Panic_SynchronizationErr)
-        reasonStr = "Cannot synchronize system or mutex";
-    else if (reason == Panic_SystemInit)
-        reasonStr = "Cannot initialize the system or mutex";
-
-    fprintf(stderr, "%s\n", reasonStr);
-
-    exit(-1);
+    fgUserPanicHandler? fgUserPanicHandler->panic(reason) : fgDefaultPanicHandler->panic(reason);	
 }
 
 // ---------------------------------------------------------------------------
@@ -247,8 +249,8 @@ unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
 
 FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
 {
-    const char* tmpFileName = XMLString::transcode(fileName);
-    ArrayJanitor<char> janText((char*)tmpFileName);
+    const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
+    ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
     FileHandle retVal = (FILE*)fopen(tmpFileName , "rb");
 
     if (retVal == NULL)
@@ -267,6 +269,18 @@ FileHandle XMLPlatformUtils::openFile(const char* const fileName)
     return retVal;
 }
 
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+{
+    const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
+    ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
+    return fopen( tmpFileName , "wb" );
+}
+
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+{
+    return fopen( fileName , "wb" );
+}
+
 FileHandle XMLPlatformUtils::openStdInHandle()
 {
 	return (FileHandle)fdopen(dup(0), "rb");
@@ -287,6 +301,41 @@ unsigned int XMLPlatformUtils::readFileBuffer(FileHandle           theFile,
     return (unsigned int)noOfItemsRead;
 }
 
+void
+XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
+                                   , long                  toWrite
+                                   , const XMLByte* const  toFlush)                                   
+{
+    if (!theFile        ||
+        (toWrite <= 0 ) ||
+        !toFlush         )
+        return;
+
+    const XMLByte* tmpFlush = (const XMLByte*) toFlush;
+    size_t bytesWritten = 0;
+
+    while (true)
+    {
+        bytesWritten=fwrite(tmpFlush, sizeof(XMLByte), toWrite, (FILE*)theFile);
+
+        if(ferror((FILE*)theFile))
+        {
+            ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+        }
+
+        if (bytesWritten < toWrite) //incomplete write
+        {
+            tmpFlush+=bytesWritten;
+            toWrite-=bytesWritten;
+            bytesWritten=0;
+        }
+        else
+            return;
+    }
+
+    return;
+}
+
 void XMLPlatformUtils::resetFile(FileHandle theFile)
 {
     if (fseek((FILE*)theFile, 0, SEEK_SET))
@@ -297,19 +346,25 @@ void XMLPlatformUtils::resetFile(FileHandle theFile)
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: File system methods
 // ---------------------------------------------------------------------------
-XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
+XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
+                                     MemoryManager* const manager)
 {
     //
     //  NOTE: The path provided has always already been opened successfully,
     //  so we know that its not some pathological freaky path. It comes in
     //  in native format, and goes out as Unicode always
     //
-    char* newSrc = XMLString::transcode(srcPath);
-    ArrayJanitor<char> janText(newSrc);
+    char* newSrc = XMLString::transcode(srcPath, fgMemoryManager);
+    ArrayJanitor<char> janText(newSrc, fgMemoryManager);
 
     // Use a local buffer that is big enough for the largest legal path
-    char *absPath = new char[pathconf(newSrc, _PC_PATH_MAX)];
-    ArrayJanitor<char> janText2(absPath);
+	 // Without the *3 we get exceptions with gcc on OpenServer 5.0.5/6 when 
+	 // relative paths are passed in
+    char *absPath = (char*) fgMemoryManager->allocate
+    (
+        (pathconf(newSrc, _PC_PATH_MAX)*3) * sizeof(char)
+    );//new char[pathconf(newSrc, _PC_PATH_MAX)*3];
+    ArrayJanitor<char> janText2(absPath, fgMemoryManager);
     // Get the absolute path
     char* retPath = realpath(newSrc, absPath);
 
@@ -318,7 +373,7 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
         ThrowXML(XMLPlatformUtilsException,
                  XMLExcepts::File_CouldNotGetBasePathName);
     }
-    return XMLString::transcode(absPath);
+    return XMLString::transcode(absPath, manager);
 }
 
 bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
@@ -339,107 +394,24 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
     return true;
 }
 
-XMLCh* XMLPlatformUtils::weavePaths(const XMLCh* const   basePath,
-                                    const XMLCh* const   relativePath)
+XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
-    // Create a buffer as large as both parts and empty it
-    XMLCh* tmpBuf = new XMLCh[XMLString::stringLen(basePath)
-                              + XMLString::stringLen(relativePath)
-                              + 2];
-    *tmpBuf = 0;
+    char  dirBuf[PATH_MAX + 1];
+    char  *curDir = getcwd(&dirBuf[0], PATH_MAX + 1);
 
-    //
-    //  If we have no base path, then just take the relative path as
-    //  is.
-    //
-    if (!basePath)
+    if (!curDir)
     {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetBasePathName);
     }
 
-    if (!*basePath)
-    {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
-    }
-
-    const XMLCh* basePtr = basePath + (XMLString::stringLen(basePath) - 1);
-    if ((*basePtr != chForwardSlash)
-		&& (*basePtr != chBackSlash))
-    {
-        while ((basePtr >= basePath)
-			   && ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
-            basePtr--;
-    }
-
-    // There is no relevant base path, so just take the relative part
-    if (basePtr < basePath)
-    {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
-    }
-
-    // After this, make sure the buffer gets handled if we exit early
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
-
-    //
-    //  We have some path part, so we need to check to see if we ahve to
-    //  weave any of the parts together.
-    //
-    const XMLCh* pathPtr = relativePath;
-    while (true)
-    {
-        // If it does not start with some period, then we are done
-        if (*pathPtr != chPeriod)
-            break;
-
-        unsigned int periodCount = 1;
-        pathPtr++;
-        if (*pathPtr == chPeriod)
-        {
-            pathPtr++;
-            periodCount++;
-        }
-
-        // Has to be followed by a \ or / or the null to mean anything
-        if ((*pathPtr != chForwardSlash)
-			&& (*pathPtr != chBackSlash)
-			&&  *pathPtr)
-            break;
-
-        if (*pathPtr)
-            pathPtr++;
-
-        // If its one period, just eat it, else move backwards in the base
-        if (periodCount == 2)
-        {
-            basePtr--;
-            while ((basePtr >= basePath) &&
-				   ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
-                basePtr--;
-
-            // The base cannot provide enough levels, so its in error
-            if (basePtr < basePath)
-                ThrowXML(XMLPlatformUtilsException,
-                         XMLExcepts::File_BasePathUnderflow);
-        }
-    }
-
-    // Copy the base part up to the base pointer
-    XMLCh* bufPtr = tmpBuf;
-    const XMLCh* tmpPtr = basePath;
-    while (tmpPtr <= basePtr)
-        *bufPtr++ = *tmpPtr++;
-
-    // And then copy on the rest of our path
-    XMLString::copyString(bufPtr, pathPtr);
-
-    // Orphan the buffer and return it
-    janBuf.orphan();
-    return tmpBuf;
+    return XMLString::transcode(curDir, manager);
 }
 
+inline bool XMLPlatformUtils::isAnySlash(XMLCh c) 
+{
+    return ( chBackSlash == c || chForwardSlash == c);
+}
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Timing Methods
@@ -476,14 +448,14 @@ void XMLPlatformUtils::platformInit()
 	{
 		delete gAtomicOpMutex;
 		gAtomicOpMutex = 0;
-		panic(XMLPlatformUtils::Panic_SystemInit);
+		panic(PanicHandler::Panic_SystemInit);
 	}
 }
 
 // -----------------------------------------------------------------------
 //  Mutex- and atomic operation methods
 // -----------------------------------------------------------------------
-class  RecursiveMutex
+class  RecursiveMutex : public XMemory
 {
 public:
 	pthread_mutex_t   mutex;
@@ -572,14 +544,14 @@ void* XMLPlatformUtils::compareAndSwap(void**              toFill,
 									   const void* const   toCompare)
 {
 	if (pthread_mutex_lock(gAtomicOpMutex))
-		panic(XMLPlatformUtils::Panic_SynchronizationErr);
+		panic(PanicHandler::Panic_SynchronizationErr);
 
 	void *retVal = *toFill;
 	if (*toFill == toCompare)
 		*toFill = (void *)newValue;
 
 	if (pthread_mutex_unlock(gAtomicOpMutex))
-		panic(XMLPlatformUtils::Panic_SynchronizationErr);
+		panic(PanicHandler::Panic_SynchronizationErr);
 
 	return retVal;
 }
@@ -587,12 +559,12 @@ void* XMLPlatformUtils::compareAndSwap(void**              toFill,
 int XMLPlatformUtils::atomicIncrement(int &location)
 {
 	if (pthread_mutex_lock(gAtomicOpMutex))
-		panic(XMLPlatformUtils::Panic_SynchronizationErr);
+		panic(PanicHandler::Panic_SynchronizationErr);
 
 	int tmp = ++location;
 
 	if (pthread_mutex_unlock(gAtomicOpMutex))
-		panic(XMLPlatformUtils::Panic_SynchronizationErr);
+		panic(PanicHandler::Panic_SynchronizationErr);
 
 	return tmp;
 }
@@ -600,12 +572,12 @@ int XMLPlatformUtils::atomicIncrement(int &location)
 int XMLPlatformUtils::atomicDecrement(int &location)
 {
 	if (pthread_mutex_lock(gAtomicOpMutex))
-		panic(XMLPlatformUtils::Panic_SynchronizationErr);
+		panic(PanicHandler::Panic_SynchronizationErr);
 
 	int tmp = --location;
 
 	if (pthread_mutex_unlock( gAtomicOpMutex))
-		panic(XMLPlatformUtils::Panic_SynchronizationErr);
+		panic(PanicHandler::Panic_SynchronizationErr);
 
 	return tmp;
 }
@@ -681,3 +653,7 @@ void XMLPlatformUtils::platformTerm()
 #endif
 }
 
+#include <xercesc/util/LogicalPath.c>
+
+XERCES_CPP_NAMESPACE_END
+
