@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: Tru64PlatformUtils.cpp,v 1.3 2002/07/26 17:18:25 tng Exp $
+ * $Id: Tru64PlatformUtils.cpp,v 1.10 2003/05/15 18:37:49 knoaman Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -78,6 +78,7 @@
 #include    <xercesc/util/XMLString.hpp>
 #include    <xercesc/util/XMLUni.hpp>
 #include    <xercesc/util/XMLUniDefs.hpp>
+#include    <xercesc/util/PanicHandler.hpp>
 
 //
 //  These control which transcoding service is used by the Tru64 version.
@@ -112,6 +113,8 @@
 #include <xercesc/util/NetAccessors/Socket/SocketNetAccessor.hpp>
 #endif
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Private Static Methods
 // ---------------------------------------------------------------------------
@@ -151,7 +154,7 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
   }
   catch(...)
   {
-    panic(XMLPlatformUtils::Panic_NoDefTranscoder);
+    panic(PanicHandler::Panic_CantLoadMsgDomain);
   }
   return retVal;
 }
@@ -182,27 +185,9 @@ XMLTransService* XMLPlatformUtils::makeTransService()
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: The panic method
 // ---------------------------------------------------------------------------
-void XMLPlatformUtils::panic(const PanicReasons reason)
+void XMLPlatformUtils::panic(const PanicHandler::PanicReasons reason)
 {
-  const char* reasonStr = "Unknown reason";
-  if (reason == Panic_NoTransService)
-    reasonStr = "Could not load a transcoding service";
-  else if (reason == Panic_NoDefTranscoder)
-    reasonStr = "Could not load a local code page transcoder";
-  else if (reason == Panic_CantFindLib)
-    reasonStr = "Could not find the xerces-c DLL";
-  else if (reason == Panic_UnknownMsgDomain)
-    reasonStr = "Unknown message domain";
-  else if (reason == Panic_CantLoadMsgDomain)
-    reasonStr = "Cannot load message domain";
-  else if (reason == Panic_SynchronizationErr)
-    reasonStr = "Cannot synchronize system or mutex";
-  else if (reason == Panic_SystemInit)
-    reasonStr = "Cannot initialize the system or mutex";
-
-  fprintf(stderr, "%s\n", reasonStr);
-
-  exit(-1);
+    fgUserPanicHandler? fgUserPanicHandler->panic(reason) : fgDefaultPanicHandler->panic(reason);	
 }
 
 // ---------------------------------------------------------------------------
@@ -263,8 +248,8 @@ FileHandle XMLPlatformUtils::openFile(const char* const fileName)
 
 FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
 {
-  const char* tmpFileName = XMLString::transcode(fileName);
-  ArrayJanitor<char> janText((char*)tmpFileName);
+  const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
+  ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
   FileHandle retVal = (FILE*)fopen( tmpFileName , "rb" );
 
   if (retVal == NULL)
@@ -274,8 +259,8 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
 
 FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
 {
-    const char* tmpFileName = XMLString::transcode(fileName);
-    ArrayJanitor<char> janText((char*)tmpFileName);
+    const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
+    ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
     return fopen( tmpFileName , "wb" );
 }
 
@@ -303,7 +288,7 @@ unsigned int XMLPlatformUtils::readFileBuffer(FileHandle theFile,
 void
 XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
                                    , long                  toWrite
-                                   , const XMLByte* const  toFlush)                                   
+                                   , const XMLByte* const  toFlush)
 {
     if (!theFile        ||
         (toWrite <= 0 ) ||
@@ -347,28 +332,28 @@ void XMLPlatformUtils::resetFile(FileHandle theFile)
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: File system methods
 // ---------------------------------------------------------------------------
-XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
+XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
+                                     MemoryManager* const manager)
 {
   //
   //  NOTE: THe path provided has always already been opened successfully,
   //  so we know that its not some pathological freaky path. It comes in
   //  in native format, and goes out as Unicode always
   //
-  char* newSrc = XMLString::transcode(srcPath);
-  ArrayJanitor<char> janText(newSrc);
+  char* newSrc = XMLString::transcode(srcPath, fgMemoryManager);
+  ArrayJanitor<char> janText(newSrc, fgMemoryManager);
 
   // Use a local buffer that is big enough for the largest legal path
-  char *absPath = new char[PATH_MAX];
-  ArrayJanitor<char> janText2(absPath);
+  char absPath[PATH_MAX + 1];
   //get the absolute path
-  char* retPath = realpath(newSrc, absPath);
+  char* retPath = realpath(newSrc, &absPath[0]);
 
   if (!retPath)
   {
     ThrowXML(XMLPlatformUtilsException,
 	     XMLExcepts::File_CouldNotGetBasePathName);
   }
-  return XMLString::transcode(absPath);
+  return XMLString::transcode(absPath, manager);
 }
 
 bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
@@ -389,113 +374,24 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
   return true;
 }
 
-XMLCh* XMLPlatformUtils::weavePaths (
-  const XMLCh* const basePath,
-  const XMLCh* const relativePath)
+XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
-// Create a buffer as large as both parts and empty it
-  XMLCh* tmpBuf = new XMLCh[XMLString::stringLen(basePath)
-			   + XMLString::stringLen(relativePath)
-			   + 2];
-  *tmpBuf = 0;
+    char  dirBuf[PATH_MAX + 1];
+    char  *curDir = getcwd(&dirBuf[0], PATH_MAX + 1);
 
-  //
-  //  If we have no base path, then just take the relative path as
-  //  is.
-  //
-  if (!basePath)
-  {
-    XMLString::copyString(tmpBuf, relativePath);
-    return tmpBuf;
-  }
-
-  if (!*basePath)
-  {
-    XMLString::copyString(tmpBuf, relativePath);
-    return tmpBuf;
-  }
-
-  const XMLCh* basePtr = basePath + (XMLString::stringLen(basePath) - 1);
-  if ((*basePtr != chForwardSlash)
-      &&  (*basePtr != chBackSlash))
-  {
-    while ((basePtr >= basePath)
-	   &&     ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
+    if (!curDir)
     {
-      basePtr--;
-    }
-  }
-
-  // There is no relevant base path, so just take the relative part
-  if (basePtr < basePath)
-  {
-    XMLString::copyString(tmpBuf, relativePath);
-    return tmpBuf;
-  }
-
-  // After this, make sure the buffer gets handled if we exit early
-  ArrayJanitor<XMLCh> janBuf(tmpBuf);
-
-  //
-  //  We have some path part, so we need to check to see if we ahve to
-  //  weave any of the parts together.
-  //
-  const XMLCh* pathPtr = relativePath;
-  while (true)
-  {
-    // If it does not start with some period, then we are done
-    if (*pathPtr != chPeriod)
-      break;
-
-    unsigned int periodCount = 1;
-    pathPtr++;
-    if (*pathPtr == chPeriod)
-    {
-      pathPtr++;
-      periodCount++;
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetBasePathName);
     }
 
-    // Has to be followed by a \ or / or the null to mean anything
-    if ((*pathPtr != chForwardSlash) && (*pathPtr != chBackSlash)
-        &&  *pathPtr)
-    {
-      break;
-    }
-    if (*pathPtr)
-      pathPtr++;
-
-    // If its one period, just eat it, else move backwards in the base
-    if (periodCount == 2)
-    {
-      basePtr--;
-      while ((basePtr >= basePath)
-	     &&     ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
-      {
-	basePtr--;
-      }
-
-      if (basePtr < basePath)
-      {
-	// The base cannot provide enough levels, so its in error
-	// <TBD>
-      }
-    }
-  }
-
-  // Copy the base part up to the base pointer
-  XMLCh* bufPtr = tmpBuf;
-  const XMLCh* tmpPtr = basePath;
-  while (tmpPtr <= basePtr)
-    *bufPtr++ = *tmpPtr++;
-
-  // And then copy on the rest of our path
-  XMLString::copyString(bufPtr, pathPtr);
-
-  // Orphan the buffer and return it
-  janBuf.orphan();
-  return tmpBuf;
+    return XMLString::transcode(curDir, manager);
 }
 
+inline bool XMLPlatformUtils::isAnySlash(XMLCh c) 
+{
+    return ( chBackSlash == c || chForwardSlash == c);
+}
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Timing Methods
@@ -532,7 +428,7 @@ void XMLPlatformUtils::platformInit()
   if (pthread_mutex_init(gAtomicOpMutex, NULL)) {
 	delete gAtomicOpMutex;
 	gAtomicOpMutex = 0;
-    panic( XMLPlatformUtils::Panic_SystemInit );
+    panic( PanicHandler::Panic_SystemInit );
   }
 }
 
@@ -635,14 +531,14 @@ void* XMLPlatformUtils::compareAndSwap (void** toFill,
   // Currently its supported only in the kernel mode
 
   if (pthread_mutex_lock( gAtomicOpMutex))
-    panic(XMLPlatformUtils::Panic_SynchronizationErr);
+    panic(PanicHandler::Panic_SynchronizationErr);
 
   void *retVal = *toFill;
   if (*toFill == toCompare)
     *toFill = (void *)newValue;
 
   if (pthread_mutex_unlock( gAtomicOpMutex))
-    panic(XMLPlatformUtils::Panic_SynchronizationErr);
+    panic(PanicHandler::Panic_SynchronizationErr);
 
   return retVal;
 }
@@ -652,12 +548,12 @@ int XMLPlatformUtils::atomicIncrement(int &location)
   //return (int)atomic_add_32_nv( (uint32_t*)&location, 1);
 
   if (pthread_mutex_lock( gAtomicOpMutex))
-    panic(XMLPlatformUtils::Panic_SynchronizationErr);
+    panic(PanicHandler::Panic_SynchronizationErr);
 
   int tmp = ++location;
 
   if (pthread_mutex_unlock( gAtomicOpMutex))
-    panic(XMLPlatformUtils::Panic_SynchronizationErr);
+    panic(PanicHandler::Panic_SynchronizationErr);
 
   return tmp;
 }
@@ -666,12 +562,12 @@ int XMLPlatformUtils::atomicDecrement(int &location)
   //return (int)atomic_add_32_nv( (uint32_t*)&location, -1);
 
   if (pthread_mutex_lock( gAtomicOpMutex))
-    panic(XMLPlatformUtils::Panic_SynchronizationErr);
+    panic(PanicHandler::Panic_SynchronizationErr);
 	
   int tmp = --location;
 
   if (pthread_mutex_unlock( gAtomicOpMutex))
-    panic(XMLPlatformUtils::Panic_SynchronizationErr);
+    panic(PanicHandler::Panic_SynchronizationErr);
 
   return tmp;
 }
@@ -737,3 +633,7 @@ void XMLPlatformUtils::platformTerm()
 	gAtomicOpMutex = 0;
 #endif
 }
+
+#include <xercesc/util/LogicalPath.c>
+
+XERCES_CPP_NAMESPACE_END
