@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: Win32PlatformUtils.cpp,v 1.8 2002/08/21 19:44:17 tng Exp $
+ * $Id: Win32PlatformUtils.cpp,v 1.20 2003/05/15 18:37:49 knoaman Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -68,7 +68,10 @@
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLUni.hpp>
+#include <xercesc/util/PanicHandler.hpp>
+
 #include <windows.h>
+#include <stdio.h>
 #include <stdlib.h>
 
 #ifdef _DEBUG
@@ -103,6 +106,8 @@
     #include <xercesc/util/MsgLoaders/InMemory/InMemMsgLoader.hpp>
 #elif defined (XML_USE_WIN32_MSGLOADER)
     #include <xercesc/util/MsgLoaders/Win32/Win32MsgLoader.hpp>
+#elif defined(XML_USE_ICU_MESSAGELOADER)
+    #include <xercesc/util/MsgLoaders/ICU/ICUMsgLoader.hpp>
 #else
     #error A message loading service must be chosen
 #endif
@@ -118,7 +123,7 @@
     #include <xercesc/util/NetAccessors/WinSock/WinSockNetAccessor.hpp>
 #endif
 
-
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Local data
@@ -133,36 +138,9 @@ static bool     gOnNT;
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: The panic method
 // ---------------------------------------------------------------------------
-void XMLPlatformUtils::panic(const PanicReasons reason)
+void XMLPlatformUtils::panic(const PanicHandler::PanicReasons reason)
 {
-    const char* reasonStr = "Unknown reason";
-    if (reason == Panic_NoTransService)
-        reasonStr = "Could not load a transcoding service";
-    else if (reason == Panic_NoDefTranscoder)
-        reasonStr = "Could not load a local code page transcoder";
-    else if (reason == Panic_CantFindLib)
-        reasonStr = "Could not find the xerces-c DLL";
-    else if (reason == Panic_UnknownMsgDomain)
-        reasonStr = "Unknown message domain";
-    else if (reason == Panic_CantLoadMsgDomain)
-        reasonStr = "Cannot load message domain";
-    else if (reason == Panic_SynchronizationErr)
-        reasonStr = "A system synchronization error occurred";
-    else if (reason == Panic_SystemInit)
-        reasonStr = "Failed to complete platfrom dependent initialization";
-
-    //
-    //  We just do a popup and exit. Replace this code to do whatever
-    //  you need to do.
-    //
-    MessageBoxA
-    (
-        0
-        , "Xerces Panic Error"
-        , reasonStr
-        , MB_OK | MB_ICONSTOP
-    );
-    exit(-1);
+    fgUserPanicHandler? fgUserPanicHandler->panic(reason) : fgDefaultPanicHandler->panic(reason);	
 }
 
 
@@ -189,15 +167,6 @@ static bool isBackSlash(XMLCh c) {
            c == chWonSign;
 }
 
-static bool isAnySlash(XMLCh c) {
-    return c == chBackSlash    ||
-           c == chForwardSlash ||
-           c == chYenSign      ||
-           c == chWonSign;
-}
-
-
-
 unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
 {
     // Get the current position
@@ -258,6 +227,36 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     if (!fileName)
         return 0;
 
+    //
+    //  We have to play a little trick here. If its /x:.....
+    //  style fully qualified path, we have to toss the leading /
+    //  character.
+    //
+    const XMLCh* nameToOpen = fileName;
+    if (*fileName == chForwardSlash)
+    {
+        if (XMLString::stringLen(fileName) > 3)
+        {
+            if (*(fileName + 2) == chColon)
+            {
+                const XMLCh chDrive = *(fileName + 1);
+                if (((chDrive >= chLatin_A) && (chDrive <= chLatin_Z))
+                ||  ((chDrive >= chLatin_a) && (chDrive <= chLatin_z)))
+                {
+                    nameToOpen = fileName + 1;
+                }
+            }
+
+            // Similarly for UNC paths
+            if ( *(fileName + 1) == *(fileName + 2) &&
+                 (*(fileName + 1) == chForwardSlash ||
+                  *(fileName + 1) == chBackSlash) )
+            {
+                nameToOpen = fileName + 1;
+            }
+        }
+    }
+
     //  Ok, this might look stupid but its a semi-expedient way to deal
     //  with a thorny problem. Shift-JIS and some other Asian encodings
     //  are fundamentally broken and map both the backslash and the Yen
@@ -277,9 +276,8 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     //   transcode correctly back to 8 bit char * form.
     //
     XMLCh *tmpUName = 0;
-    const XMLCh *nameToOpen = fileName;
 
-    const XMLCh* srcPtr = fileName;
+    const XMLCh* srcPtr = nameToOpen;
     while (*srcPtr)
     {
         if (*srcPtr == chYenSign ||
@@ -294,7 +292,7 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     //
     if (*srcPtr)
     {
-        tmpUName = XMLString::replicate(fileName);
+        tmpUName = XMLString::replicate(nameToOpen, fgMemoryManager);
 
         XMLCh* tmpPtr = tmpUName;
         while (*tmpPtr)
@@ -326,7 +324,7 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
         //  We are Win 95 / 98.  Take the Unicode file name back to (char *)
         //    so that we can open it.
         //
-        char* tmpName = XMLString::transcode(nameToOpen);
+        char* tmpName = XMLString::transcode(nameToOpen, fgMemoryManager);
         retVal = ::CreateFileA
             (
             tmpName
@@ -337,11 +335,11 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
             , FILE_FLAG_SEQUENTIAL_SCAN
             , 0
             );
-        delete [] tmpName;
+        fgMemoryManager->deallocate(tmpName);//delete [] tmpName;
     }
 
     if (tmpUName)
-        delete [] tmpUName;
+        fgMemoryManager->deallocate(tmpUName);//delete [] tmpUName;
 
     if (retVal == INVALID_HANDLE_VALUE)
         return 0;
@@ -358,7 +356,7 @@ FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
         , 0              // no shared write
         , 0
         , CREATE_ALWAYS
-        , FILE_FLAG_WRITE_THROUGH
+        , FILE_ATTRIBUTE_NORMAL
         , 0
     );
 
@@ -410,7 +408,7 @@ FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
     //
     if (*srcPtr)
     {
-        tmpUName = XMLString::replicate(fileName);
+        tmpUName = XMLString::replicate(fileName, fgMemoryManager);
 
         XMLCh* tmpPtr = tmpUName;
         while (*tmpPtr)
@@ -432,7 +430,7 @@ FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
             , 0              // no shared write
             , 0
             , CREATE_ALWAYS
-            , FILE_FLAG_WRITE_THROUGH
+            , FILE_ATTRIBUTE_NORMAL
             , 0
             );
     }
@@ -442,7 +440,7 @@ FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
         //  We are Win 95 / 98.  Take the Unicode file name back to (char *)
         //    so that we can open it.
         //
-        char* tmpName = XMLString::transcode(nameToOpen);
+        char* tmpName = XMLString::transcode(nameToOpen, fgMemoryManager);
         retVal = ::CreateFileA
             (
             tmpName
@@ -450,14 +448,14 @@ FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
             , 0              // no shared write
             , 0
             , CREATE_ALWAYS
-            , FILE_FLAG_WRITE_THROUGH
+            , FILE_ATTRIBUTE_NORMAL
             , 0
             );
-        delete [] tmpName;
+        fgMemoryManager->deallocate(tmpName);//delete [] tmpName;
     }
 
     if (tmpUName)
-        delete [] tmpUName;
+        fgMemoryManager->deallocate(tmpUName);//delete [] tmpUName;
 
     if (retVal == INVALID_HANDLE_VALUE)
         return 0;
@@ -474,8 +472,10 @@ FileHandle XMLPlatformUtils::openStdInHandle()
     //  shutting it would prevent any further output.
     //
     HANDLE stdInOrg = ::GetStdHandle(STD_INPUT_HANDLE);
-    if (stdInOrg == INVALID_HANDLE_VALUE)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile);
+    if (stdInOrg == INVALID_HANDLE_VALUE) {
+        XMLCh stdinStr[] = {chLatin_s, chLatin_t, chLatin_d, chLatin_i, chLatin_n, chNull};
+        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, stdinStr);
+    }
 
     HANDLE retHandle;
     if (!::DuplicateHandle
@@ -554,7 +554,8 @@ void XMLPlatformUtils::resetFile(FileHandle theFile)
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: File system methods
 // ---------------------------------------------------------------------------
-XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
+XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
+                                     MemoryManager* const manager)
 {
     //
     //  If we are on NT, then use wide character APIs, else use ASCII APIs.
@@ -572,13 +573,13 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
             return 0;
 
         // Return a copy of the path
-        return XMLString::replicate(tmpPath);
+        return XMLString::replicate(tmpPath, manager);
     }
      else
     {
         // Transcode the incoming string
-        char* tmpSrcPath = XMLString::transcode(srcPath);
-        ArrayJanitor<char> janSrcPath(tmpSrcPath);
+        char* tmpSrcPath = XMLString::transcode(srcPath, fgMemoryManager);
+        ArrayJanitor<char> janSrcPath(tmpSrcPath, fgMemoryManager);
 
         // Use a local buffer that is big enough for the largest legal path
         const unsigned int bufSize = 511;
@@ -589,7 +590,7 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
             return 0;
 
         // Return a transcoded copy of the path
-        return XMLString::transcode(tmpPath);
+        return XMLString::transcode(tmpPath, manager);
     }
 }
 
@@ -625,106 +626,47 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
     return true;
 }
 
-
-XMLCh* XMLPlatformUtils::weavePaths(const   XMLCh* const    basePath
-                                    , const XMLCh* const    relativePath)
-
+XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
-    // Create a buffer as large as both parts and empty it
-    XMLCh* tmpBuf = new XMLCh[XMLString::stringLen(basePath)
-                              + XMLString::stringLen(relativePath)
-                              + 2];
-    *tmpBuf = 0;
-
     //
-    //  If we have no base path, then just take the relative path as
-    //  is.
+    //  If we are on NT, then use wide character APIs, else use ASCII APIs.
+    //  We have to do it manually since we are only built in ASCII mode from
+    //  the standpoint of the APIs.
     //
-    if (!basePath)
+    if (gOnNT)
     {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
-    }
+        // Use a local buffer that is big enough for the largest legal path
+        const unsigned int bufSize = 1024;
+        XMLCh tmpPath[bufSize + 1];
 
-    if (!*basePath)
+        if (!::GetCurrentDirectoryW(bufSize, (LPWSTR)tmpPath))
+            return 0;
+
+        // Return a copy of the path
+        return XMLString::replicate(tmpPath, manager);
+    }
+     else
     {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
+        // Use a local buffer that is big enough for the largest legal path
+        const unsigned int bufSize = 511;
+        char tmpPath[511 + 1];
+
+        char* namePart = 0;
+        if (!::GetCurrentDirectoryA(bufSize, tmpPath))
+            return 0;
+
+        // Return a transcoded copy of the path
+        return XMLString::transcode(tmpPath, manager);
     }
-
-    const XMLCh* basePtr = basePath + (XMLString::stringLen(basePath) - 1);
-    while ((basePtr >= basePath)  &&  ((isAnySlash(*basePtr) == false)))
-    {
-        basePtr--;
-    }
-
-    // There is no relevant base path, so just take the relative part
-    if (basePtr < basePath)
-    {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
-    }
-
-    // After this, make sure the buffer gets handled if we exit early
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
-
-    //
-    //  We have some path part, so we need to check to see if we ahve to
-    //  weave any of the parts together.
-    //
-    const XMLCh* pathPtr = relativePath;
-    while (true)
-    {
-        // If it does not start with some period, then we are done
-        if (*pathPtr != chPeriod)
-            break;
-
-        unsigned int periodCount = 1;
-        pathPtr++;
-        if (*pathPtr == chPeriod)
-        {
-            pathPtr++;
-            periodCount++;
-        }
-
-        // Has to be followed by a \ or / or the null to mean anything
-        if (isAnySlash(*pathPtr) == false &&  *pathPtr)
-        {
-            break;
-        }
-        if (*pathPtr)
-            pathPtr++;
-
-        // If its one period, just eat it, else move backwards in the base
-        if (periodCount == 2)
-        {
-            basePtr--;
-            while ((basePtr >= basePath) &&  (isAnySlash(*basePtr) == false))
-            {
-                basePtr--;
-            }
-
-            // The base cannot provide enough levels, so its in error/
-            if (basePtr < basePath)
-                ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_BasePathUnderflow);
-        }
-    }
-
-    // Copy the base part up to the base pointer
-    XMLCh* bufPtr = tmpBuf;
-    const XMLCh* tmpPtr = basePath;
-    while (tmpPtr <= basePtr)
-        *bufPtr++ = *tmpPtr++;
-
-    // And then copy on the rest of our path
-    XMLString::copyString(bufPtr, pathPtr);
-
-    // Orphan the buffer and return it
-    janBuf.orphan();
-    return tmpBuf;
 }
 
-
+inline bool XMLPlatformUtils::isAnySlash(XMLCh c)
+{
+    return c == chBackSlash    ||
+           c == chForwardSlash ||
+           c == chYenSign      ||
+           c == chWonSign;
+}
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Timing Methods
@@ -785,7 +727,7 @@ XMLPlatformUtils::compareAndSwap(       void**      toFill
     //  otherwise we are back to using assembler.
     //  (But only if building with compilers that support inline assembler.)
     //
-    #if defined(_MSC_VER) || defined(__BCPLUSPLUS__)
+    #if (defined(_MSC_VER) || defined(__BCPLUSPLUS__)) && !defined(XERCES_NO_ASM)
 
     void*   result;
     __asm
@@ -862,6 +804,8 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
     return new InMemMsgLoader(msgDomain);
 #elif defined (XML_USE_WIN32_MSGLOADER)
     return new Win32MsgLoader(msgDomain);
+#elif defined (XML_USE_ICU_MESSAGELOADER)
+    return new ICUMsgLoader(msgDomain);
 #else
     #error You must provide a message loader
 #endif
@@ -931,3 +875,7 @@ void XMLPlatformUtils::platformTerm()
 {
     // We don't have any temrination requirements for win32 at this time
 }
+
+#include <xercesc/util/LogicalPath.c>
+
+XERCES_CPP_NAMESPACE_END
