@@ -56,6 +56,27 @@
 
 /*
  * $Log: CygwinTransService.cpp,v $
+ * Revision 1.8  2003/05/17 16:32:17  knoaman
+ * Memory manager implementation : transcoder update.
+ *
+ * Revision 1.7  2003/05/16 14:04:46  neilg
+ * fix compilation error
+ *
+ * Revision 1.6  2003/05/16 06:01:57  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.5  2003/05/15 18:47:02  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.4  2003/03/09 17:02:20  peiyongz
+ * PanicHandler
+ *
+ * Revision 1.3  2003/03/07 18:15:57  tng
+ * Return a reference instead of void for operator=
+ *
+ * Revision 1.2  2002/11/04 15:14:32  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.1  2002/08/19 18:35:56  tng
  * [Bug 6467] Installing Xerces C++ on cygwin environment.
  *
@@ -75,7 +96,7 @@
 #include <windows.h>
 #include <stdlib.h>
 
-
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Local, const data
@@ -132,7 +153,7 @@ static const LCID gLocaleId =
 //  This is the simple CPMapEntry class. It just contains an encoding name
 //  and a code page for that encoding.
 // ---------------------------------------------------------------------------
-class CPMapEntry
+class CPMapEntry : public XMemory
 {
 public :
     // -----------------------------------------------------------------------
@@ -170,7 +191,7 @@ private :
     // -----------------------------------------------------------------------
     CPMapEntry();
     CPMapEntry(const CPMapEntry&);
-    void operator=(const CPMapEntry&);
+    CPMapEntry& operator=(const CPMapEntry&);
 
 
     // -----------------------------------------------------------------------
@@ -206,7 +227,10 @@ CPMapEntry::CPMapEntry( const   char* const     encodingName
     // Transcode the name to Unicode and store that copy
     const unsigned int srcLen = strlen(encodingName);
     const unsigned int targetLen = ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, encodingName, srcLen, NULL, 0);
-    fEncodingName = new XMLCh[targetLen + 1];
+    fEncodingName = (XMLCh*) XMLPlatformUtils::fgMemoryManager->allocate
+    (
+        (targetLen + 1) * sizeof(XMLCh)
+    );//new XMLCh[targetLen + 1];
     ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, encodingName, srcLen, (LPWSTR)fEncodingName, targetLen);
     fEncodingName[targetLen] = 0;
 
@@ -225,7 +249,7 @@ CPMapEntry::CPMapEntry( const   XMLCh* const    encodingName
     , fCPId(cpId)
     , fIEId(ieId)
 {
-    fEncodingName = XMLString::replicate(encodingName);
+		fEncodingName = XMLString::replicate(encodingName, XMLPlatformUtils::fgMemoryManager);
 
     //
     //  Upper case it because we are using a hash table and need to be
@@ -237,7 +261,7 @@ CPMapEntry::CPMapEntry( const   XMLCh* const    encodingName
 
 CPMapEntry::~CPMapEntry()
 {
-    delete [] fEncodingName;
+    XMLPlatformUtils::fgMemoryManager->deallocate(fEncodingName);//delete [] fEncodingName;
 }
 
 
@@ -334,7 +358,7 @@ CygwinTransService::CygwinTransService()
             , KEY_READ
             , &encodingKey))
         {
-            XMLPlatformUtils::panic(XMLPlatformUtils::Panic_NoTransService);
+            XMLPlatformUtils::panic(PanicHandler::Panic_NoTransService);
         }
 
         //
@@ -362,7 +386,7 @@ CygwinTransService::CygwinTransService()
                 , (unsigned char*)&CPId
                 , &theSize) != ERROR_SUCCESS)
             {
-                XMLPlatformUtils::panic(XMLPlatformUtils::Panic_NoTransService);
+                XMLPlatformUtils::panic(PanicHandler::Panic_NoTransService);
             }
 
             //
@@ -381,7 +405,7 @@ CygwinTransService::CygwinTransService()
                     , (unsigned char*)&IEId
                     , &theSize) != ERROR_SUCCESS)
                 {
-                    XMLPlatformUtils::panic(XMLPlatformUtils::Panic_NoTransService);
+                    XMLPlatformUtils::panic(PanicHandler::Panic_NoTransService);
                 }
 
                 CPMapEntry* newEntry = new CPMapEntry(nameBuf, CPId, IEId);
@@ -427,7 +451,7 @@ CygwinTransService::CygwinTransService()
             , KEY_READ
             , &encodingKey))
         {
-            XMLPlatformUtils::panic(XMLPlatformUtils::Panic_NoTransService);
+            XMLPlatformUtils::panic(PanicHandler::Panic_NoTransService);
         }
 
         //
@@ -439,7 +463,10 @@ CygwinTransService::CygwinTransService()
         {
             const unsigned int srcLen = strlen(aliasBuf);
             const unsigned int targetLen = ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, aliasBuf, srcLen, NULL, 0);
-            XMLCh* uniAlias = new XMLCh[targetLen + 1];
+            XMLCh* uniAlias = (XMLCh*) XMLPlatformUtils::fgMemoryManager->allocate
+            (
+                (targetLen + 1) * sizeof(XMLCh)
+            );//new XMLCh[targetLen + 1];
             ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, aliasBuf, srcLen, (LPWSTR)uniAlias, targetLen);
             uniAlias[targetLen] = 0;
             ::LCMapStringW( gLocaleId, LCMAP_UPPERCASE, (LPCWSTR)uniAlias, targetLen, (LPWSTR)uniAlias, targetLen);
@@ -459,7 +486,7 @@ CygwinTransService::CygwinTransService()
                     fCPMap->put((void*)newEntry->getEncodingName(), newEntry);
                 }
             }
-            delete [] uniAlias;
+            XMLPlatformUtils::fgMemoryManager->deallocate(uniAlias);//delete [] uniAlias;
         }
 
         // And now close the subkey handle and bump the subkey index
@@ -533,8 +560,14 @@ int CygwinTransService::auxCompareString( const XMLCh* const comp1
     if ( ignoreCase )
     {
         // Copy const parameter strings (plus terminating nul) into locals.
-        firstBuf = new XMLCh[ ++len];
-        secondBuf = new XMLCh[ ++otherLen];
+        firstBuf = (XMLCh*) XMLPlatformUtils::fgMemoryManager->allocate
+        (
+            (++len) * sizeof(XMLCh)
+        );//new XMLCh[ ++len];
+        secondBuf = (XMLCh*) XMLPlatformUtils::fgMemoryManager->allocate
+        (
+            (++otherLen) * sizeof(XMLCh)
+        );//new XMLCh[ ++otherLen];
         memcpy( firstBuf, comp1, len * sizeof(XMLCh));
         memcpy( secondBuf, comp2, otherLen * sizeof(XMLCh));
 
@@ -561,8 +594,8 @@ int CygwinTransService::auxCompareString( const XMLCh* const comp1
     // Clean-up buffers, equivalent to if ( ignoreCase )
     if ( firstBuf )
     {
-        delete [] firstBuf;
-        delete [] secondBuf;
+        XMLPlatformUtils::fgMemoryManager->deallocate(firstBuf);//delete [] firstBuf;
+        XMLPlatformUtils::fgMemoryManager->deallocate(secondBuf);//delete [] secondBuf;
     }
 
     return theResult;
@@ -663,9 +696,10 @@ bool CygwinTransService::isAlias(const   HKEY            encodingKey
 
 
 XMLTranscoder*
-CygwinTransService::makeNewXMLTranscoder(const   XMLCh* const            encodingName
+CygwinTransService::makeNewXMLTranscoder(const   XMLCh* const           encodingName
                                         ,       XMLTransService::Codes& resValue
-                                        , const unsigned int            blockSize)
+                                        , const unsigned int            blockSize
+                                        ,       MemoryManager* const    manager)
 {
     const unsigned int upLen = 1024;
     XMLCh upEncoding[upLen + 1];
@@ -694,12 +728,13 @@ CygwinTransService::makeNewXMLTranscoder(const   XMLCh* const            encodin
     }
 
     // We found it, so return a Cygwin transcoder for this encoding
-    return new CygwinTranscoder
+    return new (manager) CygwinTranscoder
     (
         encodingName
         , theEntry->getWinCP()
         , theEntry->getIEEncoding()
         , blockSize
+        , manager
     );
 }
 
@@ -723,9 +758,10 @@ CygwinTransService::makeNewXMLTranscoder(const   XMLCh* const            encodin
 CygwinTranscoder::CygwinTranscoder(const  XMLCh* const    encodingName
                                 , const unsigned int    winCP
                                 , const unsigned int    ieCP
-                                , const unsigned int    blockSize) :
+                                , const unsigned int    blockSize
+                                , MemoryManager* const manager) :
 
-    XMLTranscoder(encodingName, blockSize)
+    XMLTranscoder(encodingName, blockSize, manager)
     , fIECP(ieCP)
     , fWinCP(winCP)
 {
@@ -1015,6 +1051,36 @@ char* CygwinLCPTranscoder::transcode(const XMLCh* const toTranscode)
     return retVal;
 }
 
+char* CygwinLCPTranscoder::transcode(const XMLCh* const toTranscode,
+                                     MemoryManager* const manager)
+{
+    if (!toTranscode)
+        return 0;
+
+    char* retVal = 0;
+    if (*toTranscode)
+    {
+        // Calc the needed size
+        const unsigned int neededLen = ::WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)toTranscode, -1, NULL, 0, NULL, NULL);
+        if (neededLen == (unsigned int)-1)
+            return 0;
+
+        // Allocate a buffer of that size plus one for the null and transcode
+        // Returned length of WideCharToMultiByte includes terminating NUL.
+        retVal = (char*) manager->allocate((neededLen+1) * sizeof(char)); //new char[neededLen+1];
+        ::WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)toTranscode, -1, retVal, neededLen+1, NULL, NULL);
+
+        // And cap it off anyway just to make sure
+        retVal[neededLen] = 0;
+    }
+    else
+    {
+        retVal = (char*) manager->allocate(sizeof(char)); //new char[1];
+        retVal[0] = 0;
+    }
+    return retVal;
+}
+
 
 XMLCh* CygwinLCPTranscoder::transcode(const char* const toTranscode)
 {
@@ -1044,6 +1110,35 @@ XMLCh* CygwinLCPTranscoder::transcode(const char* const toTranscode)
     return retVal;
 }
 
+XMLCh* CygwinLCPTranscoder::transcode(const char* const toTranscode,
+                                      MemoryManager* const manager)
+{
+    if (!toTranscode)
+        return 0;
+
+    XMLCh* retVal = 0;
+    if (*toTranscode)
+    {
+        // Calculate the buffer size required
+        const unsigned int neededLen = ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, toTranscode, -1, NULL, 0);
+        if (neededLen == (unsigned int)-1)
+            return 0;
+
+        // Allocate a buffer of that size plus one for the null and transcode
+        retVal = (XMLCh*) manager->allocate((neededLen + 1) * sizeof(XMLCh)); //new XMLCh[neededLen + 1];
+        ::MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, toTranscode, -1, (LPWSTR)retVal, neededLen + 1);
+
+        // Cap it off just to make sure. We are so paranoid!
+        retVal[neededLen] = 0;
+    }
+    else
+    {
+        retVal = (XMLCh*) manager->allocate(sizeof(XMLCh)); //new XMLCh[1];
+        retVal[0] = 0;
+    }
+    return retVal;
+}
+
 
 bool CygwinLCPTranscoder::transcode( const   char* const    toTranscode
                                     ,       XMLCh* const    toFill
@@ -1096,7 +1191,7 @@ bool CygwinLCPTranscoder::transcode( const  XMLCh* const    toTranscode
 }
 
 
-
+XERCES_CPP_NAMESPACE_END
 
 
 
