@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,17 +55,17 @@
  */
 
 /*
- * $Id: XMLUni.cpp,v 1.17 2002/08/13 20:29:07 knoaman Exp $
+ * $Id: XMLUni.cpp,v 1.30 2003/04/17 21:58:50 neilg Exp $
  */
 
 
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
-#include <xercesc/util/XercesDefs.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLUni.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  XMLUni: Static data
@@ -178,6 +178,16 @@ const XMLCh XMLUni::fgIBM1140EncodingString2[] =
     chLatin_I, chLatin_B, chLatin_M, chDigit_0, chDigit_1, chDigit_1, chDigit_4, chDigit_0, chNull
 };
 
+const XMLCh XMLUni::fgIBM1140EncodingString3[] =
+{
+    chLatin_C, chLatin_C, chLatin_S, chLatin_I, chLatin_D, chDigit_0, chDigit_1, chDigit_1, chDigit_4, chDigit_0, chNull
+};
+
+const XMLCh XMLUni::fgIBM1140EncodingString4[] =
+{
+    chLatin_C, chLatin_P, chDigit_0, chDigit_1, chDigit_1, chDigit_4, chDigit_0, chNull
+};
+
 const XMLCh XMLUni::fgIESString[] =
 {
     chLatin_I, chLatin_E, chLatin_S, chNull
@@ -253,6 +263,26 @@ const XMLCh XMLUni::fgISO88591EncodingString8[] =
     chLatin_L, chLatin_A, chLatin_T, chLatin_I, chLatin_N, chUnderscore, chDigit_1, chNull
 };
 
+const XMLCh XMLUni::fgISO88591EncodingString9[] =
+{
+    chLatin_C, chLatin_P, chDigit_8, chDigit_1, chDigit_9, chNull
+};
+
+const XMLCh XMLUni::fgISO88591EncodingString10[] =
+{
+    chLatin_C, chLatin_S, chLatin_I, chLatin_S, chLatin_O, chLatin_L, chLatin_A, chLatin_T, chLatin_I, chLatin_N, chDigit_1, chNull
+};
+
+const XMLCh XMLUni::fgISO88591EncodingString11[] =
+{
+    chLatin_I, chLatin_S, chLatin_O, chDash, chLatin_I, chLatin_R, chDash, chDigit_1, chDigit_0, chDigit_0, chNull
+};
+
+const XMLCh XMLUni::fgISO88591EncodingString12[] =
+{
+    chLatin_L, chDigit_1, chNull
+};
+
 const XMLCh XMLUni::fgLocalHostString[] =
 {
     chLatin_l, chLatin_o, chLatin_c, chLatin_a, chLatin_l, chLatin_h, chLatin_o, chLatin_s, chLatin_t, chNull
@@ -313,11 +343,16 @@ const XMLCh XMLUni::fgStandaloneString[] =
     chLatin_s, chLatin_t, chLatin_a, chLatin_n, chLatin_d, chLatin_a, chLatin_l, chLatin_o, chLatin_n, chLatin_e, chNull
 };
 
-const XMLCh XMLUni::fgSupportedVersion[] =
+const XMLCh XMLUni::fgVersion1_0[] =
 {
     chDigit_1, chPeriod, chDigit_0, chNull
 };
 
+const XMLCh XMLUni::fgVersion1_1[] =
+{
+    chDigit_1, chPeriod, chDigit_1, chNull
+};
+
 const XMLCh XMLUni::fgSysIDString[] =
 {
     chLatin_S, chLatin_Y, chLatin_S, chLatin_T, chLatin_E, chLatin_M, chNull
@@ -608,14 +643,63 @@ const XMLCh XMLUni::fgXMLURIName[] =
 
 const XMLCh XMLUni::fgYesString[] =
 {
-    chLatin_y, chLatin_e, chLatin_s, chNull
+        chLatin_y, chLatin_e, chLatin_s, chNull
 };
 
 const XMLCh XMLUni::fgZeroLenString[] = { chNull };
 
 const XMLCh XMLUni::fgDTDEntityString[] =
 {
-    chOpenSquare, chLatin_d, chLatin_t, chLatin_d, chCloseSquare, chNull
+        chOpenSquare, chLatin_d, chLatin_t, chLatin_d, chCloseSquare, chNull
+};
+
+const XMLCh XMLUni::fgAmp[] =
+{
+        chLatin_a, chLatin_m, chLatin_p, chNull
+};
+
+const XMLCh XMLUni::fgLT[] =
+{
+        chLatin_l, chLatin_t, chNull
+};
+
+const XMLCh XMLUni::fgGT[] =
+{
+        chLatin_g, chLatin_t, chNull
+};
+
+const XMLCh XMLUni::fgQuot[] =
+{
+        chLatin_q, chLatin_u, chLatin_o, chLatin_t, chNull
+};
+
+const XMLCh XMLUni::fgApos[] =
+{
+        chLatin_a, chLatin_p, chLatin_o, chLatin_s, chNull
+};
+
+const XMLCh XMLUni::fgWFXMLScanner[] =
+{
+        chLatin_W, chLatin_F, chLatin_X, chLatin_M, chLatin_L, chLatin_S
+    ,   chLatin_c, chLatin_a, chLatin_n, chLatin_n, chLatin_e, chLatin_r, chNull
+};
+
+const XMLCh XMLUni::fgIGXMLScanner[] =
+{
+        chLatin_I, chLatin_G, chLatin_X, chLatin_M, chLatin_L, chLatin_S
+    ,   chLatin_c, chLatin_a, chLatin_n, chLatin_n, chLatin_e, chLatin_r, chNull
+};
+
+const XMLCh XMLUni::fgSGXMLScanner[] =
+{
+        chLatin_S, chLatin_G, chLatin_X, chLatin_M, chLatin_L, chLatin_S
+    ,   chLatin_c, chLatin_a, chLatin_n, chLatin_n, chLatin_e, chLatin_r, chNull
+};
+
+const XMLCh XMLUni::fgDGXMLScanner[] =
+{
+        chLatin_D, chLatin_G, chLatin_X, chLatin_M, chLatin_L, chLatin_S
+    ,   chLatin_c, chLatin_a, chLatin_n, chLatin_n, chLatin_e, chLatin_r, chNull
 };
 
 //Exception strings
@@ -954,6 +1038,33 @@ const XMLCh XMLUni::fgXercesValidationErrorAsFatal[] =
     ,   chLatin_f, chLatin_a, chLatin_t, chLatin_a, chLatin_l, chNull
 };
 
+//Xerces: http://apache.org/xml/features/calculate-src-ofs
+const XMLCh XMLUni::fgXercesCalculateSrcOfs[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e
+    ,   chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s
+    ,   chForwardSlash, chLatin_c, chLatin_a, chLatin_l, chLatin_c, chLatin_u
+    ,   chLatin_l, chLatin_a, chLatin_t, chLatin_e, chDash, chLatin_s, chLatin_r
+    ,   chLatin_c, chDash, chLatin_o, chLatin_f, chLatin_s, chNull
+};
+
+//Xerces: http://apache.org/xml/features/standard-uri-conformant
+const XMLCh XMLUni::fgXercesStandardUriConformant[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e
+    ,   chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s
+    ,   chForwardSlash, chLatin_s, chLatin_t, chLatin_a, chLatin_n, chLatin_d
+    ,   chLatin_a, chLatin_r, chLatin_d, chDash, chLatin_u, chLatin_r, chLatin_i
+    ,   chDash, chLatin_c, chLatin_o, chLatin_n, chLatin_f, chLatin_o, chLatin_r
+    ,   chLatin_m, chLatin_a, chLatin_n, chLatin_t, chNull
+};
+
 
 //Property
 //Xerces: http://apache.org/xml/properties/schema/external-schemaLocation
@@ -971,6 +1082,21 @@ const XMLCh XMLUni::fgXercesSchemaExternalSchemaLocation[] =
     ,   chLatin_o, chLatin_c, chLatin_a, chLatin_t, chLatin_i, chLatin_o, chLatin_n, chNull
 };
 
+//Property
+//Xerces: http://apache.org/xml/properties/security-manager
+const XMLCh XMLUni::fgXercesSecurityManager[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_p, chLatin_r
+    ,   chLatin_o, chLatin_p, chLatin_e, chLatin_r, chLatin_t, chLatin_i
+    ,   chLatin_e, chLatin_s, chForwardSlash, chLatin_s, chLatin_e, chLatin_c
+    ,   chLatin_u, chLatin_r, chLatin_i, chLatin_t, chLatin_y, chDash
+    ,   chLatin_m, chLatin_a, chLatin_n, chLatin_a, chLatin_g, chLatin_e, chLatin_r
+    ,   chNull
+};
+
 //Property
 //Xerces: http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation
 const XMLCh XMLUni::fgXercesSchemaExternalNoNameSpaceSchemaLocation[] =
@@ -989,6 +1115,20 @@ const XMLCh XMLUni::fgXercesSchemaExternalNoNameSpaceSchemaLocation[] =
     ,   chLatin_t, chLatin_i, chLatin_o, chLatin_n, chNull
 };
 
+//Property
+//Xerces: http://apache.org/xml/properties/scannerName
+const XMLCh XMLUni::fgXercesScannerName[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h
+    ,   chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash
+    ,   chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_p, chLatin_r
+    ,   chLatin_o, chLatin_p, chLatin_e, chLatin_r, chLatin_t, chLatin_i
+    ,   chLatin_e, chLatin_s, chForwardSlash, chLatin_s, chLatin_c, chLatin_a
+    ,   chLatin_n, chLatin_n, chLatin_e, chLatin_r, chLatin_N, chLatin_a
+    ,   chLatin_m, chLatin_e, chNull
+};
+
 //Xerces: http://apache.org/xml/features/dom/user-adopts-DOMDocument
 const XMLCh XMLUni::fgXercesUserAdoptsDOMDocument[] =
 {
@@ -1197,3 +1337,38 @@ const XMLCh XMLUni::fgDOMWRTWhitespaceInElementContent[] =
 	chLatin_e, chLatin_n, chLatin_t, chDash,    chLatin_c, chLatin_o,
 	chLatin_n, chLatin_t, chLatin_e, chLatin_n, chLatin_t, chNull
 };
+
+//Xerces: http://apache.org/xml/features/dom/byte-order-mark
+const XMLCh XMLUni::fgDOMWRTBOM[] =
+{
+    chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash,
+    chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c, chLatin_h,
+    chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g, chForwardSlash,
+    chLatin_x, chLatin_m, chLatin_l, chForwardSlash, chLatin_f, chLatin_e,
+    chLatin_a, chLatin_t, chLatin_u, chLatin_r, chLatin_e, chLatin_s,
+    chForwardSlash, chLatin_d, chLatin_o, chLatin_m, chForwardSlash,
+    chLatin_b, chLatin_y, chLatin_t, chLatin_e, chDash,
+    chLatin_o, chLatin_r, chLatin_d, chLatin_e, chLatin_r, chDash,
+    chLatin_m, chLatin_a, chLatin_r, chLatin_k, chNull
+};
+
+// en_US
+const char XMLUni::fgXercescDefaultLocale[] = "en_US";
+
+// DOM Message Domain
+const XMLCh XMLUni::fgXMLDOMMsgDomain[] =
+{
+        chLatin_h, chLatin_t, chLatin_t, chLatin_p, chColon, chForwardSlash
+    ,   chForwardSlash, chLatin_a, chLatin_p, chLatin_a, chLatin_c
+    ,   chLatin_h, chLatin_e, chPeriod, chLatin_o, chLatin_r, chLatin_g
+    ,   chForwardSlash, chLatin_x, chLatin_m, chLatin_l, chForwardSlash
+    ,   chLatin_m, chLatin_e, chLatin_s, chLatin_s, chLatin_a, chLatin_g
+    ,   chLatin_e, chLatin_s, chForwardSlash, chLatin_X, chLatin_M, chLatin_L
+    ,   chLatin_D, chLatin_O, chLatin_M, chLatin_M, chLatin_s, chLatin_g
+    ,   chNull
+};
+
+
+
+XERCES_CPP_NAMESPACE_END
+
