@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
+ * Copyright (c) 2001-2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,6 +56,36 @@
 
 /*
  * $Log: RegularExpression.cpp,v $
+ * Revision 1.13  2003/05/25 21:42:41  knoaman
+ * Allocate/Deallocate Context::xxx only when necessary.
+ *
+ * Revision 1.12  2003/05/18 14:02:06  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.11  2003/05/16 21:37:00  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.10  2003/05/16 06:01:57  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.9  2003/05/16 00:03:10  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.8  2003/05/15 18:42:54  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.7  2003/05/12 10:08:22  gareth
+ * The correct file this time.
+ *
+ * Revision 1.5  2002/12/18 13:01:02  gareth
+ * New functionality - tokenize and replace. Fixed REVISIT for case insensitive match. Patch by Jennifer Schachter.
+ *
+ * Revision 1.4  2002/11/04 15:17:00  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.3  2002/10/15 18:56:02  knoaman
+ * [Bug 13604] while loop never terminates.
+ *
  * Revision 1.2  2002/03/18 19:29:53  knoaman
  * Change constant names to eliminate possible conflict with user defined ones.
  *
@@ -98,7 +128,6 @@
 //  Includes
 // ---------------------------------------------------------------------------
 #include <xercesc/util/regx/RegularExpression.hpp>
-#include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/util/regx/RegxUtil.hpp>
 #include <xercesc/util/regx/Match.hpp>
@@ -108,7 +137,10 @@
 #include <xercesc/util/regx/ParserForXMLSchema.hpp>
 #include <xercesc/util/Janitor.hpp>
 #include <xercesc/util/ParseException.hpp>
+#include <xercesc/util/IllegalArgumentException.hpp>
+#include <xercesc/framework/XMLBuffer.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Static member data initialization
@@ -132,7 +164,7 @@ RangeToken*          RegularExpression::fWordRange = 0;
 // ---------------------------------------------------------------------------
 //  RegularExpression::Context: Constructors and Destructor
 // ---------------------------------------------------------------------------
-RegularExpression::Context::Context() :
+RegularExpression::Context::Context(MemoryManager* const manager) :
     fInUse(false)
 	, fAdoptMatch(false)
     , fStart(0)
@@ -142,13 +174,16 @@ RegularExpression::Context::Context() :
 	, fOffsets(0)
 	, fMatch(0)
 	, fString(0)
+    , fMemoryManager(manager)
 {
 }
 
 RegularExpression::Context::~Context()
 {
-	delete [] fOffsets;
-    delete [] fString;
+	if (fOffsets)
+        fMemoryManager->deallocate(fOffsets);//delete [] fOffsets;
+
+    fMemoryManager->deallocate(fString);//delete [] fString;
 
 	if (fAdoptMatch)
 		delete fMatch;
@@ -161,8 +196,10 @@ void RegularExpression::Context::reset(const XMLCh* const string
                                        , const int start, const int limit
                                        , const int noClosures)
 {
-    delete [] fString;
-    fString = XMLString::replicate(string);
+    if (fString)
+        fMemoryManager->deallocate(fString);//delete [] fString;
+
+    fString = XMLString::replicate(string, fMemoryManager);
 	fStart = start;
 	fLimit = limit;
 	fLength = fLimit - fStart;
@@ -171,10 +208,12 @@ void RegularExpression::Context::reset(const XMLCh* const string
 		delete fMatch;
 	fMatch = 0;
 
-	if (fOffsets == 0 || fSize != noClosures) {
 
-		delete [] fOffsets;
-		fOffsets = new int[noClosures];
+	if (fSize != noClosures) {
+
+		if (fOffsets)
+            fMemoryManager->deallocate(fOffsets);//delete [] fOffsets;
+		fOffsets = (int*) fMemoryManager->allocate(noClosures * sizeof(int));//new int[noClosures];
 	}
 
 	fSize = noClosures;
@@ -214,7 +253,8 @@ bool RegularExpression::Context::nextCh(XMLInt32& ch, int& offset,
 // ---------------------------------------------------------------------------
 //  RegularExpression: Constructors and Destructors
 // ---------------------------------------------------------------------------
-RegularExpression::RegularExpression(const char* const pattern)
+RegularExpression::RegularExpression(const char* const pattern,
+                                     MemoryManager* const manager)
 	:fHasBackReferences(false),
 	 fFixedStringOnly(false),
 	 fNoGroups(0),
@@ -228,12 +268,14 @@ RegularExpression::RegularExpression(const char* const pattern)
 	 fOperations(0),
 	 fTokenTree(0),
 	 fFirstChar(0),
-     fTokenFactory(0)
+     fOpFactory(manager),
+     fTokenFactory(0),
+     fMemoryManager(manager)
 {
 	try {
 
-		XMLCh* tmpBuf = XMLString::transcode(pattern);
-        ArrayJanitor<XMLCh> janBuf(tmpBuf);
+		XMLCh* tmpBuf = XMLString::transcode(pattern, fMemoryManager);
+        ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
 		setPattern(tmpBuf);
 	}
 	catch (...) {
@@ -244,7 +286,8 @@ RegularExpression::RegularExpression(const char* const pattern)
 }
 
 RegularExpression::RegularExpression(const char* const pattern,
-									 const char* const options)
+									 const char* const options,
+                                     MemoryManager* const manager)
 	:fHasBackReferences(false),
 	 fFixedStringOnly(false),
 	 fNoGroups(0),
@@ -257,14 +300,17 @@ RegularExpression::RegularExpression(const char* const pattern,
 	 fFixedString(0),
 	 fOperations(0),
 	 fTokenTree(0),
-	 fFirstChar(0)
+	 fFirstChar(0),
+     fOpFactory(manager),
+     fTokenFactory(0),
+     fMemoryManager(manager)
 {
 	try {
 
-		XMLCh* tmpBuf = XMLString::transcode(pattern);
-		ArrayJanitor<XMLCh> janBuf(tmpBuf);
-		XMLCh* tmpOptions = XMLString::transcode(options);
-		ArrayJanitor<XMLCh> janOps(tmpOptions);
+		XMLCh* tmpBuf = XMLString::transcode(pattern, fMemoryManager);
+		ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
+		XMLCh* tmpOptions = XMLString::transcode(options, fMemoryManager);
+		ArrayJanitor<XMLCh> janOps(tmpOptions, fMemoryManager);
 		setPattern(tmpBuf, tmpOptions);
 	}
 	catch (...) {
@@ -275,7 +321,8 @@ RegularExpression::RegularExpression(const char* const pattern,
 }
 
 
-RegularExpression::RegularExpression(const XMLCh* const pattern)
+RegularExpression::RegularExpression(const XMLCh* const pattern,
+                                     MemoryManager* const manager)
 	:fHasBackReferences(false),
 	 fFixedStringOnly(false),
 	 fNoGroups(0),
@@ -288,7 +335,10 @@ RegularExpression::RegularExpression(const XMLCh* const pattern)
 	 fFixedString(0),
 	 fOperations(0),
 	 fTokenTree(0),
-	 fFirstChar(0)
+	 fFirstChar(0),
+     fOpFactory(manager),
+     fTokenFactory(0),
+     fMemoryManager(manager)
 {
 	try {
 
@@ -302,7 +352,8 @@ RegularExpression::RegularExpression(const XMLCh* const pattern)
 }
 
 RegularExpression::RegularExpression(const XMLCh* const pattern,
-									 const XMLCh* const options)
+									 const XMLCh* const options,
+                                     MemoryManager* const manager)
 	:fHasBackReferences(false),
 	 fFixedStringOnly(false),
 	 fNoGroups(0),
@@ -315,7 +366,10 @@ RegularExpression::RegularExpression(const XMLCh* const pattern,
 	 fFixedString(0),
 	 fOperations(0),
 	 fTokenTree(0),
-	 fFirstChar(0)
+	 fFirstChar(0),
+     fOpFactory(manager),
+     fTokenFactory(0),
+     fMemoryManager(manager)
 {
 	try {
 
@@ -339,12 +393,13 @@ RegularExpression::~RegularExpression() {
 void RegularExpression::setPattern(const XMLCh* const pattern,
 								   const XMLCh* const options) {
 
-    fTokenFactory = new TokenFactory();
+    fTokenFactory = new (fMemoryManager) TokenFactory(fMemoryManager);
 	fOptions = parseOptions(options);
-	fPattern = XMLString::replicate(pattern);
+	fPattern = XMLString::replicate(pattern, fMemoryManager);
 
 	RegxParser* regxParser = isSet(fOptions, XMLSCHEMA_MODE)
-		? new ParserForXMLSchema() : new RegxParser();
+		? new (fMemoryManager) ParserForXMLSchema(fMemoryManager) 
+        : new (fMemoryManager) RegxParser(fMemoryManager);
 
     if (regxParser) {
         regxParser->setTokenFactory(fTokenFactory);
@@ -361,32 +416,32 @@ void RegularExpression::setPattern(const XMLCh* const pattern,
 // ---------------------------------------------------------------------------
 bool RegularExpression::matches(const char* const expression) {
 
-    XMLCh* tmpBuf = XMLString::transcode(expression);
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
+    XMLCh* tmpBuf = XMLString::transcode(expression, fMemoryManager);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
 	return matches(tmpBuf, 0, XMLString::stringLen(tmpBuf), 0);
 }
 
 bool RegularExpression::matches(const char* const expression,
 								const int start, const int end) {
 
-	XMLCh* tmpBuf = XMLString::transcode(expression);
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
+	XMLCh* tmpBuf = XMLString::transcode(expression, fMemoryManager);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
 	return matches(tmpBuf, start, end, 0);
 }
 
 bool RegularExpression::matches(const char* const expression,
 								Match* const match)				{
 
-	XMLCh* tmpBuf = XMLString::transcode(expression);
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
+	XMLCh* tmpBuf = XMLString::transcode(expression, fMemoryManager);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
 	return matches(tmpBuf, 0, XMLString::stringLen(tmpBuf), match);
 }
 
 bool RegularExpression::matches(const char* const expression, const int start,
                                 const int end, Match* const pMatch)				{
 
-	XMLCh* tmpBuf = XMLString::transcode(expression);
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
+	XMLCh* tmpBuf = XMLString::transcode(expression, fMemoryManager);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf, fMemoryManager);
 	return matches(tmpBuf, start, end, pMatch);
 }
 
@@ -425,10 +480,10 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 		XMLMutexLock lockInit(&fMutex);
 
 		if (fContext == 0)
-			fContext = new Context();
+			fContext = new (fMemoryManager) Context(fMemoryManager);
 
 		if (fContext->fInUse) {
-			context = new Context();
+			context = new (fMemoryManager) Context(fMemoryManager);
 			tmpContext = context;
 		}
 		else {
@@ -448,7 +503,7 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 	}
 	else if (fHasBackReferences) {
 
-		lMatch = new Match();
+		lMatch = new (fMemoryManager) Match(fMemoryManager);
 		lMatch->setNoGroups(fNoGroups);
 		adoptMatch = true;
 	}
@@ -612,6 +667,258 @@ bool RegularExpression::matches(const XMLCh* const expression, const int start,
 	return false;
 }
 
+// ---------------------------------------------------------------------------
+//  RegularExpression: Tokenize methods
+// ---------------------------------------------------------------------------
+RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const char* const expression) {
+
+  XMLCh* tmpBuf = XMLString::transcode(expression);
+  ArrayJanitor<XMLCh> janBuf(tmpBuf);
+  return tokenize(tmpBuf, 0, XMLString::stringLen(tmpBuf));
+}
+
+RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const char* const expression,
+								const int start, const int end) {
+
+  XMLCh* tmpBuf = XMLString::transcode(expression);
+  ArrayJanitor<XMLCh> janBuf(tmpBuf);
+  return tokenize(tmpBuf, start, end);
+}
+
+
+
+// ---------------------------------------------------------------------------
+//  RegularExpression: Tokenize methods - Wide char version
+// ---------------------------------------------------------------------------
+RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const XMLCh* const expression) {
+  return tokenize(expression, 0, XMLString::stringLen(expression), 0);
+}
+
+RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const XMLCh* const expression,
+								                                     const int start, const int end)
+{
+  return tokenize(expression, start, end, 0);
+}
+
+RefArrayVectorOf<XMLCh>* RegularExpression::tokenize(const XMLCh* const expression, 
+                                                     const int start, const int end,
+                                                     RefVectorOf<Match> *subEx){
+  
+  if (fOperations == 0)
+	  prepare();
+
+  RefArrayVectorOf<XMLCh>* tokenStack = new (fMemoryManager) RefArrayVectorOf<XMLCh>(16, true, fMemoryManager);
+
+  Context* context = 0;
+  Context* tmpContext = 0;
+
+  int		 strLength = XMLString::stringLen(expression);
+ 
+  {
+ 	   XMLMutexLock lockInit(&fMutex);
+
+ 	   if (fContext == 0)
+ 	     fContext = new (fMemoryManager) Context(fMemoryManager);
+
+ 	   if (fContext->fInUse) {
+ 	     context = new (fMemoryManager) Context(fMemoryManager);
+ 	     tmpContext = context;
+ 	   }
+ 	   else {
+ 	     context = fContext;
+ 	   }
+
+ 	   context->reset(expression, start, end, fNoClosures);
+  }
+
+  Janitor<Context> janContext(tmpContext);
+
+  Match* lMatch = 0;
+  bool adoptMatch = false;
+
+  if (subEx || fHasBackReferences) {
+    lMatch = new (fMemoryManager) Match(fMemoryManager);
+    adoptMatch = true;
+    lMatch->setNoGroups(fNoGroups);
+  }
+
+  if (context->fAdoptMatch)
+ 	  delete context->fMatch;
+  
+  context->fMatch = lMatch;
+  context->fAdoptMatch = adoptMatch;
+
+  int tokStart = start;
+  int matchStart = start;
+
+  for (; matchStart <= end; matchStart++) { 
+  
+ 	  int matchEnd = match(context, fOperations, matchStart, 1);
+  
+ 	  if (matchEnd != -1) {
+
+ 	    if (context->fMatch != 0) {
+ 	      context->fMatch->setStartPos(0, context->fStart);
+ 	      context->fMatch->setEndPos(0, matchEnd);
+ 	    }
+
+      if (subEx){
+        subEx->addElement(lMatch);
+        lMatch = new (fMemoryManager) Match(*(context->fMatch));
+        adoptMatch = true;
+        
+        context->fAdoptMatch = adoptMatch;
+        context->fMatch = lMatch;
+      }
+  
+ 	    context->fInUse = false;
+
+      XMLCh* token;
+      if (tokStart == matchStart){
+  
+        if (tokStart == strLength){
+          tokStart--;
+          break;  
+        }
+
+        token = (XMLCh*) fMemoryManager->allocate(sizeof(XMLCh));//new XMLCh[1];
+        token[0] = chNull;
+
+        // When you tokenize using zero string, will return each
+        // token in the string. Since the zero string will also 
+        // match the start/end characters, resulting in empty 
+        // tokens, we ignore them and do not add them to the stack. 
+        if (!XMLString::equals(fPattern, &chNull)) 
+          tokenStack->addElement(token); 
+        else
+            fMemoryManager->deallocate(token);//delete[] token;
+
+      } else {
+        token = (XMLCh*) fMemoryManager->allocate
+        (
+            (matchStart + 1 - tokStart) * sizeof(XMLCh)
+        );//new XMLCh[matchStart + 1 - tokStart];
+        XMLString::subString(token, expression, tokStart, matchStart);
+        tokenStack->addElement(token);
+      } 
+
+      tokStart = matchEnd;
+
+      //decrement matchStart as will increment it at the top of the loop
+      if (matchStart < matchEnd - 1) 
+        matchStart = matchEnd - 1; 	    
+    }
+  }
+ 
+  XMLCh* token;
+ 
+  if (matchStart == tokStart + 1){
+    token = (XMLCh*) fMemoryManager->allocate(sizeof(XMLCh));//new XMLCh[1];
+    token[0] = chNull;
+  
+  } else {
+    token = (XMLCh*) fMemoryManager->allocate
+    (
+        (strLength + 1 - tokStart) * sizeof(XMLCh)
+    );//new XMLCh[strLength + 1 - tokStart];
+    XMLString::subString(token, expression, tokStart, strLength);
+  }  
+
+  if (!XMLString::equals(fPattern, &chNull)) 
+    tokenStack->addElement(token);
+  else
+    fMemoryManager->deallocate(token);//delete[] token;
+
+  return tokenStack;
+
+}
+
+
+// -----------------------------------------------------------------------
+//  RegularExpression: Replace methods
+// -----------------------------------------------------------------------
+XMLCh* RegularExpression::replace(const char* const matchString, 
+                                  const char* const replaceString){
+
+	XMLCh* tmpBuf = XMLString::transcode(matchString);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf);
+	XMLCh* tmpBuf2 = XMLString::transcode(replaceString);
+    ArrayJanitor<XMLCh> janBuf2(tmpBuf2);
+
+	return replace(tmpBuf, tmpBuf2, 0, XMLString::stringLen(tmpBuf));
+}
+
+XMLCh* RegularExpression::replace(const char* const matchString, 
+                                  const char* const replaceString,
+                                  const int start, const int end){
+
+ 	XMLCh* tmpBuf = XMLString::transcode(matchString);
+    ArrayJanitor<XMLCh> janBuf(tmpBuf);
+ 	XMLCh* tmpBuf2 = XMLString::transcode(replaceString);
+    ArrayJanitor<XMLCh> janBuf2(tmpBuf2);
+  
+  return replace(tmpBuf, tmpBuf2, start, end);
+}
+
+
+// ---------------------------------------------------------------------------
+//  RegularExpression: Replace methods - Wide char version
+// ---------------------------------------------------------------------------
+XMLCh* RegularExpression::replace(const XMLCh* const matchString, 
+                                  const XMLCh* const replaceString){
+
+  return replace(matchString, replaceString, 0, 
+                 XMLString::stringLen(matchString));
+}
+
+XMLCh* RegularExpression::replace(const XMLCh* const matchString,  
+                                  const XMLCh* const replaceString,
+                                  const int start, const int end)
+{
+  //check if matches zero length string - throw error if so
+  if (matches(XMLUni::fgZeroLenString)){
+		ThrowXML(RuntimeException, XMLExcepts::Regex_RepPatMatchesZeroString);
+  }
+      
+  RefVectorOf<Match> *subEx = new (fMemoryManager) RefVectorOf<Match>(10, true, fMemoryManager);
+	  Janitor<RefVectorOf<Match> > janSubEx(subEx);
+
+  //Call to tokenize with Match vector so that we keep track of the locations
+  //of the subExpression within each of the matches
+  RefArrayVectorOf<XMLCh>* tokenStack = tokenize(matchString, start, end, subEx);
+	  Janitor<RefArrayVectorOf<XMLCh> > janTokStack(tokenStack);
+    
+  XMLBuffer result(1023, fMemoryManager);
+  
+  int numSubEx = 0;
+  
+  if (subEx && subEx->size() > 0)
+    numSubEx = subEx->elementAt(0)->getNoGroups() - 1;
+  
+  int tokStackSize = tokenStack->size();
+  const XMLCh* curRepString = XMLString::replicate(replaceString);
+    
+  for (int i = 0; i < tokStackSize; i++){
+      
+    result.append(tokenStack->elementAt(i));
+  
+    if (i != tokStackSize - 1) {
+       
+      //if there are subExpressions, then determine the string we want to 
+      //substitute in.
+        if (numSubEx != 0) {
+            delete[] (XMLCh*)curRepString;
+            curRepString = subInExp(replaceString, matchString, subEx->elementAt(i));     
+        }
+      result.append(curRepString);
+    }
+  }  
+    
+  delete[] (XMLCh*)curRepString;
+  return XMLString::replicate(result.getRawBuffer()); 
+    
+}
+
 // ---------------------------------------------------------------------------
 //  RegularExpression: Helpers methods
 // ---------------------------------------------------------------------------
@@ -658,6 +965,7 @@ int RegularExpression::getOptionValue(const XMLCh ch) {
 	return ret;
 }
 
+
 int RegularExpression::match(Context* const context, const Op* const operations
 							 , int offset, const short direction)
 {
@@ -693,13 +1001,13 @@ int RegularExpression::match(Context* const context, const Op* const operations
 		case Op::O_ANCHOR:
 			if (!matchAnchor(context, tmpOp->getData(), offset))
 				return -1;
-			tmpOp->getNextOp();
+			tmpOp = tmpOp->getNextOp();
 			break;
 		case Op::O_BACKREFERENCE:
 			if (!matchBackReference(context, tmpOp->getData(), offset,
 									direction, ignoreCase))
 				return -1;
-			tmpOp->getNextOp();
+			tmpOp = tmpOp->getNextOp();
 			break;
 		case Op::O_STRING:
 			if (!matchString(context, tmpOp->getLiteral(), offset, direction,
@@ -808,7 +1116,6 @@ int RegularExpression::match(Context* const context, const Op* const operations
 	
 	return offset;
 }
-
 bool RegularExpression::matchChar(Context* const context,
 								  const XMLInt32 ch, int& offset,
 								  const short direction, const bool ignoreCase)
@@ -825,7 +1132,6 @@ bool RegularExpression::matchChar(Context* const context,
 
 	bool match = ignoreCase ? matchIgnoreCase(ch, strCh)
 		                    : (ch == strCh);
-	
 	if (!match)
 		return false;
 
@@ -1175,6 +1481,86 @@ Op* RegularExpression::compile(const Token* const token, Op* const next,
 	return ret;
 }
 
+/*
+ * Helper for Replace. This method prepares the replacement string by substituting
+ * in actual values for parenthesized sub expressions. 
+ *
+ * An error will be thrown if:
+ *  1) repString references an undefined subExpression
+ *  2) there is an unescaped chDollar which is not followed by a digit
+ *
+ */
+const XMLCh* RegularExpression::subInExp(const XMLCh* const repString, 
+                                         const XMLCh* const origString, 
+                                         const Match* subEx){
+
+  int numSubExp = subEx->getNoGroups() - 1;
+
+  if (numSubExp == 0)
+    return XMLString::replicate(repString);
+  
+  bool notEscaped = true;                 
+  
+  XMLBuffer newString(1023, fMemoryManager);                   
+  
+  XMLCh indexStr[2]; //holds the string rep of a 
+
+  indexStr[1] = chNull;
+  int index = -1;
+
+  for (const XMLCh* ptr = repString; *ptr != chNull; ptr++){
+
+    if ((*ptr == chDollarSign) && notEscaped) {
+      
+      ptr++;
+      
+      //check that after the $ is a digit 
+      if (!XMLString::isDigit(*ptr)){
+       
+        //invalid replace string - $ must be followed by a digit
+				ThrowXML(RuntimeException, XMLExcepts::Regex_InvalidRepPattern);
+      }
+        
+      indexStr[0] = *ptr;                     //get the digit 
+      index = XMLString::parseInt(indexStr);  //convert it to an int
+
+      //now check that the index is legal
+      if (index > numSubExp){
+				ThrowXML(RuntimeException, XMLExcepts::Regex_InvalidRepPattern);
+      }
+        
+      int start = subEx->getStartPos(index);
+      int end = subEx->getEndPos(index);
+
+      //now copy the substring into the new string
+      for (int i=start; i<end; i++){
+        newString.append(origString[i]);
+      }
+          
+    } else {
+ 
+      //if you have a slash and then a character that's not a $ or /, 
+      //then it's an invalid replace string  
+      if (!notEscaped && (*ptr != chDollarSign && *ptr != chBackSlash)){
+				ThrowXML(RuntimeException, XMLExcepts::Regex_InvalidRepPattern);
+      }
+      
+      if (*ptr == chBackSlash){
+        notEscaped = false;
+        continue;
+        
+      }else   
+        notEscaped = true;  
+
+      newString.append(*ptr);
+    }
+  }
+
+  return XMLString::replicate(newString.getRawBuffer());
+       
+}
+
+
 /*
  * Prepares for matching. This method is called just before starting matching
  */
@@ -1207,28 +1593,28 @@ void RegularExpression::prepare() {
 		fFixedStringOnly = true;
 
 		if (fOperations->getOpType() == Op::O_STRING) {
-			delete [] fFixedString;
-			fFixedString = XMLString::replicate(fOperations->getLiteral());
+			fMemoryManager->deallocate(fFixedString);//delete [] fFixedString;
+			fFixedString = XMLString::replicate(fOperations->getLiteral(), fMemoryManager);
 		}
 		else{
 			
 			XMLInt32 ch = fOperations->getData();
 
 			if ( ch >= 0x10000) { // add as constant
-				delete [] fFixedString;
-				fFixedString = RegxUtil::decomposeToSurrogates(ch);
+				fMemoryManager->deallocate(fFixedString);//delete [] fFixedString;
+				fFixedString = RegxUtil::decomposeToSurrogates(ch, fMemoryManager);
 			}
 			else {
 
-				XMLCh* dummyStr = new XMLCh[2];
+				XMLCh* dummyStr = (XMLCh*) fMemoryManager->allocate(2 * sizeof(XMLCh));//new XMLCh[2];
 				dummyStr[0] = (XMLCh) fOperations->getData();
 				dummyStr[1] = chNull;
-				delete [] fFixedString;
+				fMemoryManager->deallocate(fFixedString);//delete [] fFixedString;
 				fFixedString = dummyStr;
 			}
 		}
 
-		fBMPattern = new BMPattern(fFixedString, 256,
+		fBMPattern = new (fMemoryManager) BMPattern(fFixedString, 256,
 								  isSet(fOptions, IGNORE_CASE));
 	}
 	else if (!isSet(fOptions, XMLSCHEMA_MODE) &&
@@ -1237,20 +1623,20 @@ void RegularExpression::prepare() {
 		int fixedOpts = 0;
 		Token* tok = fTokenTree->findFixedString(fOptions, fixedOpts);
 
-		delete [] fFixedString;
+		fMemoryManager->deallocate(fFixedString);//delete [] fFixedString;
 
 		fFixedString = (tok == 0) ? 0
-			: XMLString::replicate(tok->getString());
+			: XMLString::replicate(tok->getString(), fMemoryManager);
 
 		if (fFixedString != 0 && XMLString::stringLen(fFixedString) < 2) {
 
-			delete [] fFixedString;
+			fMemoryManager->deallocate(fFixedString);//delete [] fFixedString;
 			fFixedString = 0;
 		}
 		
 		if (fFixedString != 0) {
 
-			fBMPattern = new BMPattern(fFixedString, 256,
+			fBMPattern = new (fMemoryManager) BMPattern(fFixedString, 256,
 									   isSet(fixedOpts, IGNORE_CASE));
 		}
 	}
@@ -1306,6 +1692,9 @@ unsigned short RegularExpression::getCharType(const XMLCh ch) {
     return WT_OTHER;
 }
 
+
+XERCES_CPP_NAMESPACE_END
+
 /**
   *	End of file RegularExpression.cpp
   */
