@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DTDValidator.cpp,v 1.2 2002/07/11 18:55:44 knoaman Exp $
+ * $Id: DTDValidator.cpp,v 1.14 2003/05/16 21:43:19 knoaman Exp $
  */
 
 
@@ -69,6 +69,8 @@
 #include <xercesc/internal/XMLScanner.hpp>
 #include <xercesc/validators/DTD/DTDValidator.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 // ---------------------------------------------------------------------------
 //  DTDValidator: Constructors and Destructor
 // ---------------------------------------------------------------------------
@@ -199,7 +201,9 @@ bool DTDValidator::requiresNamespaces() const
 
 void
 DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
-                                , const XMLCh* const    attrValue)
+                                , const XMLCh* const    attrValue
+                                , bool                  preValidation
+                                , const XMLElementDecl* elemDecl)
 {
     //
     //  Get quick refs to lost of of the stuff in the passed objects in
@@ -215,10 +219,13 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
     //
     //  If the default type is fixed, then make sure the passed value maps
     //  to the fixed value.
+    //  If during preContentValidation, the value we are validating is the fixed value itself
+    //  so no need to compare.
+    //  Only need to do this for regular attribute value validation
     //
-    if (defType == XMLAttDef::Fixed)
+    if (defType == XMLAttDef::Fixed && !preValidation)
     {
-        if (XMLString::compareString(attrValue, valueText))
+        if (!XMLString::equals(attrValue, valueText))
             emitError(XMLValid::NotSameAsFixedValue, fullName, attrValue, valueText);
     }
 
@@ -229,6 +236,8 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
     if (type == XMLAttDef::CData)
         return;
 
+
+
     // An empty string cannot be valid for any of the other types
     if (!attrValue[0])
     {
@@ -294,6 +303,8 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
 
     XMLCh* valPtr = pszTmpVal;
 
+    bool doNamespace = getScanner()->getDoNamespaces();
+
     while (true)
     {
         // Reset the trigger flags
@@ -307,7 +318,7 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
         if (firstNameChar)
         {
             // If its not, emit and error but try to keep going
-            if (!XMLReader::isFirstNameChar(*valPtr))
+            if (!getReaderMgr()->getCurrentReader()->isFirstNameChar(*valPtr))
                 emitError(XMLValid::AttrValNotName, fullName);
             valPtr++;
         }
@@ -319,7 +330,7 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
             //  If we hit a whitespace, its either a break between two
             //  or more values, or an error if we have a single value.
             //
-            if (XMLReader::isWhitespace(*valPtr))
+            if (getReaderMgr()->getCurrentReader()->isWhitespace(*valPtr))
             {
                 if (!multipleValues)
                 {
@@ -329,7 +340,15 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
                 break;
             }
 
-            if (!XMLReader::isNameChar(*valPtr))
+            // Now this attribute can be of type
+            //     ID, IDREF, IDREFS, ENTITY, ENTITIES, NOTATION, NMTOKEN, NMTOKENS, ENUMERATION
+            //  All these must be valid XMLName
+            // If namespace is enabled, colon is not allowed in the first 6
+
+            if (doNamespace && *valPtr == chColon && firstNameChar)
+                emitError(XMLValid::ColonNotValidWithNS);
+
+            if (!getReaderMgr()->getCurrentReader()->isNameChar(*valPtr))
             {
                 emitError(XMLValid::AttrValNotName, fullName);
                 return;
@@ -363,7 +382,13 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
                 }
                  else
                 {
-                    find = new XMLRefInfo(pszTmpVal);
+                    find = new (getScanner()->getMemoryManager()) XMLRefInfo
+                    (
+                        pszTmpVal
+                        , false
+                        , false
+                        , getScanner()->getMemoryManager()
+                    );
                     getScanner()->getIDRefList()->put((void*)find->getRefName(), find);
                 }
 
@@ -373,8 +398,11 @@ DTDValidator::validateAttrValue(const   XMLAttDef*      attDef
                 //
                 if (type == XMLAttDef::ID)
                     find->setDeclared(true);
-                else
-                    find->setUsed(true);
+                else {
+                    if (!preValidation) {
+                        find->setUsed(true);
+                    }
+                }
             }
         }
          else if ((type == XMLAttDef::Entity) || (type == XMLAttDef::Entities))
@@ -519,7 +547,7 @@ void DTDValidator::preContentValidation(bool reuseGrammar,
 
                 seenId = true;
             }
-             else if (curAttDef.getType() == XMLAttDef::Notation)
+             else if (curAttDef.getType() == XMLAttDef::Notation && curAttDef.getEnumeration())
             {
                 //
                 //  We need to verify that all of its possible values
@@ -576,6 +604,8 @@ void DTDValidator::preContentValidation(bool reuseGrammar,
                 (
                     &curAttDef
                     , curAttDef.getValue()
+                    , true
+                    , &curElem
                 );
             }
         }
@@ -613,3 +643,5 @@ void DTDValidator::postParseValidation()
     //  validator.
     //
 }
+
+XERCES_CPP_NAMESPACE_END
