@@ -55,8 +55,35 @@
  */
 
 /*
- * $Id: AbstractNumericFacetValidator.cpp,v 1.3 2002/04/01 20:17:46 peiyongz Exp $
+ * $Id: AbstractNumericFacetValidator.cpp,v 1.12 2003/05/15 18:53:26 knoaman Exp $
  * $Log: AbstractNumericFacetValidator.cpp,v $
+ * Revision 1.12  2003/05/15 18:53:26  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.11  2003/02/03 16:34:35  peiyongz
+ * fix to misplaced derived and base value in error message.
+ *
+ * Revision 1.10  2003/02/02 23:55:40  peiyongz
+ * getFormattedString() added to return original and converted value.
+ *
+ * Revision 1.9  2003/01/30 21:56:22  tng
+ * Performance: call getRawData instead of toString
+ *
+ * Revision 1.8  2003/01/25 16:33:25  peiyongz
+ * wrong error code.
+ *
+ * Revision 1.7  2002/12/18 14:17:55  gareth
+ * Fix to bug #13438. When you eant a vector that calls delete[] on its members you should use RefArrayVectorOf.
+ *
+ * Revision 1.6  2002/11/26 22:19:15  peiyongz
+ * Schema Errata E2-16 maxExclusive
+ *
+ * Revision 1.5  2002/11/04 14:53:27  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.4  2002/09/24 19:44:40  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
  * Revision 1.3  2002/04/01 20:17:46  peiyongz
  * Bug#7551: Exceptions are caught by value, rather than by reference
  *
@@ -91,37 +118,31 @@
 //  Includes
 // ---------------------------------------------------------------------------
 #include <xercesc/validators/datatype/AbstractNumericFacetValidator.hpp>
-#include <xercesc/validators/schema/SchemaSymbols.hpp>
 #include <xercesc/validators/datatype/InvalidDatatypeFacetException.hpp>
 #include <xercesc/util/NumberFormatException.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 const int AbstractNumericFacetValidator::INDETERMINATE = 2;
 
 #define  REPORT_FACET_ERROR(val1, val2, except_code)    \
-  XMLCh* value1 = (val1)->toString();                   \
-  ArrayJanitor<XMLCh> jan1(value1);                     \
-  XMLCh* value2 = (val2)->toString();                   \
-  ArrayJanitor<XMLCh> jan2(value2);                     \
   ThrowXML2(InvalidDatatypeFacetException               \
           , except_code                                 \
-          , value2                                      \
-          , value1);
-
+          , val1->getFormattedString()                  \
+          , val2->getFormattedString());
 
 #define  FROM_BASE_VALUE_SPACE(val, facetFlag, except_code)   \
   if ((thisFacetsDefined & facetFlag) != 0)                   \
 {                                                             \
-    XMLCh* value1 = (val)->toString();                        \
-    ArrayJanitor<XMLCh> jan(value1);                          \
     try                                                       \
 {                                                             \
-        numBase->checkContent(value1, false);                 \
+        numBase->checkContent(val->getRawData(), false);      \
 }                                                             \
     catch ( XMLException& )                                   \
 {                                                             \
         ThrowXML1(InvalidDatatypeFacetException               \
                 , except_code                                 \
-                , value1);                                    \
+                , val->getRawData());                         \
 }                                                             \
 }
 
@@ -155,8 +176,9 @@ AbstractNumericFacetValidator::AbstractNumericFacetValidator(
                           DatatypeValidator*            const baseValidator
                         , RefHashTableOf<KVStringPair>* const facets
                         , const int                           finalSet
-                        , const ValidatorType                 type)
-:DatatypeValidator(baseValidator, facets, finalSet, type)
+                        , const ValidatorType                 type
+                        , MemoryManager* const                manager)
+:DatatypeValidator(baseValidator, facets, finalSet, type, manager)
 , fMaxInclusiveInherited(false)
 , fMaxExclusiveInherited(false)
 , fMinInclusiveInherited(false)
@@ -175,7 +197,7 @@ AbstractNumericFacetValidator::AbstractNumericFacetValidator(
 //
 //  P1. Enumeration
 //
-void AbstractNumericFacetValidator::init(RefVectorOf<XMLCh>* const enums)
+void AbstractNumericFacetValidator::init(RefArrayVectorOf<XMLCh>* const enums)
 {
 
     fStrEnumeration = enums; // save the literal value
@@ -216,14 +238,14 @@ void AbstractNumericFacetValidator::assignFacet()
         key = pair.getKey();
         value = pair.getValue();
 
-        if (XMLString::compareString(key, SchemaSymbols::fgELT_PATTERN) == 0)
+        if (XMLString::equals(key, SchemaSymbols::fgELT_PATTERN))
         {
             setPattern(value);
             if (getPattern())
                 setFacetsDefined(DatatypeValidator::FACET_PATTERN);
             // do not construct regex until needed
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MAXINCLUSIVE)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MAXINCLUSIVE))
         {
             try
             {
@@ -235,7 +257,7 @@ void AbstractNumericFacetValidator::assignFacet()
             }
             setFacetsDefined(DatatypeValidator::FACET_MAXINCLUSIVE);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MAXEXCLUSIVE)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MAXEXCLUSIVE))
         {
             try
             {
@@ -247,7 +269,7 @@ void AbstractNumericFacetValidator::assignFacet()
             }
             setFacetsDefined(DatatypeValidator::FACET_MAXEXCLUSIVE);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MININCLUSIVE)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MININCLUSIVE))
         {
             try
             {
@@ -259,7 +281,7 @@ void AbstractNumericFacetValidator::assignFacet()
             }
             setFacetsDefined(DatatypeValidator::FACET_MININCLUSIVE);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MINEXCLUSIVE)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MINEXCLUSIVE))
         {
             try
             {
@@ -271,7 +293,7 @@ void AbstractNumericFacetValidator::assignFacet()
             }
             setFacetsDefined(DatatypeValidator::FACET_MINEXCLUSIVE);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgATT_FIXED)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgATT_FIXED))
         {
             unsigned int val;
             bool         retStatus;
@@ -443,15 +465,15 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             if (((baseFixed & DatatypeValidator::FACET_MAXINCLUSIVE) != 0) &&
                  (result != 0 ))
             {
-                REPORT_FACET_ERROR(baseMaxInclusive
-                                 , thisMaxInclusive
+                REPORT_FACET_ERROR(thisMaxInclusive
+                                 , baseMaxInclusive
                                  , XMLExcepts::FACET_maxIncl_base_fixed)
             }
 
             if (result == 1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMaxInclusive
-                                 , thisMaxInclusive
+                REPORT_FACET_ERROR(thisMaxInclusive
+                                 , baseMaxInclusive
                                  , XMLExcepts::FACET_maxIncl_base_maxIncl)
             }
 
@@ -462,8 +484,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxInclusive, baseMaxExclusive);
             if (result != -1 )
             {
-                REPORT_FACET_ERROR(baseMaxExclusive
-                                 , thisMaxInclusive
+                REPORT_FACET_ERROR(thisMaxInclusive
+                                 , baseMaxExclusive
                                  , XMLExcepts::FACET_maxIncl_base_maxExcl)
             }
         }
@@ -474,8 +496,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxInclusive, baseMinInclusive);
             if (result == -1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMinInclusive
-                                 , thisMaxInclusive
+                REPORT_FACET_ERROR(thisMaxInclusive
+                                 , baseMinInclusive
                                  , XMLExcepts::FACET_maxIncl_base_minIncl)
             }
         }
@@ -485,8 +507,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxInclusive, baseMinExclusive);
             if (result != 1 )
             {
-                REPORT_FACET_ERROR(baseMinExclusive
-                                 , thisMaxInclusive
+                REPORT_FACET_ERROR(thisMaxInclusive
+                                 , baseMinExclusive
                                  , XMLExcepts::FACET_maxIncl_base_minExcl)
             }
         }
@@ -508,17 +530,39 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             if (((baseFixed & DatatypeValidator::FACET_MAXEXCLUSIVE) != 0) &&
                  (result != 0 ))
             {
-                REPORT_FACET_ERROR(baseMaxExclusive
-                                 , thisMaxExclusive
+                REPORT_FACET_ERROR(thisMaxExclusive
+                                 , baseMaxExclusive
                                  , XMLExcepts::FACET_maxExcl_base_fixed)
              }
 
             if (result == 1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMaxExclusive
-                                 , thisMaxExclusive
+                REPORT_FACET_ERROR(thisMaxExclusive
+                                 , baseMaxExclusive
                                  , XMLExcepts::FACET_maxExcl_base_maxExcl)
             }
+
+            /**
+             * Schema Errata
+             * E2-16 maxExclusive
+             *
+             *   derived type's maxExclusive must either be
+             *   1) equal to base' maxExclusive or
+             *   2) from the base type value space
+             *
+             */
+            if (result != 0)
+            {
+                FROM_BASE_VALUE_SPACE(thisMaxExclusive
+                        , DatatypeValidator::FACET_MAXEXCLUSIVE
+                        , XMLExcepts::FACET_maxExcl_notFromBase)
+            }
+        }
+        else  // base has no maxExclusive
+        {
+            FROM_BASE_VALUE_SPACE(thisMaxExclusive
+                        , DatatypeValidator::FACET_MAXEXCLUSIVE
+                        , XMLExcepts::FACET_maxExcl_notFromBase)
         }
 
         if (( baseFacetsDefined & DatatypeValidator::FACET_MAXINCLUSIVE) != 0)
@@ -526,8 +570,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxExclusive, baseMaxInclusive);
             if (result == 1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMaxInclusive
-                                 , thisMaxExclusive
+                REPORT_FACET_ERROR(thisMaxExclusive
+                                 , baseMaxInclusive
                                  , XMLExcepts::FACET_maxExcl_base_maxIncl)
             }
         }
@@ -537,8 +581,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxExclusive, baseMinExclusive);
             if (result != 1)
             {
-                REPORT_FACET_ERROR(baseMinExclusive
-                                 , thisMaxExclusive
+                REPORT_FACET_ERROR(thisMaxExclusive
+                                 , baseMinExclusive
                                  , XMLExcepts::FACET_maxExcl_base_minExcl)
             }
         }
@@ -548,9 +592,9 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMaxExclusive, baseMinInclusive);
             if (result != 1)
             {
-                REPORT_FACET_ERROR(baseMinInclusive
-                                 , thisMaxExclusive
-                                 , XMLExcepts::FACET_maxExcl_base_minExcl)
+                REPORT_FACET_ERROR(thisMaxExclusive
+                                 , baseMinInclusive
+                                 , XMLExcepts::FACET_maxExcl_base_minIncl)
             }
         }
     }
@@ -570,17 +614,41 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             if (((baseFixed & DatatypeValidator::FACET_MINEXCLUSIVE) != 0) &&
                  (result != 0 ))
             {
-                REPORT_FACET_ERROR(baseMinExclusive
-                                 , thisMinExclusive
+                REPORT_FACET_ERROR(thisMinExclusive
+                                 , baseMinExclusive
                                  , XMLExcepts::FACET_minExcl_base_fixed)
             }
 
             if (result == -1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMinExclusive
-                                 , thisMinExclusive
+                REPORT_FACET_ERROR(thisMinExclusive
+                                 , baseMinExclusive
                                  , XMLExcepts::FACET_minExcl_base_minExcl)
             }
+
+            /**
+             * Schema Errata
+             * E2-16 maxExclusive
+             *
+             *   derived type's minExclusive must either be
+             *   1) equal to base' minxExclusive or
+             *   2) from the base type value space
+             *
+             *  Note: deduced from, not explicitly expressed in the Errata
+             */
+            if (result != 0)
+            {
+                FROM_BASE_VALUE_SPACE(thisMinExclusive
+                        , DatatypeValidator::FACET_MINEXCLUSIVE
+                        , XMLExcepts::FACET_minExcl_notFromBase)
+            }
+        }
+        else // base has no minExclusive
+        {
+
+            FROM_BASE_VALUE_SPACE(thisMinExclusive
+                        , DatatypeValidator::FACET_MINEXCLUSIVE
+                        , XMLExcepts::FACET_minExcl_notFromBase)
         }
 
         if (( baseFacetsDefined & DatatypeValidator::FACET_MAXINCLUSIVE) != 0)
@@ -588,8 +656,9 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinExclusive, baseMaxInclusive);
             if (result == 1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMaxInclusive
-                                 , thisMinExclusive
+                REPORT_FACET_ERROR(thisMinExclusive
+                                 , baseMaxInclusive
+
                                  , XMLExcepts::FACET_minExcl_base_maxIncl)
             }
         }
@@ -599,8 +668,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinExclusive, baseMinInclusive);
             if (result == -1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMinInclusive
-                                 , thisMinExclusive
+                REPORT_FACET_ERROR(thisMinExclusive
+                                 , baseMinInclusive
                                  , XMLExcepts::FACET_minExcl_base_minIncl)
             }
         }
@@ -610,8 +679,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinExclusive, baseMaxExclusive);
             if (result != -1)
             {
-                REPORT_FACET_ERROR(baseMaxExclusive
-                                 , thisMinExclusive
+                REPORT_FACET_ERROR(thisMinExclusive
+                                 , baseMaxExclusive
                                  , XMLExcepts::FACET_minExcl_base_maxExcl)
             }
         }
@@ -634,15 +703,15 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             if (((baseFixed & DatatypeValidator::FACET_MININCLUSIVE) != 0) &&
                  (result != 0 ))
             {
-                REPORT_FACET_ERROR(baseMinInclusive
-                                 , thisMinInclusive
+                REPORT_FACET_ERROR(thisMinInclusive
+                                 , baseMinInclusive
                                  , XMLExcepts::FACET_minIncl_base_fixed)
             }
 
             if (result == -1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMinInclusive
-                                 , thisMinInclusive
+                REPORT_FACET_ERROR(thisMinInclusive
+                                 , baseMinInclusive
                                  , XMLExcepts::FACET_minIncl_base_minIncl)
             }
         }
@@ -652,8 +721,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinInclusive, baseMaxInclusive);
             if (result == 1 || result == INDETERMINATE)
             {
-                REPORT_FACET_ERROR(baseMaxInclusive
-                                 , thisMinInclusive
+                REPORT_FACET_ERROR(thisMinInclusive
+                                 , baseMaxInclusive
                                  , XMLExcepts::FACET_minIncl_base_maxIncl)
             }
         }
@@ -663,8 +732,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinInclusive, baseMinExclusive);
             if (result != 1)
             {
-                REPORT_FACET_ERROR(baseMinExclusive
-                                 , thisMinInclusive
+                REPORT_FACET_ERROR(thisMinInclusive
+                                 , baseMinExclusive
                                  , XMLExcepts::FACET_minIncl_base_minExcl)
             }
         }
@@ -674,8 +743,8 @@ void AbstractNumericFacetValidator::inspectFacetBase()
             int result = compareValues(thisMinInclusive, baseMaxExclusive);
             if (result != -1)
             {
-                REPORT_FACET_ERROR(baseMaxExclusive
-                                 , thisMinInclusive
+                REPORT_FACET_ERROR(thisMinInclusive
+                                 , baseMaxExclusive
                                  , XMLExcepts::FACET_minIncl_base_maxExcl)
             }
         }
@@ -705,18 +774,10 @@ void AbstractNumericFacetValidator::inspectFacetBase()
                         , DatatypeValidator::FACET_MAXINCLUSIVE
                         , XMLExcepts::FACET_maxIncl_notFromBase)
 
-    FROM_BASE_VALUE_SPACE(thisMaxExclusive
-                        , DatatypeValidator::FACET_MAXEXCLUSIVE
-                        , XMLExcepts::FACET_maxExcl_notFromBase)
-
     FROM_BASE_VALUE_SPACE(thisMinInclusive
                         , DatatypeValidator::FACET_MININCLUSIVE
                         , XMLExcepts::FACET_minIncl_notFromBase)
 
-    FROM_BASE_VALUE_SPACE(thisMinExclusive
-                        , DatatypeValidator::FACET_MINEXCLUSIVE
-                        , XMLExcepts::FACET_minExcl_notFromBase)
-
 } //end of inspectFacetBase
 
 //
@@ -790,11 +851,13 @@ void AbstractNumericFacetValidator::inheritFacet()
 
 }
 
-const RefVectorOf<XMLCh>* AbstractNumericFacetValidator::getEnumString() const
+const RefArrayVectorOf<XMLCh>* AbstractNumericFacetValidator::getEnumString() const
 {
 	return (fEnumerationInherited? getBaseValidator()->getEnumString() : fStrEnumeration );
 }
 
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file AbstractNumericFacetValidator::cpp
   */
