@@ -56,6 +56,33 @@
 
 /*
  * $Log: AbstractStringValidator.cpp,v $
+ * Revision 1.12  2003/05/16 06:01:57  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.11  2003/05/15 18:53:26  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.10  2003/02/22 18:28:26  peiyongz
+ * Schema Errata E2-35 Length, minLength and maxLength in different derivation steps.
+ *
+ * Revision 1.9  2003/01/27 19:24:17  peiyongz
+ * normalize Base64 data before checking against enumeration.
+ *
+ * Revision 1.8  2003/01/24 23:18:34  peiyongz
+ * normalizeEnumeration() added to remove optional ws in Base64 data.
+ *
+ * Revision 1.7  2002/12/18 14:17:55  gareth
+ * Fix to bug #13438. When you eant a vector that calls delete[] on its members you should use RefArrayVectorOf.
+ *
+ * Revision 1.6  2002/11/04 14:53:27  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.5  2002/10/02 13:29:12  tng
+ * Since the compare function return int, so use XMLString::compareString instead of XMLString::equals there.
+ *
+ * Revision 1.4  2002/09/24 19:44:40  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
  * Revision 1.3  2002/04/01 20:17:46  peiyongz
  * Bug#7551: Exceptions are caught by value, rather than by reference
  *
@@ -101,11 +128,12 @@
 //  Includes
 // ---------------------------------------------------------------------------
 #include <xercesc/validators/datatype/AbstractStringValidator.hpp>
-#include <xercesc/validators/schema/SchemaSymbols.hpp>
 #include <xercesc/validators/datatype/InvalidDatatypeFacetException.hpp>
 #include <xercesc/validators/datatype/InvalidDatatypeValueException.hpp>
 #include <xercesc/util/NumberFormatException.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
 static const int BUF_LEN = 64;
 static XMLCh value1[BUF_LEN+1];
 static XMLCh value2[BUF_LEN+1];
@@ -144,8 +172,9 @@ AbstractStringValidator::AbstractStringValidator(
                           DatatypeValidator*            const baseValidator
                         , RefHashTableOf<KVStringPair>* const facets
                         , const int                           finalSet
-                        , const ValidatorType                 type)
-:DatatypeValidator(baseValidator, facets, finalSet, type)
+                        , const ValidatorType                 type
+                        , MemoryManager* const                manager)
+:DatatypeValidator(baseValidator, facets, finalSet, type, manager)
 ,fLength(0)
 ,fMaxLength(SchemaSymbols::fgINT_MAX_VALUE)
 ,fMinLength(0)
@@ -157,11 +186,14 @@ AbstractStringValidator::AbstractStringValidator(
     // assigneAdditionalFacet(), inheritAdditionalFacet().
 }
 
-void AbstractStringValidator::init(RefVectorOf<XMLCh>*           const enums)
+void AbstractStringValidator::init(RefArrayVectorOf<XMLCh>*           const enums)
 {
 
     if (enums)
+    {
         setEnumeration(enums, false);
+        normalizeEnumeration();
+    }
 
     assignFacet();
     inspectFacet();
@@ -193,7 +225,7 @@ void AbstractStringValidator::assignFacet()
         key = pair.getKey();
         value = pair.getValue();
 
-        if (XMLString::compareString(key, SchemaSymbols::fgELT_LENGTH)==0)
+        if (XMLString::equals(key, SchemaSymbols::fgELT_LENGTH))
         {
             int val;
             try
@@ -211,7 +243,7 @@ void AbstractStringValidator::assignFacet()
                 setLength(val);
                 setFacetsDefined(DatatypeValidator::FACET_LENGTH);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MINLENGTH)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MINLENGTH))
         {
             int val;
             try
@@ -229,7 +261,7 @@ void AbstractStringValidator::assignFacet()
             setMinLength(val);
             setFacetsDefined(DatatypeValidator::FACET_MINLENGTH);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_MAXLENGTH)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_MAXLENGTH))
         {
             int val;
             try
@@ -247,14 +279,14 @@ void AbstractStringValidator::assignFacet()
             setMaxLength(val);
             setFacetsDefined(DatatypeValidator::FACET_MAXLENGTH);
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgELT_PATTERN)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgELT_PATTERN))
         {
             setPattern(value);
             if (getPattern())
                 setFacetsDefined(DatatypeValidator::FACET_PATTERN);
             // do not construct regex until needed
         }
-        else if (XMLString::compareString(key, SchemaSymbols::fgATT_FIXED)==0)
+        else if (XMLString::equals(key, SchemaSymbols::fgATT_FIXED))
         {
             unsigned int val;
             bool         retStatus;
@@ -276,7 +308,7 @@ void AbstractStringValidator::assignFacet()
             //no setFacetsDefined here
         }
         //
-        // else if (XMLString::compareString(key, SchemaSymbols::fgELT_SPECIAL_TOKEN)==0)
+        // else if (XMLString::equals(key, SchemaSymbols::fgELT_SPECIAL_TOKEN))
         // TODO
         //
         // Note: whitespace is taken care of by TraverseSchema.
@@ -362,24 +394,53 @@ void AbstractStringValidator::inspectFacetBase()
     /***
                 Non coexistence of derived' length and base'    (minLength | maxLength)
                                    base'    length and derived' (minLength | maxLength)
+
+     E2-35
+     It is an error for both length and either of minLength or maxLength to be members of {facets},
+     unless they are specified in different derivation steps in which case the following must be true: 
+     the {value} of minLength <= the {value} of length <= the {value} of maxLength   
     ***/
 
-    // check 4.3.1.c1 error: length & (base.maxLength | base.minLength)
+    // error: length > base.maxLength 
+    //        length < base.minLength
     if ((thisFacetsDefined & DatatypeValidator::FACET_LENGTH) !=0)
     {
-        if ((baseFacetsDefined & DatatypeValidator::FACET_MAXLENGTH) !=0)
-            ThrowXML(InvalidDatatypeFacetException, XMLExcepts::FACET_Len_maxLen);
-        else if ((baseFacetsDefined & DatatypeValidator::FACET_MINLENGTH) !=0)
-            ThrowXML(InvalidDatatypeFacetException, XMLExcepts::FACET_Len_minLen);
+        if (((baseFacetsDefined & DatatypeValidator::FACET_MAXLENGTH) !=0) &&
+             (thisLength > baseMaxLength)                                   )
+        {
+            REPORT_FACET_ERROR(thisLength
+                             , baseMaxLength
+                             , XMLExcepts::FACET_Len_baseMaxLen)
+        }
+        
+        if (((baseFacetsDefined & DatatypeValidator::FACET_MINLENGTH) !=0) &&
+             (thisLength < baseMinLength)                                   )
+        {
+            REPORT_FACET_ERROR(thisLength
+                             , baseMinLength
+                             , XMLExcepts::FACET_Len_baseMinLen)
+        }
     }
 
-    // check 4.3.1.c1 error: base.length & (maxLength | minLength)
+    // error: baseLength > maxLength 
+    //        baseLength < minLength
     if ((baseFacetsDefined & DatatypeValidator::FACET_LENGTH) !=0)
     {
-        if ((thisFacetsDefined & DatatypeValidator::FACET_MAXLENGTH) !=0)
-            ThrowXML(InvalidDatatypeFacetException, XMLExcepts::FACET_Len_maxLen);
-        else if ((thisFacetsDefined & DatatypeValidator::FACET_MINLENGTH) !=0)
-            ThrowXML(InvalidDatatypeFacetException, XMLExcepts::FACET_Len_minLen);
+        if (((thisFacetsDefined & DatatypeValidator::FACET_MAXLENGTH) !=0) &&
+             (baseLength > thisMaxLength)                                   )
+        {
+            REPORT_FACET_ERROR(thisMaxLength
+                             , baseLength
+                             , XMLExcepts::FACET_maxLen_baseLen)
+        }
+        
+        if (((thisFacetsDefined & DatatypeValidator::FACET_MINLENGTH) !=0) &&
+             (baseLength < thisMinLength)                                   )
+        {
+            REPORT_FACET_ERROR(thisMinLength
+                             , baseLength
+                             , XMLExcepts::FACET_minLen_baseLen)
+        }
     }
 
     // check 4.3.1.c2 error: length != base.length
@@ -588,7 +649,7 @@ void AbstractStringValidator::checkContent( const XMLCh* const content, bool asB
         // lazy construction
         if (getRegex() ==0) {
             try {
-                setRegex(new RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption));
+                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));
             }
             catch (XMLException &e)
             {
@@ -643,11 +704,15 @@ void AbstractStringValidator::checkContent( const XMLCh* const content, bool asB
     if ((thisFacetsDefined & DatatypeValidator::FACET_ENUMERATION) != 0 &&
         (getEnumeration() != 0))
     {
+        XMLCh* normContent = XMLString::replicate(content, fMemoryManager);
+        ArrayJanitor<XMLCh>  jan(normContent, fMemoryManager);
+        normalizeContent(normContent);
+
         int i=0;
         int enumLength = getEnumeration()->size();
         for ( ; i < enumLength; i++)
         {
-            if (XMLString::compareString(content, getEnumeration()->elementAt(i))==0)
+            if (XMLString::equals(normContent, getEnumeration()->elementAt(i)))
                 break;
         }
 
@@ -659,11 +724,25 @@ void AbstractStringValidator::checkContent( const XMLCh* const content, bool asB
 
 }
 
-const RefVectorOf<XMLCh>* AbstractStringValidator::getEnumString() const
+const RefArrayVectorOf<XMLCh>* AbstractStringValidator::getEnumString() const
 {
 	return getEnumeration();
 }
 
+void AbstractStringValidator::normalizeEnumeration()
+{
+    // default implementation: do nothing
+    return;
+}
+
+void AbstractStringValidator::normalizeContent(XMLCh* const) const
+{
+    // default implementation: do nothing
+    return;
+}
+
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file AbstractStringValidator.cpp
   */
