@@ -56,6 +56,43 @@
 
 /*
  * $Log: DatatypeValidatorFactory.cpp,v $
+ * Revision 1.16  2003/05/18 14:02:07  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.15  2003/05/16 21:43:20  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.14  2003/05/15 18:53:26  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.13  2003/03/18 19:42:17  knoaman
+ * Schema Errata E2-18.
+ *
+ * Revision 1.12  2003/02/26 16:14:27  peiyongz
+ * Schema Errata E2-43: disallow trailing decimal point and a new pattern added
+ * to the Integer definition.
+ *
+ * Revision 1.11  2003/01/29 19:51:48  gareth
+ * type names / uris are now added to DatatypeValidators.
+ *
+ * Revision 1.10  2003/01/03 16:39:45  tng
+ * Performance: no need to create that many members for fBuiltInRegistry
+ *
+ * Revision 1.9  2002/12/18 14:17:55  gareth
+ * Fix to bug #13438. When you eant a vector that calls delete[] on its members you should use RefArrayVectorOf.
+ *
+ * Revision 1.8  2002/12/02 16:01:24  gareth
+ * Fix to bug #12188. NCNAME validator now has a base validator of NAME. Patch by Peter Volchek.
+ *
+ * Revision 1.7  2002/12/02 13:25:51  gareth
+ * Fix for bug #12188. Create NMTOKEN, ID, IDREF, ENTITY, NAME, NCNAME with appropriate base types. Some reordering of creation was required where dependencies resulted.
+ *
+ * Revision 1.6  2002/11/21 23:43:19  peiyongz
+ * Schema Errata: E2-25 language
+ *
+ * Revision 1.5  2002/11/04 14:53:28  tng
+ * C++ Namespace Support.
+ *
  * Revision 1.4  2002/05/08 12:25:30  tng
  * Schema Fix: re-add the ID, IDREF ... datatype validators only if they were not there.
  *
@@ -190,6 +227,9 @@
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/util/XMLRegisterCleanup.hpp>
 
+XERCES_CPP_NAMESPACE_BEGIN
+
+
 // ---------------------------------------------------------------------------
 //  DatatypeValidatorFactory: Local const data
 // ---------------------------------------------------------------------------
@@ -198,18 +238,24 @@ const XMLCh fgTokPattern[] =
     chBackSlash, chLatin_c, chPlus, chNull
 };
 
-//"([a-zA-Z]{2}|[iI]-[a-zA-Z]+|[xX]-[a-zA-Z]+)(-[a-zA-Z]+)*"
+// http://www.w3.org/2001/05/xmlschema-errata#Errata2
+// E2-25
+//"([a-zA-Z]{1,8})(-[a-zA-Z0-9]{1,8})*"
 const XMLCh fgLangPattern[] =
 {
-    chOpenParen, chOpenSquare, chLatin_a, chDash, chLatin_z, chLatin_A, chDash,
-    chLatin_Z, chCloseSquare, chOpenCurly, chDigit_2, chCloseCurly, chPipe,
-    chOpenSquare, chLatin_i, chLatin_I, chCloseSquare, chDash, chOpenSquare,
-    chLatin_a, chDash, chLatin_z, chLatin_A, chDash, chLatin_Z, chCloseSquare,
-    chPlus, chPipe, chOpenSquare, chLatin_x, chLatin_X, chCloseSquare, chDash,
-    chOpenSquare, chLatin_a, chDash, chLatin_z, chLatin_A, chDash, chLatin_Z,
-	chCloseSquare, chPlus, chCloseParen, chOpenParen, chDash, chOpenSquare,
-    chLatin_a, chDash, chLatin_z, chLatin_A, chDash, chLatin_Z, chCloseSquare,
-    chPlus, chCloseParen, chAsterisk, chNull
+    chOpenParen,   chOpenSquare,  chLatin_a, chDash, chLatin_z, chLatin_A, chDash, chLatin_Z,
+    chCloseSquare, chOpenCurly, chDigit_1, chComma, chDigit_8, chCloseCurly, chCloseParen,
+    chOpenParen,   chDash, chOpenSquare, chLatin_a, chDash, chLatin_z, chLatin_A, chDash, chLatin_Z,
+	chDigit_0,     chDash, chDigit_9, chCloseSquare, chOpenCurly, chDigit_1, chComma, chDigit_8,
+    chCloseCurly,  chCloseParen, chAsterisk, chNull
+};
+
+//E2-43
+//[+\-]?[0-9]+
+const XMLCh fgIntegerPattern[] =
+{
+    chOpenSquare, chPlus, chBackSlash, chDash, chCloseSquare, chQuestion,
+    chOpenSquare, chDigit_0, chDash, chDigit_9, chCloseSquare, chPlus, chNull
 };
 
 //"\\i\\c*"
@@ -346,8 +392,9 @@ RefHashTableOf<DatatypeValidator>* DatatypeValidatorFactory::fBuiltInRegistry =
 // ---------------------------------------------------------------------------
 //  DatatypeValidatorFactory: Constructors and Destructor
 // ---------------------------------------------------------------------------
-DatatypeValidatorFactory::DatatypeValidatorFactory()
+DatatypeValidatorFactory::DatatypeValidatorFactory(MemoryManager* const manager)
     : fUserDefinedRegistry(0)
+    , fMemoryManager(manager)
 {
 }
 
@@ -383,10 +430,9 @@ void DatatypeValidatorFactory::reinitRegistry() {
 void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
 {
     static XMLRegisterCleanup builtInRegistryCleanup;
-
     // Initialize common Schema/DTD Datatype validator set if not initialized
     if (fBuiltInRegistry == 0) {
-        RefHashTableOf<DatatypeValidator>* t = new RefHashTableOf<DatatypeValidator>(109);
+        RefHashTableOf<DatatypeValidator>* t = new RefHashTableOf<DatatypeValidator>(29);
         if (XMLPlatformUtils::compareAndSwap((void **)&fBuiltInRegistry, t, 0) != 0)
         {
             delete t;
@@ -395,53 +441,85 @@ void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
         {
             builtInRegistryCleanup.registerCleanup(reinitRegistry);
 
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_STRING,
-                       new StringDatatypeValidator());
-            fBuiltInRegistry->put((void*) XMLUni::fgNotationString,
-                       new NOTATIONDatatypeValidator());
+            DatatypeValidator *dv = new StringDatatypeValidator(); 
+            dv->setTypeName(SchemaSymbols::fgDT_STRING, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_STRING, dv);
+            
+            dv = new NOTATIONDatatypeValidator();
+            dv->setTypeName(XMLUni::fgNotationString, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) XMLUni::fgNotationString, dv);
 
+            dv = new AnySimpleTypeDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_ANYSIMPLETYPE, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_ANYSIMPLETYPE, dv);
 
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_ANYSIMPLETYPE,
-                           new AnySimpleTypeDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_BOOLEAN,
-                           new BooleanDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DECIMAL,
-                           new DecimalDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_HEXBINARY,
-                           new HexBinaryDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_BASE64BINARY,
-                           new Base64BinaryDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DOUBLE,
-                           new DoubleDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_FLOAT,
-                           new FloatDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_ANYURI,
-                           new AnyURIDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_QNAME,
-                           new QNameDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_NAME,
-                           new NameDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_NCNAME,
-                           new NCNameDatatypeValidator());
+            dv = new BooleanDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_BOOLEAN, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_BOOLEAN, dv);
 
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DATETIME,
-                           new DateTimeDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DATE,
-                           new DateDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_TIME,
-                           new TimeDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DAY,
-                           new DayDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_MONTH,
-                           new MonthDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_MONTHDAY,
-                           new MonthDayDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_YEAR,
-                           new YearDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_YEARMONTH,
-                           new YearMonthDatatypeValidator());
-            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DURATION,
-                           new DurationDatatypeValidator());
+            dv = new DecimalDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DECIMAL, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DECIMAL, dv);
+
+            dv = new HexBinaryDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_HEXBINARY, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_HEXBINARY, dv);
+
+            dv = new Base64BinaryDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_BASE64BINARY, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_BASE64BINARY, dv);
+
+            dv = new DoubleDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DOUBLE, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DOUBLE, dv);
+
+            dv = new FloatDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_FLOAT, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_FLOAT, dv);
+
+            dv = new AnyURIDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_ANYURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_ANYURI, dv);
+
+            dv = new QNameDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_QNAME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_QNAME, dv);
+
+            dv = new DateTimeDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DATETIME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DATETIME, dv);
+
+            dv = new DateDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DATE, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DATE, dv);
+
+            dv = new TimeDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_TIME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_TIME, dv);
+
+            dv = new DayDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DAY, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DAY, dv);
+
+            dv = new MonthDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_MONTH, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_MONTH, dv);
+
+            dv = new MonthDayDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_MONTHDAY, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_MONTHDAY, dv);
+
+            dv = new YearDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_YEAR, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_YEAR, dv);
+
+            dv = new YearMonthDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_YEARMONTH, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_YEARMONTH, dv);
+
+            dv = new DurationDatatypeValidator();
+            dv->setTypeName(SchemaSymbols::fgDT_DURATION, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_DURATION, dv);
 
             // REVISIT
             // We are creating a lot of Hashtables for the facets of the different
@@ -450,31 +528,16 @@ void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
 
             RefHashTableOf<KVStringPair>* facets = new RefHashTableOf<KVStringPair>(3);
 
-            // Create 'NMTOKEN' datatype validator
-            facets->put((void*) SchemaSymbols::fgELT_PATTERN ,
-                            new KVStringPair(SchemaSymbols::fgELT_PATTERN,fgTokPattern));
-            facets->put((void*) SchemaSymbols::fgELT_WHITESPACE,
-                            new KVStringPair(SchemaSymbols::fgELT_WHITESPACE, SchemaSymbols::fgWS_COLLAPSE));
-
-            createDatatypeValidator(XMLUni::fgNmTokenString,
-                            getDatatypeValidator(SchemaSymbols::fgDT_STRING),facets, 0, false, 0, false);
-
-            // Create 'NMTOKENS' datatype validator
-            createDatatypeValidator(XMLUni::fgNmTokensString,
-        	                 getDatatypeValidator(XMLUni::fgNmTokenString), 0, 0, true, 0, false);
-
             // Create 'normalizedString' datatype validator
-            facets = new RefHashTableOf<KVStringPair>(3);
             facets->put((void*) SchemaSymbols::fgELT_WHITESPACE,
                         new KVStringPair(SchemaSymbols::fgELT_WHITESPACE, SchemaSymbols::fgWS_REPLACE));
 
             createDatatypeValidator(SchemaSymbols::fgDT_NORMALIZEDSTRING,
-                        getDatatypeValidator(SchemaSymbols::fgDT_STRING),
-    				    facets, 0, false, 0, false);
+                                    getDatatypeValidator(SchemaSymbols::fgDT_STRING),
+                                    facets, 0, false, 0, false);
 
             // Create 'token' datatype validator
             facets = new RefHashTableOf<KVStringPair>(3);
-
             facets->put((void*) SchemaSymbols::fgELT_WHITESPACE,
                         new KVStringPair(SchemaSymbols::fgELT_WHITESPACE, SchemaSymbols::fgWS_COLLAPSE));
 
@@ -482,6 +545,31 @@ void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
                           getDatatypeValidator(SchemaSymbols::fgDT_NORMALIZEDSTRING),
                           facets, 0, false, 0, false);
 
+
+            dv = new NameDatatypeValidator(getDatatypeValidator(SchemaSymbols::fgDT_TOKEN), 0, 0, 0);
+            dv->setTypeName(SchemaSymbols::fgDT_NAME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_NAME, dv);
+
+
+            dv = new NCNameDatatypeValidator(getDatatypeValidator(SchemaSymbols::fgDT_NAME), 0, 0, 0);
+            dv->setTypeName(SchemaSymbols::fgDT_NCNAME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+            fBuiltInRegistry->put((void*) SchemaSymbols::fgDT_NCNAME, dv);
+
+            // Create 'NMTOKEN' datatype validator
+            facets = new RefHashTableOf<KVStringPair>(3);
+
+            facets->put((void*) SchemaSymbols::fgELT_PATTERN ,
+                            new KVStringPair(SchemaSymbols::fgELT_PATTERN,fgTokPattern));
+            facets->put((void*) SchemaSymbols::fgELT_WHITESPACE,
+                            new KVStringPair(SchemaSymbols::fgELT_WHITESPACE, SchemaSymbols::fgWS_COLLAPSE));
+
+            createDatatypeValidator(XMLUni::fgNmTokenString,
+                            getDatatypeValidator(SchemaSymbols::fgDT_TOKEN),facets, 0, false, 0, false);
+
+            // Create 'NMTOKENS' datatype validator
+            createDatatypeValidator(XMLUni::fgNmTokensString,
+        	                 getDatatypeValidator(XMLUni::fgNmTokenString), 0, 0, true, 0, false);
+
             // Create 'language' datatype validator
             facets = new RefHashTableOf<KVStringPair>(3);
 
@@ -498,6 +586,9 @@ void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
             facets->put((void*) SchemaSymbols::fgELT_FRACTIONDIGITS,
                         new KVStringPair(SchemaSymbols::fgELT_FRACTIONDIGITS, fgValueZero));
 
+            facets->put((void*) SchemaSymbols::fgELT_PATTERN,
+                        new KVStringPair(SchemaSymbols::fgELT_PATTERN, fgIntegerPattern));
+
             createDatatypeValidator(SchemaSymbols::fgDT_INTEGER,
                           getDatatypeValidator(SchemaSymbols::fgDT_DECIMAL),
                           facets, 0, false, 0, false);
@@ -637,23 +728,43 @@ void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
     // So instead of storing them in the static fBuiltInRegistry,
     //   store them in local data fUserDefinedRegistry
     if (fUserDefinedRegistry == 0)
-        fUserDefinedRegistry = new RefHashTableOf<DatatypeValidator>(29);
+        fUserDefinedRegistry = new (fMemoryManager) RefHashTableOf<DatatypeValidator>(29, fMemoryManager);
 
     if (!getDatatypeValidator(XMLUni::fgIDRefsString)) {
-        fUserDefinedRegistry->put((void*) XMLUni::fgIDString,
-                           new IDDatatypeValidator());
-        fUserDefinedRegistry->put((void*) XMLUni::fgIDRefString,
-                           new IDREFDatatypeValidator());
-        fUserDefinedRegistry->put((void*) XMLUni::fgEntityString,
-                           new ENTITYDatatypeValidator());
+
+        DatatypeValidator *dv = new (fMemoryManager) IDDatatypeValidator(getDatatypeValidator(SchemaSymbols::fgDT_NCNAME), 0, 0, 0, fMemoryManager);
+        dv->setTypeName(XMLUni::fgIDString, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+        fUserDefinedRegistry->put((void*) XMLUni::fgIDString, dv);
+
+        dv = new (fMemoryManager) IDREFDatatypeValidator(getDatatypeValidator(SchemaSymbols::fgDT_NCNAME), 0, 0, 0, fMemoryManager);
+        dv->setTypeName(XMLUni::fgIDRefString, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+        fUserDefinedRegistry->put((void*) XMLUni::fgIDRefString, dv);
+
+        dv = new (fMemoryManager) ENTITYDatatypeValidator(getDatatypeValidator(SchemaSymbols::fgDT_NCNAME), 0, 0, 0, fMemoryManager);
+        dv->setTypeName(XMLUni::fgEntityString, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
+        fUserDefinedRegistry->put((void*) XMLUni::fgEntityString, dv);
 
         // Create 'IDREFS' datatype validator
-    	 createDatatypeValidator(XMLUni::fgIDRefsString,
-                        getDatatypeValidator(XMLUni::fgIDRefString), 0, 0, true, 0, true);
+    	createDatatypeValidator
+        (
+            XMLUni::fgIDRefsString
+            , getDatatypeValidator(XMLUni::fgIDRefString)
+            , 0
+            , 0
+            , true
+            , 0
+        );
 
         // Create 'ENTITIES' datatype validator
-        createDatatypeValidator(XMLUni::fgEntitiesString,
-    		            getDatatypeValidator(XMLUni::fgEntityString), 0, 0, true, 0, true);
+        createDatatypeValidator
+        (
+            XMLUni::fgEntitiesString
+    		, getDatatypeValidator(XMLUni::fgEntityString)
+            , 0
+            , 0
+            , true
+            , 0
+        );
     }
 
 }
@@ -661,14 +772,16 @@ void DatatypeValidatorFactory::expandRegistryToFullSchemaSet()
 // ---------------------------------------------------------------------------
 //  DatatypeValidatorFactory: factory methods
 // ---------------------------------------------------------------------------
-DatatypeValidator*
-DatatypeValidatorFactory::createDatatypeValidator(const XMLCh* const typeName,
-		                                          DatatypeValidator* const baseValidator,
-                                                  RefHashTableOf<KVStringPair>* const facets,
-                                                  RefVectorOf<XMLCh>* const enums,
-                                                  const bool derivedByList,
-                                                  const int finalSet,
-                                                  const bool userDefined)
+DatatypeValidator* DatatypeValidatorFactory::createDatatypeValidator
+(
+      const XMLCh* const                  typeName
+	, DatatypeValidator* const            baseValidator
+    , RefHashTableOf<KVStringPair>* const facets
+    , RefArrayVectorOf<XMLCh>* const      enums
+    , const bool                          isDerivedByList
+    , const int                           finalSet
+    , const bool                          isUserDefined
+)
 {
 	if (baseValidator == 0) {
 
@@ -684,9 +797,11 @@ DatatypeValidatorFactory::createDatatypeValidator(const XMLCh* const typeName,
     }
 
 	DatatypeValidator* datatypeValidator = 0;
+    MemoryManager* const manager = (isUserDefined)
+        ? fMemoryManager : XMLPlatformUtils::fgMemoryManager;
 
-    if (derivedByList) {
-        datatypeValidator = new ListDatatypeValidator(baseValidator, facets, enums, finalSet);
+    if (isDerivedByList) {
+        datatypeValidator = new (manager) ListDatatypeValidator(baseValidator, facets, enums, finalSet, manager);
     }
     else {
 
@@ -699,15 +814,21 @@ DatatypeValidatorFactory::createDatatypeValidator(const XMLCh* const typeName,
             }
         }
 
-        datatypeValidator = baseValidator->newInstance(facets, enums, finalSet);
+        datatypeValidator = baseValidator->newInstance
+        (
+            facets
+            , enums
+            , finalSet
+            , manager
+        );
     }
 
     if (datatypeValidator != 0) {
 
-        if (userDefined) {
+        if (isUserDefined) {
 
             if (!fUserDefinedRegistry) {
-                fUserDefinedRegistry = new RefHashTableOf<DatatypeValidator>(29);
+                fUserDefinedRegistry = new (fMemoryManager) RefHashTableOf<DatatypeValidator>(29, fMemoryManager);
             }
 
             fUserDefinedRegistry->put((void *)typeName, datatypeValidator);
@@ -715,31 +836,37 @@ DatatypeValidatorFactory::createDatatypeValidator(const XMLCh* const typeName,
         else {
             fBuiltInRegistry->put((void *)typeName, datatypeValidator);
         }
+
+        datatypeValidator->setTypeName(typeName);
     }
 
     return datatypeValidator;
 }
 
 
-DatatypeValidator*
-DatatypeValidatorFactory::createDatatypeValidator(const XMLCh* const typeName,
-                                                  RefVectorOf<DatatypeValidator>* const validators,
-                                                  const int finalSet,
-                                                  const bool userDefined)
+DatatypeValidator* DatatypeValidatorFactory::createDatatypeValidator
+(
+      const XMLCh* const                    typeName
+    , RefVectorOf<DatatypeValidator>* const validators
+    , const int                             finalSet
+    , const bool                            userDefined
+)
 {
     if (validators == 0)
         return 0;
 
     DatatypeValidator* datatypeValidator = 0;
+    MemoryManager* const manager = (userDefined)
+        ? fMemoryManager : XMLPlatformUtils::fgMemoryManager;
 
-    datatypeValidator = new UnionDatatypeValidator(validators, finalSet);
+    datatypeValidator = new (manager) UnionDatatypeValidator(validators, finalSet, manager);
 
     if (datatypeValidator != 0) {
 
         if (userDefined) {
 
             if (!fUserDefinedRegistry) {
-                fUserDefinedRegistry = new RefHashTableOf<DatatypeValidator>(29);
+                fUserDefinedRegistry = new (fMemoryManager) RefHashTableOf<DatatypeValidator>(29, fMemoryManager);
             }
 
             fUserDefinedRegistry->put((void *)typeName, datatypeValidator);
@@ -747,11 +874,14 @@ DatatypeValidatorFactory::createDatatypeValidator(const XMLCh* const typeName,
         else {
             fBuiltInRegistry->put((void *)typeName, datatypeValidator);
         }
+        datatypeValidator->setTypeName(typeName);
     }
 
     return datatypeValidator;
 }
 
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file DatatypeValidatorFactory.cpp
   */
