@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
+ * Copyright (c) 2001-2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,6 +56,33 @@
 
 /*
  * $Log: DecimalDatatypeValidator.cpp,v $
+ * Revision 1.11  2003/05/18 14:02:07  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.10  2003/05/16 06:01:57  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.9  2003/05/15 18:53:26  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.8  2003/03/01 20:59:06  peiyongz
+ * TotalDigits value must be a positiveInteger
+ *
+ * Revision 1.7  2003/02/25 17:24:37  peiyongz
+ * Schema Errata: E2-44 totalDigits/fractDigits
+ *
+ * Revision 1.6  2003/01/30 21:56:22  tng
+ * Performance: call getRawData instead of toString
+ *
+ * Revision 1.5  2002/12/18 14:17:55  gareth
+ * Fix to bug #13438. When you eant a vector that calls delete[] on its members you should use RefArrayVectorOf.
+ *
+ * Revision 1.4  2002/11/04 14:53:28  tng
+ * C++ Namespace Support.
+ *
+ * Revision 1.3  2002/09/24 19:44:40  tng
+ * Performance: use XMLString::equals instead of XMLString::compareString
+ *
  * Revision 1.2  2002/04/01 20:17:46  peiyongz
  * Bug#7551: Exceptions are caught by value, rather than by reference
  *
@@ -127,6 +154,9 @@
 #include <xercesc/validators/datatype/InvalidDatatypeFacetException.hpp>
 #include <xercesc/validators/datatype/InvalidDatatypeValueException.hpp>
 #include <xercesc/util/NumberFormatException.hpp>
+#include <xercesc/util/XMLBigDecimal.hpp>
+
+XERCES_CPP_NAMESPACE_BEGIN
 
 static const int BUF_LEN = 64;
 static XMLCh value1[BUF_LEN+1];
@@ -135,8 +165,8 @@ static XMLCh value2[BUF_LEN+1];
 // ---------------------------------------------------------------------------
 //  Constructors and Destructor
 // ---------------------------------------------------------------------------
-DecimalDatatypeValidator::DecimalDatatypeValidator()
-:AbstractNumericValidator(0, 0, 0, DatatypeValidator::Decimal)
+DecimalDatatypeValidator::DecimalDatatypeValidator(MemoryManager* const manager)
+:AbstractNumericValidator(0, 0, 0, DatatypeValidator::Decimal, manager)
 , fTotalDigits(0)
 , fFractionDigits(0)
 {}
@@ -144,9 +174,10 @@ DecimalDatatypeValidator::DecimalDatatypeValidator()
 DecimalDatatypeValidator::DecimalDatatypeValidator(
                           DatatypeValidator*            const baseValidator
                         , RefHashTableOf<KVStringPair>* const facets
-                        , RefVectorOf<XMLCh>*           const enums
-                        , const int                           finalSet)
-:AbstractNumericValidator(baseValidator, facets, finalSet, DatatypeValidator::Decimal)
+                        , RefArrayVectorOf<XMLCh>*           const enums
+                        , const int                           finalSet
+                        , MemoryManager* const manager)
+:AbstractNumericValidator(baseValidator, facets, finalSet, DatatypeValidator::Decimal, manager)
 , fTotalDigits(0)
 , fFractionDigits(0)
 {
@@ -162,20 +193,21 @@ DecimalDatatypeValidator::~DecimalDatatypeValidator()
 int DecimalDatatypeValidator::compare(const XMLCh* const lValue
                                     , const XMLCh* const rValue)
 {
-    XMLBigDecimal * lObj = new XMLBigDecimal(lValue);
-    Janitor<XMLBigDecimal> jname1(lObj);
-    XMLBigDecimal * rObj = new XMLBigDecimal(rValue);
-    Janitor<XMLBigDecimal> jname2(rObj);
+    XMLBigDecimal lObj(lValue, fMemoryManager);
+    XMLBigDecimal rObj(rValue, fMemoryManager);
 
-    return compareValues(lObj, rObj);
+    return compareValues(&lObj, &rObj);
 }
 
-DatatypeValidator* DecimalDatatypeValidator::newInstance(
-                                      RefHashTableOf<KVStringPair>* const facets
-                                    , RefVectorOf<XMLCh>*           const enums
-                                    , const int                           finalSet)
+DatatypeValidator* DecimalDatatypeValidator::newInstance
+(
+      RefHashTableOf<KVStringPair>* const facets
+    , RefArrayVectorOf<XMLCh>* const      enums
+    , const int                           finalSet
+    , MemoryManager* const                manager
+)
 {
-    return (DatatypeValidator*) new DecimalDatatypeValidator(this, facets, enums, finalSet);
+    return (DatatypeValidator*) new (manager) DecimalDatatypeValidator(this, facets, enums, finalSet, manager);
 }
 
 // -----------------------------------------------------------------------
@@ -184,8 +216,9 @@ DatatypeValidator* DecimalDatatypeValidator::newInstance(
 DecimalDatatypeValidator::DecimalDatatypeValidator(DatatypeValidator*            const baseValidator
                                                  , RefHashTableOf<KVStringPair>* const facets
                                                  , const int                           finalSet
-                                                 , const ValidatorType                 type)
-:AbstractNumericValidator(baseValidator, facets, finalSet, type)
+                                                 , const ValidatorType                 type
+                                                 , MemoryManager* const                manager)
+:AbstractNumericValidator(baseValidator, facets, finalSet, type, manager)
 , fTotalDigits(0)
 , fFractionDigits(0)
 {
@@ -195,7 +228,7 @@ DecimalDatatypeValidator::DecimalDatatypeValidator(DatatypeValidator*
 void DecimalDatatypeValidator::assignAdditionalFacet(const XMLCh* const key
                                                    , const XMLCh* const value)
 {
-    if (XMLString::compareString(key, SchemaSymbols::fgELT_TOTALDIGITS)==0)
+    if (XMLString::equals(key, SchemaSymbols::fgELT_TOTALDIGITS))
     {
         int val;
         try
@@ -209,12 +242,12 @@ void DecimalDatatypeValidator::assignAdditionalFacet(const XMLCh* const key
 
         // check 4.3.11.c0 must: totalDigits > 0
         if ( val <= 0 )
-            ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_NonNeg_TotalDigit, value);
+            ThrowXML1(InvalidDatatypeFacetException, XMLExcepts::FACET_PosInt_TotalDigit, value);
 
         setTotalDigits(val);
         setFacetsDefined(DatatypeValidator::FACET_TOTALDIGITS);
     }
-    else if (XMLString::compareString(key, SchemaSymbols::fgELT_FRACTIONDIGITS)==0)
+    else if (XMLString::equals(key, SchemaSymbols::fgELT_FRACTIONDIGITS))
     {
         int val;
         try
@@ -249,23 +282,23 @@ void DecimalDatatypeValidator::inheritAdditionalFacet()
     if (!numBase)
         return;
 
-    int thisFacetsDefined = getFacetsDefined();  
+    int thisFacetsDefined = getFacetsDefined();
     int baseFacetsDefined = numBase->getFacetsDefined();
 
-    // inherit totalDigits         
-    if ((( baseFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) != 0) &&          
-        (( thisFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) == 0) )              
-    {          
-        setTotalDigits(numBase->fTotalDigits);              
-        setFacetsDefined(DatatypeValidator::FACET_TOTALDIGITS);              
+    // inherit totalDigits
+    if ((( baseFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) != 0) &&
+        (( thisFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) == 0) )
+    {
+        setTotalDigits(numBase->fTotalDigits);
+        setFacetsDefined(DatatypeValidator::FACET_TOTALDIGITS);
     }
-          
-    // inherit fractionDigits          
-    if ((( baseFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) != 0) &&          
-        (( thisFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) == 0) )              
-    {          
-        setFractionDigits(numBase->fFractionDigits);              
-        setFacetsDefined(DatatypeValidator::FACET_FRACTIONDIGITS);              
+
+    // inherit fractionDigits
+    if ((( baseFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) != 0) &&
+        (( thisFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) == 0) )
+    {
+        setFractionDigits(numBase->fFractionDigits);
+        setFacetsDefined(DatatypeValidator::FACET_FRACTIONDIGITS);
     }
 }
 
@@ -326,7 +359,7 @@ void DecimalDatatypeValidator::checkAdditionalFacetConstraintsBase() const
                                  , XMLExcepts::FACET_totalDigit_base_fixed
                                  , value1
                                  , value2);
-        }                    
+        }
     }
 
     if (( thisFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) != 0)
@@ -379,22 +412,22 @@ int  DecimalDatatypeValidator::compareValues(const XMLNumber* const lValue
 
 void  DecimalDatatypeValidator::setMaxInclusive(const XMLCh* const value)
 {
-    fMaxInclusive = new XMLBigDecimal(value);
+    fMaxInclusive = new (fMemoryManager) XMLBigDecimal(value, fMemoryManager);
 }
 
 void  DecimalDatatypeValidator::setMaxExclusive(const XMLCh* const value)
 {
-    fMaxExclusive = new XMLBigDecimal(value);
+    fMaxExclusive = new (fMemoryManager) XMLBigDecimal(value, fMemoryManager);
 }
 
 void  DecimalDatatypeValidator::setMinInclusive(const XMLCh* const value)
 {
-    fMinInclusive = new XMLBigDecimal(value);
+    fMinInclusive = new (fMemoryManager) XMLBigDecimal(value, fMemoryManager);
 }
 
 void  DecimalDatatypeValidator::setMinExclusive(const XMLCh* const value)
 {
-    fMinExclusive = new XMLBigDecimal(value);
+    fMinExclusive = new (fMemoryManager) XMLBigDecimal(value, fMemoryManager);
 }
 
 void DecimalDatatypeValidator::setEnumeration()
@@ -419,7 +452,7 @@ void DecimalDatatypeValidator::setEnumeration()
             {
                 numBase->checkContent(fStrEnumeration->elementAt(i), false);
             }
-        }              
+        }
         catch (XMLException&)
         {
             ThrowXML1(InvalidDatatypeFacetException
@@ -430,19 +463,19 @@ void DecimalDatatypeValidator::setEnumeration()
 
     // We put the this->checkContent in a separate loop
     // to not block original message with in that method.
-    // 
+    //
     for ( i = 0; i < enumLength; i++)
     {
         checkContent(fStrEnumeration->elementAt(i), false);
-    }              
+    }
 
-    fEnumeration = new RefVectorOf<XMLNumber>(enumLength, true);
+    fEnumeration = new (fMemoryManager) RefVectorOf<XMLNumber>(enumLength, true, fMemoryManager);
     fEnumerationInherited = false;
 
     for ( i = 0; i < enumLength; i++)
     {
-        fEnumeration->insertElementAt(new XMLBigDecimal(fStrEnumeration->elementAt(i)), i);
-    }              
+        fEnumeration->insertElementAt(new (fMemoryManager) XMLBigDecimal(fStrEnumeration->elementAt(i), fMemoryManager), i);
+    }
 
 }
 
@@ -465,7 +498,7 @@ void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool as
         // lazy construction
         if (getRegex() ==0) {
             try {
-                setRegex(new RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption));
+                setRegex(new (fMemoryManager) RegularExpression(getPattern(), SchemaSymbols::fgRegEx_XOption, fMemoryManager));
             }
             catch (XMLException &e)
             {
@@ -488,7 +521,7 @@ void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool as
         return;
 
     try {
-        XMLBigDecimal theValue(content);
+        XMLBigDecimal theValue(content, fMemoryManager);
         XMLBigDecimal *theData = &theValue;
 
         if (getEnumeration())
@@ -510,14 +543,12 @@ void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool as
         if ( (thisFacetsDefined & DatatypeValidator::FACET_FRACTIONDIGITS) != 0 )
         {
             if ( theData->getScale() > fFractionDigits )
-            {
-                XMLCh* value = theData->toString();
-                ArrayJanitor<XMLCh> jan(value);
+            {                
                 XMLString::binToText(theData->getScale(), value1, BUF_LEN, 10);
                 XMLString::binToText(fFractionDigits, value2, BUF_LEN, 10);
                 ThrowXML3(InvalidDatatypeFacetException
                                  , XMLExcepts::VALUE_exceed_fractDigit
-                                 , value
+                                 , theData->getRawData()
                                  , value1
                                  , value2);
             }
@@ -526,17 +557,34 @@ void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool as
         if ( (thisFacetsDefined & DatatypeValidator::FACET_TOTALDIGITS) != 0 )
         {
             if ( theData->getTotalDigit() > fTotalDigits )
-            {
-                XMLCh* value = theData->toString();
-                ArrayJanitor<XMLCh> jan(value);
+            {                
                 XMLString::binToText(theData->getTotalDigit(), value1, BUF_LEN, 10);
                 XMLString::binToText(fTotalDigits, value2, BUF_LEN, 10);
                 ThrowXML3(InvalidDatatypeFacetException
                                  , XMLExcepts::VALUE_exceed_totalDigit
-                                 , value
+                                 , theData->getRawData()
                                  , value1
                                  , value2);
             }
+
+            /***
+             E2-44 totalDigits
+
+             ... by restricting it to numbers that are expressible as i  10^-n
+             where i and n are integers such that |i| < 10^totalDigits and 0 <= n <= totalDigits.
+            ***/
+
+            if ( theData->getScale() > fTotalDigits )  
+            {                
+                XMLString::binToText(theData->getScale(), value1, BUF_LEN, 10);
+                XMLString::binToText(fTotalDigits, value2, BUF_LEN, 10);
+                ThrowXML3(InvalidDatatypeFacetException
+                                 , XMLExcepts::VALUE_exceed_totalDigit
+                                 , theData->getRawData()
+                                 , value1
+                                 , value2);
+            }
+        
         }
 
     }
@@ -547,6 +595,8 @@ void DecimalDatatypeValidator::checkContent( const XMLCh* const content, bool as
 
 }
 
+XERCES_CPP_NAMESPACE_END
+
 /**
   * End of file DecimalDatatypeValidator::cpp
   */
