@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: InitTermTest.cpp,v 1.5 2002/05/21 18:54:16 tng Exp $
+ * $Id: InitTermTest.cpp,v 1.6 2003/02/05 18:55:21 tng Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -247,9 +247,9 @@ int TestInit4SAX2(const char* xmlFile, bool gDoNamespaces, bool gDoSchema, bool
     XMLCh* fullSchemaCheckFeature = XMLString::transcode("http://apache.org/xml/features/validation/schema-full-checking");
     parser->setFeature(fullSchemaCheckFeature, gSchemaFullChecking);
 
-    delete [] doNamespaceFeature;
-    delete [] doSchemaFeature;
-    delete [] fullSchemaCheckFeature;
+    XMLString::release(&doNamespaceFeature);
+    XMLString::release(&doSchemaFeature);
+    XMLString::release(&fullSchemaCheckFeature);
 
     TESTINITPOST;
 }
