@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: ThreadTest.cpp,v 1.22 2002/08/26 14:47:49 tng Exp $
+ * $Id: ThreadTest.cpp,v 1.30 2003/03/13 22:13:28 tng Exp $
  *
  * @author Andy Heninger, IBM
  */
@@ -71,6 +71,9 @@
 
 #include <xercesc/dom/DOM.hpp>
 
+#include <xercesc/framework/StdOutFormatTarget.hpp>
+
+
 void clearFileInfoMemory();
 
 //------------------------------------------------------------------------------
@@ -119,7 +122,7 @@ void ThreadFuncs::startThread(ThreadFunc func, void *param)
 }
 
 
-#elif defined (AIX) || defined(SOLARIS) || defined(LINUX) || defined(HPUX) || defined (OS390) || defined(FREEBSD) || defined(__CYGWIN__)
+#elif defined (AIX) || defined(SOLARIS) || defined(LINUX) || defined(HPUX) || defined (OS390) || defined(FREEBSD) || defined(__CYGWIN__) || defined(__QNXNTO__)
 #include <pthread.h>
 #include <unistd.h>
 #include <errno.h>
@@ -271,6 +274,7 @@ RunInfo         gRunInfo;
 ThreadInfo      *gThreadInfo;
 
 
+XERCES_CPP_NAMESPACE_USE
 
 //------------------------------------------------------------------------------
 //
@@ -288,7 +292,7 @@ private:
     int           fCheckSum;
     SAXParser*    fSAXParser;
     XercesDOMParser*    fXercesDOMParser;
-    DOMDocument * fDoc;
+    XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * fDoc;
 
 
 public:                               //  This is the API used by the rest of the test program
@@ -304,7 +308,6 @@ public:                               //  This is the API used by the rest of th
                                       //  for DOM, re-walk the tree.
                                       //  for SAX, can't do, just return previous value.
 
-    void domPrint(const DOMNode *node); // Dump out the contents of a node,
     void domPrint();                  //   including any children.  Default (no param)
                                        //   version dumps the entire document.
 
@@ -410,7 +413,7 @@ int ThreadParser::parse(int fileNum)
                 fXercesDOMParser->parse(*mbis);
             else
                 fXercesDOMParser->parse(fInfo->fileName);
-            DOMDocument* doc = fXercesDOMParser->getDocument();
+            XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument* doc = fXercesDOMParser->getDocument();
             domCheckSum(doc);
         }
         else
@@ -428,7 +431,7 @@ int ThreadParser::parse(int fileNum)
         char *exceptionMessage = XMLString::transcode(e.getMessage());
         fprintf(stderr, " during parsing: %s \n Exception message is: %s \n",
             fInfo->fileName, exceptionMessage);
-        delete [] exceptionMessage;
+        XMLString::release(&exceptionMessage);
         errors = true;
     }
     catch (const DOMException& toCatch)
@@ -442,7 +445,7 @@ int ThreadParser::parse(int fileNum)
         char *exceptionMessage = XMLString::transcode(e.getMessage());
         fprintf(stderr, " during parsing: %s \n Exception message is: %s \n",
             fInfo->fileName, exceptionMessage);
-        delete [] exceptionMessage;
+        XMLString::release(&exceptionMessage);
         errors = true;
     }
     catch (...)
@@ -594,7 +597,23 @@ void ThreadParser::domPrint()
 {
     printf("Begin DOMPrint ...\n");
     if (gRunInfo.dom)
-        domPrint(fXercesDOMParser->getDocument());
+    {
+        try
+        {
+            XMLCh tempStr[100];
+            XMLString::transcode("LS", tempStr, 99);
+            DOMImplementation *impl          = DOMImplementationRegistry::getDOMImplementation(tempStr);
+            DOMWriter         *theSerializer = ((DOMImplementationLS*)impl)->createDOMWriter();
+            XMLFormatTarget   *myFormTarget  = new StdOutFormatTarget();
+            DOMNode           *doc           = fXercesDOMParser->getDocument();
+            theSerializer->writeNode(myFormTarget, *doc);
+            delete theSerializer;
+        }
+        catch (...)
+        {
+            // do nothing
+        }
+    }
     printf("End DOMPrint\n");
 }
 
@@ -605,70 +624,6 @@ static void printString(const XMLCh *str)
     delete s;
 }
 
-
-void ThreadParser::domPrint(const DOMNode *node)
-{
-
-    DOMNode          *child;
-    DOMNamedNodeMap  *attributes;
-
-    switch (node->getNodeType() )
-    {
-    case DOMNode::ELEMENT_NODE:
-        {
-            printf("<");
-            printString(node->getNodeName());   // the element name
-
-            attributes = node->getAttributes();  // Element's attributes
-            int numAttributes = attributes->getLength();
-            int i;
-            for (i=0; i<numAttributes; i++) {
-                domPrint(attributes->item(i));
-            }
-            printf(">");
-
-            for (child=node->getFirstChild(); child!=0; child=child->getNextSibling())
-                domPrint(child);
-
-            printf("</");
-            printString(node->getNodeName());
-            printf(">");
-            break;
-        }
-
-
-    case DOMNode::ATTRIBUTE_NODE:
-        {
-            printf(" ");
-            printString(node->getNodeName());   // The attribute name
-            printf("= \"");
-            printString(node->getNodeValue());  // The attribute value
-            printf("\"");
-            break;
-        }
-
-
-    case DOMNode::TEXT_NODE:
-    case DOMNode::CDATA_SECTION_NODE:
-        {
-            printString(node->getNodeValue());
-            break;
-        }
-
-    case DOMNode::ENTITY_REFERENCE_NODE:
-    case DOMNode::DOCUMENT_NODE:
-        {
-            // For entity references and the document, nothing is dirctly
-            //  printed, but we do want to process the chidren nodes.
-            //
-            for (child=node->getFirstChild(); child!=0; child=child->getNextSibling())
-                domPrint(child);
-            break;
-        }
-    }
-}
-
-
 //----------------------------------------------------------------------
 //
 //   parseCommandLine   Read through the command line, and save all
@@ -726,9 +681,6 @@ void parseCommandLine(int argc, char **argv)
                 else
                     throw 1;
             }
-            else if (strcmp(argv[argnum], "-dom") == 0) {
-                gRunInfo.dom = true;
-            }
             else if (strcmp(argv[argnum], "-reuse") == 0)
                 gRunInfo.reuseParser = true;
             else if (strcmp(argv[argnum], "-dump") == 0)
@@ -856,7 +808,7 @@ void clearFileInfoMemory()
         for (fileNum = 0; fileNum <gRunInfo.numInputFiles; fileNum++)
         {
             InFileInfo *fInfo = &gRunInfo.files[fileNum];
-            delete [] fInfo->uFileName;
+            XMLString::release(&fInfo->uFileName);
             delete [] fInfo->fileContent;
         }
     }
@@ -1026,11 +978,6 @@ int main (int argc, char **argv)
 
     delete mainParser;
 
-    if (errors) {
-        clearFileInfoMemory();
-        exit(1);
-    }
-
     //
     //  Fire off the requested number of parallel threads
     //
